@@ -39,32 +39,34 @@
 #include "CompilerDriver_pix.h"
 #include "PolicyCompiler_pix.h"
 #include "NATCompiler_pix.h"
+#include "NATCompiler_asa8.h"
 #include "RoutingCompiler_pix.h"
 #include "OSConfigurator_pix_os.h"
+#include "NamedObjectsAndGroupsSupport.h"
+#include "NamedObjectsManagerPIX.h"
+#include "NamedObjectsManagerASA8.h"
+#include "AutomaticRules_cisco.h"
 
 #include "Helper.h"
 
-#include "fwbuilder/Resources.h"
-#include "fwbuilder/FWObjectDatabase.h"
-#include "fwbuilder/XMLTools.h"
-#include "fwbuilder/FWException.h"
-#include "fwbuilder/Firewall.h"
-#include "fwbuilder/Interface.h"
-#include "fwbuilder/Policy.h"
-#include "fwbuilder/NAT.h"
-#include "fwbuilder/Routing.h"
-#include "fwbuilder/IPv4.h"
-#include "fwbuilder/IPv6.h"
-
-#include "fwcompiler/Preprocessor.h"
-
 #include "fwbuilder/Cluster.h"
 #include "fwbuilder/ClusterGroup.h"
-#include "fwbuilder/Firewall.h"
-#include "fwbuilder/Interface.h"
-#include "fwbuilder/Policy.h"
-#include "fwbuilder/StateSyncClusterGroup.h"
+#include "fwbuilder/FWException.h"
+#include "fwbuilder/FWObjectDatabase.h"
 #include "fwbuilder/FailoverClusterGroup.h"
+#include "fwbuilder/Firewall.h"
+#include "fwbuilder/IPv4.h"
+#include "fwbuilder/IPv6.h"
+#include "fwbuilder/Interface.h"
+#include "fwbuilder/Library.h"
+#include "fwbuilder/NAT.h"
+#include "fwbuilder/Policy.h"
+#include "fwbuilder/Resources.h"
+#include "fwbuilder/Routing.h"
+#include "fwbuilder/StateSyncClusterGroup.h"
+#include "fwbuilder/XMLTools.h"
+
+#include "fwcompiler/Preprocessor.h"
 
 #include <QStringList>
 #include <QFileInfo>
@@ -98,12 +100,14 @@ class sort_by_net_zone {
     }
 };
 
-QString CompilerDriver_pix::assembleManifest(Cluster *cluster, Firewall* fw, bool cluster_member)
+QString CompilerDriver_pix::assembleManifest(Cluster*, Firewall*, bool)
 {
     QString script_buffer;
     QTextStream script(&script_buffer, QIODevice::WriteOnly);
-    QString ofname = determineOutputFileName(cluster, fw, cluster_member, ".fw");
-    script << "!" << MANIFEST_MARKER << "* " << this->escapeFileName(ofname) << endl;
+
+    script << "!" << MANIFEST_MARKER
+           << "* " << this->escapeFileName(file_names[FW_FILE]) << endl;
+
     return script_buffer;
 }
 
@@ -117,8 +121,8 @@ QString CompilerDriver_pix::assembleFwScript(Cluster *cluster,
                                              bool cluster_member,
                                              OSConfigurator *oscnf)
 {
-    Configlet script_skeleton(fw, "cisco", "script_skeleton");
-    Configlet top_comment(fw, "cisco", "top_comment");
+    Configlet script_skeleton(fw, "pix_os", "script_skeleton");
+    Configlet top_comment(fw, "pix_os", "top_comment");
 
     FWOptions* options = fw->getOptionsObject();
     options->setStr("prolog_script", options->getStr("pix_prolog_script"));
@@ -127,24 +131,50 @@ QString CompilerDriver_pix::assembleFwScript(Cluster *cluster,
 
     string vers = fw->getStr("version");
     string platform = fw->getStr("platform");
-    bool outbound_acl_supported = Resources::platform_res[platform]->getResourceBool(
-        string("/FWBuilderResources/Target/options/")+
-        "version_"+vers+
-        "/pix_outbound_acl_supported");
+
+    bool outbound_acl_supported =
+        Resources::platform_res[platform]->getResourceBool(
+            string("/FWBuilderResources/Target/options/")+
+            "version_"+vers+
+            "/pix_outbound_acl_supported");
+
     bool afpa = options->getBool("pix_assume_fw_part_of_any");
     bool emulate_outb_acls = options->getBool("pix_emulate_out_acl");
     bool generate_outb_acls = options->getBool("pix_generate_out_acl");
 
-    top_comment.setVariable("outbound_acl_supported", QString((outbound_acl_supported)?"supported":"not supported"));
-    top_comment.setVariable("emulate_outb_acls", QString((emulate_outb_acls)?"yes":"no"));
-    top_comment.setVariable("generate_outb_acls", QString((generate_outb_acls)?"yes":"no"));
+    top_comment.setVariable(
+        "outbound_acl_supported",
+        QString((outbound_acl_supported) ? "supported" : "not supported"));
+
+    top_comment.setVariable("emulate_outb_acls",
+                            QString((emulate_outb_acls)?"yes":"no"));
+
+    top_comment.setVariable("generate_outb_acls",
+                            QString((generate_outb_acls)?"yes":"no"));
+
     top_comment.setVariable("afpa", QString((afpa)?"yes":"no"));
 
     script_skeleton.setVariable("short_script", options->getBool("short_script"));
-    script_skeleton.setVariable("not_short_script", ! options->getBool("short_script"));
+
+    script_skeleton.setVariable("not_short_script",
+                                ! options->getBool("short_script"));
+
+    script_skeleton.setVariable("preamble_commands", 
+                                QString::fromUtf8(
+                                    preamble_commands.c_str()));
+
+    script_skeleton.setVariable("clear_commands", 
+                                QString::fromUtf8(
+                                    clear_commands.c_str()));
 
     script_skeleton.setVariable("system_configuration_script", 
-                                QString::fromUtf8(system_configuration_script.c_str()));
+                                QString::fromUtf8(
+                                    system_configuration_script.c_str()));
+
+    script_skeleton.setVariable("named_objects_and_object_groups",
+                                QString::fromUtf8(
+                                    named_objects_and_groups.c_str()));
+
     script_skeleton.setVariable("policy_script",
                                 QString::fromUtf8(policy_script.c_str()));
     script_skeleton.setVariable("nat_script",
@@ -152,7 +182,9 @@ QString CompilerDriver_pix::assembleFwScript(Cluster *cluster,
     script_skeleton.setVariable("routing_script",
                                 QString::fromUtf8(routing_script.c_str()));
 
-    assembleFwScriptInternal(cluster, fw, cluster_member, oscnf, &script_skeleton, &top_comment, "!");
+    assembleFwScriptInternal(cluster, fw, cluster_member, oscnf,
+                             &script_skeleton, &top_comment, "!", true);
+
     return script_skeleton.expand();
 }
 
@@ -161,11 +193,9 @@ QString CompilerDriver_pix::run(const std::string &cluster_id,
                                 const std::string &single_rule_id)
 {
     Cluster *cluster = NULL;
-    if (!cluster_id.empty())
-        cluster = Cluster::cast(objdb->findInIndex(objdb->getIntId(cluster_id)));
+    Firewall *fw = NULL;
 
-    Firewall *fw = Firewall::cast(objdb->findInIndex(objdb->getIntId(firewall_id)));
-    assert(fw);
+    getFirewallAndClusterObjects(cluster_id, firewall_id, &cluster, &fw);
 
     // Copy rules from the cluster object
     populateClusterElements(cluster, fw);
@@ -228,13 +258,23 @@ QString CompilerDriver_pix::run(const std::string &cluster_id,
 #endif
 
 
-    QString ofname = determineOutputFileName(cluster, fw, !cluster_id.empty(), ".fw");
+    determineOutputFileNames(cluster, fw, !cluster_id.empty(),
+                             QStringList(""), QStringList("fw"),
+                             QStringList(""));
+
     FWOptions* options = fw->getOptionsObject();
 
     QString script_buffer;
 
+    std::auto_ptr<NATCompiler_pix> n;
+    std::auto_ptr<PolicyCompiler_pix> c;
+    std::auto_ptr<RoutingCompiler_pix> r;
+
+
     try
     {
+        clearReadOnly(fw);
+
         commonChecks2(cluster, fw);
 
         pixClusterConfigurationChecks(cluster, fw);
@@ -280,9 +320,12 @@ QString CompilerDriver_pix::run(const std::string &cluster_id,
             copies_of_cluster_interfaces.pop_front();
         }
 
+        NamedObjectsManagerPIX named_objects_manager(persistent_objects, fw);
+
         all_interfaces = fw->getByTypeDeep(Interface::TYPENAME);
 
-        for (std::list<FWObject*>::iterator i=all_interfaces.begin(); i!=all_interfaces.end(); ++i)
+        for (std::list<FWObject*>::iterator i=all_interfaces.begin();
+             i!=all_interfaces.end(); ++i)
         {
             Interface *iface = Interface::cast(*i);
             assert(iface);
@@ -337,6 +380,16 @@ QString CompilerDriver_pix::run(const std::string &cluster_id,
          std::sort(fw->begin(), fw->end(), sort_by_net_zone() );
         */
 
+
+        try
+        {
+            AutomaticRules_cisco auto_rules(fw, persistent_objects);
+            auto_rules.addSshAccessRule();
+        } catch (FWException &ex)
+        {
+            abort(ex.toString());
+        }
+
         std::auto_ptr<Preprocessor> prep(
             new Preprocessor(objdb , fw, false));
         if (inTestMode()) prep->setTestMode();
@@ -351,17 +404,28 @@ QString CompilerDriver_pix::run(const std::string &cluster_id,
         oscnf->prolog();
         oscnf->processFirewallOptions();
 
+        bool have_named_objects = false;
+        bool have_object_groups = false;
 
 /* create compilers and run the whole thing */
+        string version = fw->getStr("version");
 
-        std::auto_ptr<NATCompiler_pix> n(
-            new NATCompiler_pix(objdb, fw, false, oscnf.get()));
+        if (XMLTools::version_compare(version, "8.3")>=0)
+            n = std::auto_ptr<NATCompiler_pix>(
+                new NATCompiler_asa8(objdb, fw, false, oscnf.get()));
+        else
+            n = std::auto_ptr<NATCompiler_pix>(
+                new NATCompiler_pix(objdb, fw, false, oscnf.get()));
 
         RuleSet *nat = RuleSet::cast(fw->getFirstByType(NAT::TYPENAME));
         if (nat)
         {
+            nat->assignUniqueRuleIds();
+
+            n->setNamedObjectsManager(&named_objects_manager);
             n->setSourceRuleSet(nat);
             n->setRuleSetName(nat->getName());
+            n->setPersistentObjects(persistent_objects);
 
             if (inTestMode()) n->setTestMode();
             if (inEmbeddedMode()) n->setEmbeddedMode();
@@ -374,18 +438,30 @@ QString CompilerDriver_pix::run(const std::string &cluster_id,
             {
                 n->compile();
                 n->epilog();
+
+                preamble_commands += n->printPreambleCommands();
+                clear_commands += n->printClearCommands();
+                have_named_objects = (have_named_objects ||
+                                      named_objects_manager.haveNamedObjects());
+                have_object_groups = (have_object_groups ||
+                                      named_objects_manager.haveObjectGroups());
+                //named_objects_manager.saveObjectGroups();
             } else
                 info(" Nothing to compile in NAT");
         }
 
-        std::auto_ptr<PolicyCompiler_pix> c(
+        c = std::auto_ptr<PolicyCompiler_pix>(
             new PolicyCompiler_pix(objdb, fw, false, oscnf.get() , n.get()));
 
         RuleSet *policy = RuleSet::cast(fw->getFirstByType(Policy::TYPENAME));
         if (policy)
         {
+            policy->assignUniqueRuleIds();
+
+            c->setNamedObjectsManager(&named_objects_manager);
             c->setSourceRuleSet(policy);
             c->setRuleSetName(policy->getName());
+            c->setPersistentObjects(persistent_objects);
 
             if (inTestMode()) c->setTestMode();
             if (inEmbeddedMode()) c->setEmbeddedMode();
@@ -398,19 +474,31 @@ QString CompilerDriver_pix::run(const std::string &cluster_id,
             {
                 c->compile();
                 c->epilog();
+
+                preamble_commands += c->printPreambleCommands();
+                clear_commands += c->printClearCommands();
+                have_named_objects = (have_named_objects ||
+                                      named_objects_manager.haveNamedObjects());
+                have_object_groups = (have_object_groups ||
+                                      named_objects_manager.haveObjectGroups());
+                //named_objects_manager.saveObjectGroups();
             } else
                 info(" Nothing to compile in Policy");
         }
 
-        std::auto_ptr<RoutingCompiler_pix> r(
+        r = std::auto_ptr<RoutingCompiler_pix>(
             new RoutingCompiler_pix(objdb, fw, false, oscnf.get()));
 
         RuleSet *routing = RuleSet::cast(fw->getFirstByType(Routing::TYPENAME));
         if (routing)
         {
+            routing->assignUniqueRuleIds();
+
+            r->setNamedObjectsManager(&named_objects_manager);
             r->setSourceRuleSet(routing);
             r->setRuleSetName(routing->getName());
-
+            r->setPersistentObjects(persistent_objects);
+                
             if (inTestMode()) r->setTestMode();
             if (inEmbeddedMode()) r->setEmbeddedMode();
             r->setSingleRuleCompileMode(single_rule_id);
@@ -426,16 +514,24 @@ QString CompilerDriver_pix::run(const std::string &cluster_id,
                 info(" Nothing to compile in Routing");
         }
 
+        /*
+         * compilers detach persistent objects when they finish, this
+         * means at this point library persistent_objects is not part
+         * of any object tree.
+         */
+        objdb->reparent(persistent_objects);
+
         if (haveErrorsAndWarnings())
         {
             all_errors.push_front(getErrors("").c_str());
         }
 
-        system_configuration_script = oscnf->getCompiledScript();
         policy_script = c->getCompiledScript();
         nat_script = n->getCompiledScript();
         routing_script = r->getCompiledScript();
 
+        named_objects_and_groups = named_objects_manager.getNamedObjectsDefinitions();
+
         if (c->haveErrorsAndWarnings())
             all_errors.push_back(c->getErrors("C ").c_str());
         if (n->haveErrorsAndWarnings())
@@ -443,17 +539,27 @@ QString CompilerDriver_pix::run(const std::string &cluster_id,
         if (r->haveErrorsAndWarnings())
             all_errors.push_back(r->getErrors("R ").c_str());
 
-        script_buffer = assembleFwScript(
-            cluster, fw, !cluster_id.empty(), oscnf.get());
-
         if (single_rule_compile_on)
         {
             return formSingleRuleCompileOutput(
                 QString::fromUtf8(
-                    (policy_script + nat_script + routing_script).c_str()));
+                    (named_objects_and_groups +
+                     policy_script + nat_script + routing_script).c_str()));
         }
 
-        ofname = getAbsOutputFileName(ofname);
+        system_configuration_script = oscnf->getCompiledScript();
+        system_configuration_script += "\n";
+
+        clear_commands += named_objects_manager.getClearCommands() + "\n";
+
+        // system_configuration_script += preamble_commands;
+        // system_configuration_script += clear_commands;
+
+
+        script_buffer = assembleFwScript(
+            cluster, fw, !cluster_id.empty(), oscnf.get());
+
+        QString ofname = getAbsOutputFileName(file_names[FW_FILE]);
 
         info("Output file name: " + ofname.toStdString());
         QFile fw_file(ofname);
@@ -472,12 +578,14 @@ QString CompilerDriver_pix::run(const std::string &cluster_id,
         } else
         {
             QString err(" Failed to open file %1 for writing: %2; Current dir: %3");
-            abort(err.arg(fw_file.fileName()).arg(fw_file.error()).arg(QDir::current().path()).toStdString());
+            abort(err.arg(fw_file.fileName())
+                  .arg(fw_file.error()).arg(QDir::current().path()).toStdString());
         }
     }
-    catch (FatalErrorInSingleRuleCompileMode &ex)
+    catch (FWException &ex)
     {
-        return QString::fromUtf8(getErrors("").c_str());
+        status = BaseCompiler::FWCOMPILER_ERROR;
+        return QString::fromUtf8(ex.toString().c_str());
     }
 
     return "";
@@ -537,20 +645,21 @@ void CompilerDriver_pix::pixSecurityLevelChecks(Firewall *fw,
                 iface2->getOptionsObject()->getBool("cluster_interface"))
                 continue;
 
-            if (iface->getSecurityLevel()==iface2->getSecurityLevel())
-            {
-                QString err(
-                    "Security level of each interface should be unique, "
-                    "however interfaces %1 (%2) and %3 (%4)"
-                    " have the same security level."
-                );
-                abort(fw, NULL, NULL,
-                      err.arg(iface->getName().c_str())
-                      .arg(iface->getLabel().c_str())
-                      .arg(iface2->getName().c_str())
-                      .arg(iface2->getLabel().c_str()).toStdString());
-                throw FatalErrorInSingleRuleCompileMode();
-            }
+            // see #2351. Security levels do not have to be unique
+            // if (iface->getSecurityLevel()==iface2->getSecurityLevel())
+            // {
+            //     QString err(
+            //         "Security level of each interface should be unique, "
+            //         "however interfaces %1 (%2) and %3 (%4)"
+            //         " have the same security level."
+            //     );
+            //     abort(fw, NULL, NULL,
+            //           err.arg(iface->getName().c_str())
+            //           .arg(iface->getLabel().c_str())
+            //           .arg(iface2->getName().c_str())
+            //           .arg(iface2->getLabel().c_str()).toStdString());
+            //     throw FatalErrorInSingleRuleCompileMode();
+            // }
 
             if (iface->getLabel()==iface2->getLabel())
             {
@@ -662,6 +771,21 @@ void CompilerDriver_pix::pixNetworkZoneChecks(Firewall *fw,
                 throw FatalErrorInSingleRuleCompileMode();
             }
 
+/*
+  Commented out for SF bug 3213019
+
+  currently we do not support ipv6 with PIX/ASA and FWSM. If user
+  creates a group to be used as network zone object and places ipv6
+  address in it, this address should be ignored while compiling the
+  policy but this should not be an error. Compiler uses network zone
+  group to do various address matching operations when it tries to
+  determine an interface for a rule where user did not specify
+  one. Since we never (should) have ipv6 in policy and nat rules,
+  compiler is not going to have anything to compare to ipv6 address in
+  the network zone even if there is one and this ipv6 address is going
+  to be ignored.
+
+
             if (addr->getAddressPtr()->isV6())
             {
                 QString err("Network zone of interface '%1' uses object '%2' "
@@ -671,7 +795,7 @@ void CompilerDriver_pix::pixNetworkZoneChecks(Firewall *fw,
                       .arg((*j)->getName().c_str()).toStdString());
                 throw FatalErrorInSingleRuleCompileMode();
             }
-
+*/
             netzone_objects.insert(
                 pair<string,FWObject*>(iface->getLabel(),*j));
             nz->addRef(*j);
