@@ -25,6 +25,8 @@
 
 #include "PolicyCompiler_iosacl.h"
 #include "IOSObjectGroup.h"
+#include "NamedObjectsAndGroupsSupport.h"
+#include "PortRangeConverter.h"
 
 #include "fwbuilder/Firewall.h"
 #include "fwbuilder/AddressRange.h"
@@ -111,57 +113,6 @@ bool PolicyCompiler_iosacl::ClearACLs::processNext()
     return true;
 }
 
-bool PolicyCompiler_iosacl::printClearCommands::processNext()
-{
-    PolicyCompiler_iosacl *iosacl_comp=dynamic_cast<PolicyCompiler_iosacl*>(compiler);
-
-    slurp();
-    if (tmp_queue.size()==0) return false;
-
-    iosacl_comp->_printClearCommands();
-
-    return true;
-}
-
-void PolicyCompiler_iosacl::_printClearCommands()
-{
-    string vers = fw->getStr("version");
-    string platform = fw->getStr("platform");
-
-    string xml_element = "clear_ip_acl";
-    if (ipv6) xml_element = "clear_ipv6_acl";
-
-    string clearACLCmd = Resources::platform_res[platform]->getResourceStr(
-        string("/FWBuilderResources/Target/options/")+
-        "version_"+vers+"/iosacl_commands/" + xml_element);
-
-    assert( !clearACLCmd.empty());
-
-    if (!inSingleRuleCompileMode())
-    {
-        // No need to output "clear" commands in single rule compile mode
-        if ( fw->getOptionsObject()->getBool("iosacl_acl_basic") ||
-             fw->getOptionsObject()->getBool("iosacl_acl_substitution"))
-        {
-            for (map<string,ciscoACL*>::iterator i=acls.begin(); i!=acls.end(); ++i)
-            {
-                ciscoACL *acl = (*i).second;
-                output << clearACLCmd << " " << acl->workName() << endl;
-            }
-            output << endl;
-
-            for (FWObject::iterator i=object_groups->begin(); i!=object_groups->end(); ++i)
-            {
-                BaseObjectGroup *og = dynamic_cast<BaseObjectGroup*>(*i);
-                assert(og!=NULL);
-                output << "no "  << og->getObjectGroupHeader();
-            }
-        }
-    }
-    output << endl;
-}
-        
-
 void PolicyCompiler_iosacl::PrintCompleteACLs::printRulesForACL::operator()(
     Rule* rule)
 {
@@ -216,7 +167,7 @@ string PolicyCompiler_iosacl::PrintRule::_printRule(PolicyRule *rule)
     PolicyCompiler_iosacl *iosacl_comp = 
         dynamic_cast<PolicyCompiler_iosacl*>(compiler);
     string platform = compiler->fw->getStr("platform");
-    FWOptions  *ruleopt =rule->getOptionsObject();
+    //FWOptions  *ruleopt =rule->getOptionsObject();
     bool write_comments = compiler->fw->getOptionsObject()->getBool(
         platform + "_include_comments");
 
@@ -396,30 +347,21 @@ string PolicyCompiler_iosacl::PrintRule::_printLog(PolicyRule *rule)
     return "";
 }
 
+string PolicyCompiler_iosacl::PrintRule::_printPortRangeOp(int rs, int re)
+{
+    return PortRangeConverter(rs, re).toString();
+}
+
 string PolicyCompiler_iosacl::PrintRule::_printSrcService(Service *srv)
 {
-    ostringstream  str;
-
     if (TCPService::isA(srv) || UDPService::isA(srv)) 
     {
-	int rs=TCPUDPService::cast(srv)->getSrcRangeStart();
-	int re=TCPUDPService::cast(srv)->getSrcRangeEnd();
-
-        if (rs<0) rs=0;
-        if (re<0) re=0;
-
-	if (rs>0 || re>0) {
-	    if (rs==re)  str << "eq " << rs << " ";
-	    else
-		if (rs==0 && re!=0)      str << "lt " << re << " ";
-		else
-		    if (rs!=0 && re==65535)  str << "gt " << rs << " ";
-		    else
-			str << "range " << rs << " " << re << " ";
-	}
+	int rs = TCPUDPService::cast(srv)->getSrcRangeStart();
+	int re = TCPUDPService::cast(srv)->getSrcRangeEnd();
+        return _printPortRangeOp(rs, re);
     }
 
-    return str.str();
+    return "";
 }
 
 string PolicyCompiler_iosacl::PrintRule::_printIPServiceOptions(PolicyRule *r)
@@ -457,21 +399,9 @@ string PolicyCompiler_iosacl::PrintRule::_printDstService(Service *srv)
 
     if (TCPService::isA(srv) || UDPService::isA(srv))
     {
-	int rs=TCPUDPService::cast(srv)->getDstRangeStart();
-	int re=TCPUDPService::cast(srv)->getDstRangeEnd();
-
-        if (rs<0) rs=0;
-        if (re<0) re=0;
-
-	if (rs>0 || re>0) {
-	    if (rs==re)  str << "eq " << rs << " ";
-	    else
-		if (rs==0 && re!=0)      str << "lt " << re << " ";
-		else
-		    if (rs!=0 && re==65535)  str << "gt " << rs << " ";
-		    else
-			str << "range " << rs << " " << re << " ";
-	}
+	int rs = TCPUDPService::cast(srv)->getDstRangeStart();
+	int re = TCPUDPService::cast(srv)->getDstRangeEnd();
+        str << _printPortRangeOp(rs, re);
     }
 
     if (TCPService::isA(srv))
@@ -480,8 +410,11 @@ string PolicyCompiler_iosacl::PrintRule::_printDstService(Service *srv)
         else str << _printTCPFlags(TCPService::cast(srv));
     }
 
-    if ((ICMPService::isA(srv) || ICMP6Service::isA(srv)) && srv->getInt("type")!=-1)
-	    str << srv->getStr("type") << " ";
+    if ((ICMPService::isA(srv) || ICMP6Service::isA(srv)) &&
+        srv->getInt("type")!=-1)
+    {
+        str << srv->getStr("type") << " ";
+    }
 
     if (CustomService::isA(srv)) 
 	str << CustomService::cast(srv)->getCodeForPlatform(
