@@ -46,22 +46,23 @@
 #include "interfaceProperties.h"
 #include "interfacePropertiesObjectFactory.h"
 
-#include "fwbuilder/FWObject.h"
-#include "fwbuilder/FWObjectDatabase.h"
-#include "fwbuilder/FWException.h"
 #include "fwbuilder/Cluster.h"
 #include "fwbuilder/ClusterGroup.h"
+#include "fwbuilder/FWException.h"
+#include "fwbuilder/FWObject.h"
+#include "fwbuilder/FWObjectDatabase.h"
+#include "fwbuilder/FailoverClusterGroup.h"
 #include "fwbuilder/Firewall.h"
-#include "fwbuilder/Interface.h"
 #include "fwbuilder/IPv4.h"
 #include "fwbuilder/IPv6.h"
-#include "fwbuilder/Rule.h"
-#include "fwbuilder/Policy.h"
+#include "fwbuilder/Interface.h"
+#include "fwbuilder/Library.h"
 #include "fwbuilder/NAT.h"
-#include "fwbuilder/Routing.h"
+#include "fwbuilder/Policy.h"
 #include "fwbuilder/Resources.h"
+#include "fwbuilder/Routing.h"
+#include "fwbuilder/Rule.h"
 #include "fwbuilder/StateSyncClusterGroup.h"
-#include "fwbuilder/FailoverClusterGroup.h"
 
 #include "fwcompiler/Compiler.h"
 
@@ -80,19 +81,31 @@ CompilerDriver::CompilerDriver(FWObjectDatabase *db) : BaseCompiler()
     filename = "";
     wdir = "";
     fwobjectname = "";
-    fw_file_name = "";
-    dl = 0;
-    drp = -1;
-    rule_debug_on = false;
     single_rule_compile_on = false;
     prepend_cluster_name_to_output_file = false;
+    dl = 0;
+    rule_debug_on = false;
+    drp = -1;
     drn = -1;
+    drr = -1;
     verbose = 0;
     have_dynamic_interfaces = false;
     ipv4_run = true;
     ipv6_run = true;
     fw_by_id = false;
+
     objdb = new FWObjectDatabase(*db);
+
+    //objdb = db;
+
+    persistent_objects = new Library();
+    persistent_objects->setName("Persistent Objects");
+    objdb->add(persistent_objects);
+
+    workspace = new Library();
+    workspace->setName("Workspace");
+    objdb->add(workspace);
+
     prolog_done = false;
     epilog_done = false;
     have_filter = false;
@@ -102,6 +115,28 @@ CompilerDriver::CompilerDriver(FWObjectDatabase *db) : BaseCompiler()
 
 CompilerDriver::~CompilerDriver()
 {
+    if (persistent_objects->getParent() == NULL)
+        delete persistent_objects;
+    else
+    {
+        if (persistent_objects->getParent() == objdb)
+        {
+            objdb->remove(persistent_objects, false);
+            delete persistent_objects;
+        }
+    }
+    
+    if (workspace->getParent() == NULL)
+        delete workspace;
+    else
+    {
+        if (workspace->getParent() == objdb)
+        {
+            objdb->remove(workspace, false);
+            delete workspace;
+        }
+    }
+
     delete objdb;
 }
 
@@ -160,7 +195,7 @@ bool CompilerDriver::configure(const QStringList &args)
         if (arg == "-o")
         {
             idx++;
-            fw_file_name = args.at(idx);
+            file_name_setting_from_command_line = args.at(idx);
             continue;
         }
         if (arg == "-O")
@@ -223,6 +258,16 @@ bool CompilerDriver::configure(const QStringList &args)
             continue;
         }
 
+        if (arg == "-xr")
+        {
+            idx++;
+            bool ok = false;
+            drr = args.at(idx).toInt(&ok);
+            if (!ok) return false;
+            rule_debug_on = true;
+            continue;
+        }
+
         if (arg == "-s")
         {
             idx++;
@@ -253,6 +298,26 @@ void CompilerDriver::chDir()
     }
 }
 
+/*
+ * See #1994. We need to reset read-only flag on the firewall and up
+ * the tree all the way to the root in order to let compilers make any
+ * modifications they need. Note that this resets read-only flags in
+ * the copy of the database this class works with.
+ */
+void CompilerDriver::clearReadOnly(Firewall *fw)
+{
+    if (fw->isReadOnly())
+    {
+        FWObject *p = fw;
+        while (p)
+        {
+            p->setReadOnly(false);
+            p = p->getParent();
+        }
+    }
+}
+          
+
 QString CompilerDriver::getAbsOutputFileName(const QString &output_file_name)
 {
     QFileInfo finfo(output_file_name);
@@ -397,9 +462,8 @@ void CompilerDriver::commonChecks2(Cluster *cluster, Firewall *fw)
 
   if (!iface->isDyn())
   {
-  char errstr[256];
-  sprintf(errstr,
-  _("Wildcard interface '%s' must be dynamic."),
+  QString errstr;
+  errstr.sprintf(_("Wildcard interface '%s' must be dynamic."),
   iface->getName().c_str() );
   throw FWException(errstr);
   }
@@ -524,12 +588,17 @@ void CompilerDriver::commonChecks2(Cluster *cluster, Firewall *fw)
                 interfacePropertiesObjectFactory::getInterfacePropertiesObject(
                     os_family));
 
+#if 0
+            // See #2103. All interface name validation checks should
+            // be done in the GUI.
             QString err;
             if (!int_prop->validateInterface(parent, iface, true, err))
             {
                 abort(fw, NULL, NULL, err.toStdString());
                 throw FatalErrorInSingleRuleCompileMode();
             }
+#endif
+
 
             string interface_type = iface->getOptionsObject()->getStr("type");
             if (interface_type.empty()) interface_type = "ethernet";
@@ -595,48 +664,6 @@ Firewall* CompilerDriver::locateObject()
 
     return obj;
 }
- 
-/**
- *  Determine output file name. If compiling standalone firewall, the
- *  name can be enforced via -o command line switch in which case it
- *  is in fw_file_name already. If not, determine automatically using
- *  firewall name.
- * 
- *  If compiling a cluster, the name could have been enforced via -O
- *  command line switch, in which case it will be found in
- *  member_file_names. If not, determine automatically using member
- *  firewall name.
- * 
- *  Returns determined output file name
- */
-QString CompilerDriver::determineOutputFileName(Cluster *cluster,
-                                                Firewall *current_fw,
-                                                bool cluster_member,
-                                                const QString &ext)
-{
-    QString current_firewall_name = current_fw->getName().c_str();
-    if (!cluster_member)
-    {
-        // standalone firewall
-        if (fw_file_name.isEmpty())
-        {
-            return current_firewall_name + ext;
-        } else
-            return fw_file_name;
-    }
-    
-    // member of a cluster
-    QString fw_id = objdb->getStringId(current_fw->getId()).c_str();
-    if (member_file_names.contains(fw_id))
-        return member_file_names[fw_id];
-    else
-    {
-        if (prepend_cluster_name_to_output_file && cluster!=NULL)
-            return QString("%1_%2%3").arg(cluster->getName().c_str())
-                .arg(current_firewall_name).arg(ext);
-    }
-    return current_firewall_name + ext;
-}
 
 /* Find rulesets that belong to other firewall objects but are
  * referenced by rules of this firewall using action Branch.
@@ -672,6 +699,9 @@ void CompilerDriver::findImportedRuleSets(Firewall *fw,
             RuleSet *branch_ruleset = rule->getBranch();
             if (branch_ruleset!=NULL)
             {
+                // qDebug() << "ruleset=" << ruleset->getName().c_str()
+                //          << "branch=" << branch_ruleset->getName().c_str();
+
                 map<FWObject*, int> referenced_branch_rulesets;
 
                 _findImportedRuleSetsRecursively(
@@ -684,19 +714,28 @@ void CompilerDriver::findImportedRuleSets(Firewall *fw,
                     RuleSet *branch_ruleset = RuleSet::cast(it->first);
                     int counter = it->second;
 
+                    // qDebug() << "    " 
+                    //          << "branch=" << branch_ruleset->getName().c_str()
+                    //          << "counter=" << counter;
+
                     if (counter > 1)
                     {
-                        QString err("Rule set %1 of firewall %2 has branching rule that loops back to it");
+                        QString err(
+                            "Rule branches to rule set %1 which branches "
+                            "back to it, creating a loop");
                         warning(ruleset->getParent(), ruleset, rule,
-                                err.arg(ruleset->getName().c_str())
-                                .arg(fw->getName().c_str()).toStdString());
+                                err.arg(branch_ruleset->getName().c_str())
+                                .toStdString());
                     }
 
                     if (branch_ruleset->isChildOf(fw)) continue;
 
-                    list<FWObject*>::iterator it = std::find(imported_policies.begin(),
-                                                             imported_policies.end(),
-                                                             branch_ruleset);
+                    list<FWObject*>::iterator it =
+                        std::find(
+                            imported_policies.begin(),
+                            imported_policies.end(),
+                            branch_ruleset);
+
                     if (it != imported_policies.end()) continue;
 
                     // Additional check: the rule set may be child of a
@@ -712,6 +751,7 @@ void CompilerDriver::findImportedRuleSets(Firewall *fw,
             }
         }
     }
+
     if (imported_policies.size() > 0)
         all_policies.insert(all_policies.end(),
                             imported_policies.begin(), imported_policies.end());
@@ -720,6 +760,9 @@ void CompilerDriver::findImportedRuleSets(Firewall *fw,
 void CompilerDriver::_findImportedRuleSetsRecursively(
     Firewall *fw, RuleSet *branch_ruleset, map<FWObject*, int> &branch_rulesets)
 {
+    // multiple rules in the rule set may branch to the same branch rule set
+    map<FWObject*, int> local_branch_ruleset_counters;
+
     int c = branch_rulesets[branch_ruleset];
     branch_rulesets[branch_ruleset] = ++c;
     if (c > 1) return;  // we have seen this one already
@@ -730,13 +773,21 @@ void CompilerDriver::_findImportedRuleSetsRecursively(
         if (rule == NULL) continue; // skip RuleSetOptions object
 
         RuleSet *next_branch_ruleset = rule->getBranch();
-        if (next_branch_ruleset!=NULL)
+        if (next_branch_ruleset!=NULL &&
+            local_branch_ruleset_counters.count(next_branch_ruleset)==0)
         {
-            _findImportedRuleSetsRecursively(fw, next_branch_ruleset, branch_rulesets);
+            local_branch_ruleset_counters[next_branch_ruleset] = 1;
+            _findImportedRuleSetsRecursively(
+                fw, next_branch_ruleset, branch_rulesets);
         }
     }
 }
 
+void CompilerDriver::assignUniqueRuleIds(list<FWObject*> &all_rulesets)
+{
+    for_each(all_rulesets.begin(), all_rulesets.end(),
+             RuleSet::UniqueRuleIdsSetter());
+}
 
 QString CompilerDriver::run(const std::string&, const std::string&, const std::string&)
 {
@@ -858,8 +909,10 @@ void CompilerDriver::mergeRuleSets(Cluster *cluster, Firewall *fw,
     {
         FWObject *ruleset = *p;
 
-        FWObject::iterator i = std::find_if(fw->begin(), fw->end(),
-                                            FWObjectNameEQPredicate(ruleset->getName()));
+        FWObject::iterator i = std::find_if(
+            fw->begin(), fw->end(),
+            FWObjectNameEQPredicate(ruleset->getName()));
+
         if (i!=fw->end() && (*i)->getTypeName() == type)
         {
             FWObject *fw_ruleset = *i;
@@ -928,6 +981,20 @@ void CompilerDriver::populateClusterElements(Cluster *cluster, Firewall *fw)
 {
     if (cluster==NULL) return;
 
+#ifdef DEBUG_CLUSTER_INTERFACES
+    cerr << "CompilerDriver::populateClusterElements " << endl;
+
+    cerr << cluster->getPath(false, true) << endl;
+    list<FWObject*> cl_interfaces = cluster->getByTypeDeep(Interface::TYPENAME);
+    cerr << cl_interfaces.size() << " interface" << endl;
+    cluster->dump(false, true);
+
+    cerr << fw->getPath(false, true) << endl;
+    list<FWObject*> fw_interfaces = fw->getByTypeDeep(Interface::TYPENAME);
+    cerr << fw_interfaces.size() << " interface" << endl;
+    fw->dump(false, true);
+#endif
+
 //    int addedPolicies = 0;
     set<string> state_sync_types;
 
@@ -1188,30 +1255,51 @@ QString CompilerDriver::formSingleRuleCompileOutput(const QString &generated_cod
     return res;
 }
 
-/*
- * Replace ' ' with '\ ' in the string. 
- *
- * DO NOT CHANGE WITHOUT EXTENSIVE TESTING OF THE POLICY INSTALLER !
- *
- * This is used in different places to deal with generated files with
- * spaces in the name. This can happen if firewall object name has
- * spaces. This method of escaping the space is used when we generate
- * manifest line in the .fw file to find other generated files. The
- * same method is also used to generate command line arguments for scp
- * where spaces cause all sorts of problems, for example scp can't
- * find the file to copy or issues "ambiguous target" error when
- * remote file name has a space. Finally, the same method is used to
- * escape space in the file name before using it for the configlet
- * variable because this name is used in the shell script that we run
- * on the firewall.
- */
-QString CompilerDriver::escapeFileName(QString fileName)
+void CompilerDriver::getFirewallAndClusterObjects(const string &cluster_id,
+                                                  const string &firewall_id,
+                                                  Cluster **cl,
+                                                  Firewall **fw)
 {
-    return fileName.replace(' ', "\\ ");
-}
+    if (!cluster_id.empty())
+    {
+        Cluster *orig_cluster = Cluster::cast(
+            objdb->findInIndex(objdb->getIntId(cluster_id)));
+
+#ifdef WORK_ON_COPIES
+        *cl = objdb->createCluster();
+        workspace->add(*cl);
+        (*cl)->duplicate(orig_cluster);
+#else
+
+        *cl = orig_cluster;
+
+#endif
+
+    }
+
+    Firewall *orig_fw = Firewall::cast(
+        objdb->findInIndex(objdb->getIntId(firewall_id)));
+    assert(orig_fw);
+
+#ifdef WORK_ON_COPIES
+
+    *fw = objdb->createFirewall();
+    workspace->add(*fw);
+    (*fw)->duplicate(orig_fw);
+
+    if (*cl != NULL)
+    {
+        const map<int, int> &id_map = (*fw)->getIDMappingTable();
+        map<int, int>::const_iterator it;
+        for (it=id_map.begin(); it!=id_map.end(); ++it)
+            (*cl)->replaceRef(it->first, it->second);
+    }
+#else
+
+    *fw = orig_fw;
+
+#endif
 
-QString CompilerDriver::unescapeFileName(QString fileName)
-{
-    return fileName.replace("\\ ", " ");
 }
 
+        
