@@ -70,6 +70,8 @@
 #include "fwbuilder/XMLTools.h"
 #include "fwbuilder/Resources.h"
 #include "fwbuilder/FWException.h"
+#include "fwbuilder/Constants.h"
+
 
 #if defined(Q_WS_MAC)
 extern void connectOdocHandler();
@@ -186,7 +188,7 @@ int main( int argc, char *argv[] )
 
     INIT2;
 
-    string full_res_path = respath + FS_SEPARATOR + "resources.xml";
+    string full_res_path = Constants::getResourcesFilePath();
 
     if (fwbdebug)
     {
@@ -200,7 +202,6 @@ int main( int argc, char *argv[] )
         exit(1);
     }
 
-    //respath = RES_DIR;
     Resources res(full_res_path);
 
     if (fwbdebug) qDebug("done");
@@ -233,7 +234,7 @@ int main( int argc, char *argv[] )
     QString local = QLocale::system().name();//"en_US";//
     QTranslator translator(0);
     translator.load(QLatin1String("fwbuilder_") +
-                    QString(local), localepath.c_str());
+                    QString(local), Constants::getLocaleDirectory().c_str());
     app->installTranslator (&translator);
 
     QString qt_resource_dir =
@@ -264,7 +265,13 @@ int main( int argc, char *argv[] )
     // setup single shot timer to call startupLoad()
     QTimer::singleShot(500, mw, SLOT(startupLoad()));
 
-    app->exec();
+    try
+    {
+        app->exec();
+    } catch (const libfwbuilder::FWException &ex)
+    {
+        qDebug() << "Caught FWException: " << ex.toString().c_str();
+    }
 
     mw->hide();  // must do this before settings object is destroyed
 
