@@ -47,6 +47,8 @@
 #include "OSConfigurator_linux24.h"
 #include "OSConfigurator_secuwall.h"
 #include "OSConfigurator_ipcop.h"
+#include "combinedAddress.h"
+#include "AutomaticRules_ipt.h"
 
 #include "Configlet.h"
 
@@ -66,6 +68,7 @@
 #include "fwbuilder/Resources.h"
 #include "fwbuilder/StateSyncClusterGroup.h"
 #include "fwbuilder/FailoverClusterGroup.h"
+#include "fwbuilder/Library.h"
 
 #include <QString>
 #include <QStringList>
@@ -74,15 +77,23 @@
 #include <QDir>
 #include <QTextStream>
 #include <QtDebug>
+#include <QTime>
 
 
 using namespace std;
 using namespace libfwbuilder;
 using namespace fwcompiler;
 
-extern QString build_num;
 extern QString user_name;
 
+FWObject* create_combinedAddress(int id)
+{
+    FWObject *nobj = new combinedAddress();
+    if (id > -1) nobj->setId(id);
+    return nobj;
+}
+
+
 /*
  * Go through paces to compile firewall which may be a member of a
  * cluster.  Note that both firewall and cluster are defined by their
@@ -94,19 +105,24 @@ QString CompilerDriver_ipt::run(const std::string &cluster_id,
                                 const std::string &firewall_id,
                                 const std::string &single_rule_id)
 {
-    Cluster *cluster = NULL;
-    if (!cluster_id.empty())
-        cluster = Cluster::cast(
-            objdb->findInIndex(objdb->getIntId(cluster_id)));
 
-    Firewall *fw = Firewall::cast(
-        objdb->findInIndex(objdb->getIntId(firewall_id)));
-    assert(fw);
+    FWObjectDatabase::registerObjectType(combinedAddress::TYPENAME,
+                                         &create_combinedAddress);
+
+    // see #2212 Create temporary copy of the firewall and cluster
+    // objects and pass them to the compilers.
+
+    Cluster *cluster = NULL;
+    Firewall *fw = NULL;
+
+    getFirewallAndClusterObjects(cluster_id, firewall_id, &cluster, &fw);
 
     string generated_script;
 
     try
     {
+        clearReadOnly(fw);
+
         // Copy rules from the cluster object
         populateClusterElements(cluster, fw);
 
@@ -124,8 +140,6 @@ QString CompilerDriver_ipt::run(const std::string &cluster_id,
         // firewall fw This happens when we compile a member of a cluster
         current_firewall_name = fw->getName().c_str();
 
-        fw_file_name = determineOutputFileName(cluster, fw, !cluster_id.empty(), ".fw");
-
         if (fw->getOptionsObject()->getStr("prolog_place") == "after_flush" &&
             fw->getOptionsObject()->getBool("use_iptables_restore"))
         {
@@ -220,6 +234,23 @@ QString CompilerDriver_ipt::run(const std::string &cluster_id,
         findBranchesInMangleTable(fw, all_policies);
         findImportedRuleSets(fw, all_nat);
 
+        try
+        {
+            AutomaticRules_ipt auto_rules(fw, persistent_objects);
+            auto_rules.addConntrackRule();
+            auto_rules.addFailoverRules();
+        } catch (FWException &ex)
+        {
+            abort(ex.toString());
+        }
+
+        // assign unique rule ids that later will be used to generate
+        // chain names.  This should be done after calls to
+        // findImportedRuleSets()
+
+        assignUniqueRuleIds(all_policies);
+        assignUniqueRuleIds(all_nat);
+
         // command line options -4 and -6 control address family for which
         // script will be generated. If "-4" is used, only ipv4 part will 
         // be generated. If "-6" is used, only ipv6 part will be generated.
@@ -392,10 +423,11 @@ QString CompilerDriver_ipt::run(const std::string &cluster_id,
         {
             routing_compiler->setSourceRuleSet(routing);
             routing_compiler->setRuleSetName(routing->getName());
+            routing_compiler->setPersistentObjects(persistent_objects);
 
             routing_compiler->setSingleRuleCompileMode(single_rule_id);
             routing_compiler->setDebugLevel( dl );
-            if (rule_debug_on) routing_compiler->setDebugRule(drp);
+            if (rule_debug_on) routing_compiler->setDebugRule(drr);
             routing_compiler->setVerbose( verbose );
             if (inTestMode()) routing_compiler->setTestMode();
             if (inEmbeddedMode()) routing_compiler->setEmbeddedMode();
@@ -410,6 +442,13 @@ QString CompilerDriver_ipt::run(const std::string &cluster_id,
                 all_errors.push_back(routing_compiler->getErrors("").c_str());
         }
 
+        /*
+         * compilers detach persistent objects when they finish, this
+         * means at this point library persistent_objects is not part
+         * of any object tree.
+         */
+        objdb->reparent(persistent_objects);
+
         if (haveErrorsAndWarnings())
         {
             all_errors.push_front(getErrors("").c_str());
@@ -435,7 +474,7 @@ QString CompilerDriver_ipt::run(const std::string &cluster_id,
  * now write generated scripts to files
  */
 
-        char *timestr;
+        char *timestr = NULL;
         time_t tm;
         struct tm *stm;
 
@@ -674,17 +713,22 @@ QString CompilerDriver_ipt::run(const std::string &cluster_id,
         Configlet top_comment(fw, "linux24", "top_comment");
 
         top_comment.setVariable("version", VERSION);
-        top_comment.setVariable("build", build_num);
         top_comment.setVariable("timestamp", timestr);
         top_comment.setVariable("tz", tzname[stm->tm_isdst]);
         top_comment.setVariable("user", user_name);
         top_comment.setVariable("database", objdb->getFileName().c_str());
 
-        QFileInfo fw_file_info(fw_file_name);
+        determineOutputFileNames(cluster, fw, !cluster_id.empty(),
+                                 QStringList(""), QStringList("fw"),
+                                 QStringList("script_name_on_firewall"));
+
         script_buffer = "";
-        script << MANIFEST_MARKER << "* " << this->escapeFileName(fw_file_info.fileName());
-        string remote_name = fw->getOptionsObject()->getStr("script_name_on_firewall");
-        if (!remote_name.empty()) script << " " << this->escapeFileName(remote_name.c_str());
+        script << MANIFEST_MARKER
+               << "* "
+               << this->escapeFileName(file_names[FW_FILE]);
+
+        if (!remote_file_names[FW_FILE].isEmpty())
+            script << " " << this->escapeFileName(remote_file_names[FW_FILE]);
         script << "\n";
 
         /* Add additional files to manifest if specified.  Currently there
@@ -716,11 +760,9 @@ QString CompilerDriver_ipt::run(const std::string &cluster_id,
         script_skeleton.setVariable("errors_and_warnings",
                                     prepend("# ", all_errors.join("\n")));
 
-        fw_file_name = getAbsOutputFileName(fw_file_name);
+        info("Output file name: " + file_names[FW_FILE].toStdString());
 
-        info("Output file name: " + fw_file_name.toStdString());
-
-        QFile fw_file(fw_file_name);
+        QFile fw_file(file_names[FW_FILE]);
         if (fw_file.open(QIODevice::WriteOnly))
         {
             QTextStream fw_str(&fw_file);
@@ -737,13 +779,16 @@ QString CompilerDriver_ipt::run(const std::string &cluster_id,
         } else
         {
             QString err(" Failed to open file %1 for writing: %2; Current dir: %3");
-            abort(err.arg(fw_file.fileName()).arg(fw_file.error()).arg(QDir::current().path()).toStdString());
+            abort(err.arg(fw_file.fileName())
+                  .arg(fw_file.error()).arg(QDir::current().path()).toStdString());
         }
 
+        free(timestr);
     }
-    catch (FatalErrorInSingleRuleCompileMode &ex)
+    catch (FWException &ex)
     {
-        return QString::fromUtf8(getErrors("").c_str());
+        status = BaseCompiler::FWCOMPILER_ERROR;
+        return QString::fromUtf8(ex.toString().c_str());
     }
     
     return "";
