@@ -46,18 +46,6 @@
 
 #include "combinedAddress.h"
 
-#include <iostream>
-#if __GNUC__ > 3 || \
-    (__GNUC__ == 3 && (__GNUC_MINOR__ > 2 || (__GNUC_MINOR__ == 2 ) ) ) || \
-    _MSC_VER
-#  include <streambuf>
-#else
-#  include <streambuf.h>
-#endif
-#include <iomanip>
-#include <fstream>
-#include <sstream>
-
 #include <assert.h>
 
 #include <QString>
@@ -181,12 +169,15 @@ string NATCompiler_ipt::PrintRule::_printRuleLabel(NATRule *rule)
                 res << "# " << line.toStdString() << endl;
             }
             //res << "# " << endl;
+
+            string err = compiler->getErrorsForRule(rule, "# ");
+            if (!err.empty()) res << err << endl;
         }
         current_rule_label=rl;
     }
 
-    string err = rule->getStr(".error_msg");
-    if (!err.empty()) res << "# " << err << endl;
+//    string err = rule->getCompilerMessage();
+//    if (!err.empty()) res << "# " << err << endl;
 
     return res.str();
 }
@@ -195,43 +186,39 @@ string NATCompiler_ipt::PrintRule::_printRuleLabel(NATRule *rule)
  *  check and create new chain if needed
  */
 
+QString NATCompiler_ipt::PrintRule::getInterfaceName(RuleElement *itf_re)
+{
+    if (itf_re->isAny()) return "";
+    FWObject *iface = FWObjectReference::getObject(itf_re->front());
+    QString iface_name = iface->getName().c_str();
+    if (iface_name.endsWith("*")) iface_name.replace("*", "+");
+    return iface_name;
+}
+
 /**
  *-----------------------------------------------------------------------
  */
 string NATCompiler_ipt::PrintRule::_printChainDirectionAndInterface(NATRule *rule)
 {
-    std::ostringstream  ostr;
+    QStringList res;
 
-    string       iface_name = rule->getInterfaceStr();
-    if (iface_name=="nil") iface_name="";
+    RuleElementItfInb *itf_in_re = rule->getItfInb(); assert(itf_in_re!=NULL);
+    RuleElementItfOutb *itf_out_re = rule->getItfOutb(); assert(itf_out_re!=NULL);
 
-/* if interface name ends with '*', this is a wildcard
- * interface. Iptables supports wildcard interfaces but uses '+' as a
- * wildcard symbol */
+    QString iface_in_name = getInterfaceName(itf_in_re);
+    QString iface_out_name = getInterfaceName(itf_out_re);
 
-    string::size_type n;
-    if ( (n=iface_name.find("*"))!=string::npos)    iface_name[n]='+';
+    if (rule->getStr(".iface_in") == "nil") iface_in_name = "";
+    if (rule->getStr(".iface_out") == "nil") iface_out_name = "";
 
-    ostr << rule->getStr("ipt_chain") << " ";
+    res << rule->getStr("ipt_chain").c_str();
 
-    switch (rule->getRuleType())
-    {
-    case NATRule::SNAT:  
-	if (!iface_name.empty()) ostr << "-o " << iface_name;
-	break;
-    case NATRule::Masq:  
-	if (!iface_name.empty()) ostr << "-o " << iface_name;
-	break;
-    case NATRule::DNAT:  
-	if (!iface_name.empty()) ostr << "-i " << iface_name;
-	break;
-    case NATRule::Redirect: 
-	if (!iface_name.empty()) ostr << "-i " << iface_name;
-	break;
-    default: break;
-    }
-    ostr << " ";
-    return ostr.str();
+    if ( ! iface_in_name.isEmpty()) res << "-i" << iface_in_name;
+    if ( ! iface_out_name.isEmpty()) res << "-o" << iface_out_name;
+
+    res << "";
+
+    return res.join(" ").toStdString();
 }
 
 string NATCompiler_ipt::PrintRule::_printProtocol(Service *srv)
@@ -642,10 +629,6 @@ bool NATCompiler_ipt::PrintRule::processNext()
     Address  *tdst = compiler->getFirstTDst(rule);  assert(tdst);
     Service  *tsrv = compiler->getFirstTSrv(rule);  assert(tsrv);
 
-//    Interface *iface=
-//	Interface::cast( rule->getRoot()->getById(rule->getInterfaceId() ,true) );
-
-
     cmdout << _startRuleLine();
 
     cmdout << _printChainDirectionAndInterface(rule);
