@@ -34,10 +34,13 @@
 #include <map>
 #include <string>
 
+#include <QMap>
 #include <QSet>
 #include <QString>
 
-namespace libfwbuilder {
+
+namespace libfwbuilder
+{
     class Host;
     class IPService;
     class ICMPService;
@@ -51,10 +54,11 @@ namespace libfwbuilder {
     class RuleElementTSrv;
 };
 
-namespace fwcompiler {
-
-    class NATCompiler_ipt : public NATCompiler {
+namespace fwcompiler
+{
 
+    class NATCompiler_ipt : public NATCompiler
+    {
 	protected:
 
         class PrintRule;
@@ -73,6 +77,9 @@ namespace fwcompiler {
         // This map is located in CompilerDriver_ipt
         const std::map<std::string, std::list<std::string> > *branch_ruleset_to_chain_mapping;
 
+        QMap<QString, libfwbuilder::FWObject*> regular_interfaces;
+
+        
         static const std::list<std::string>& getStandardChains();
         std::string getInterfaceVarName(libfwbuilder::FWObject *iface,
                                         bool v6=false);
@@ -125,6 +132,12 @@ namespace fwcompiler {
 	 */
         DECLARE_NAT_RULE_PROCESSOR(VerifyRules2);
 
+	/**
+	 *  verifies correctness of the combination of interface
+	 *  specification and chain.
+	 */
+        DECLARE_NAT_RULE_PROCESSOR(VerifyRules3);
+
 	/**
 	 * splits rule with multiple objects in ODst. This needs to be
 	 * done only * for DNAT rules. Call this processor when
@@ -241,7 +254,6 @@ namespace fwcompiler {
         friend class AssignInterface;
         class AssignInterface : public NATRuleProcessor
         {
-            QSet<QString> regular_interfaces;
             public:
             AssignInterface(const std::string &name) : NATRuleProcessor(name) {}
             virtual bool processNext();
@@ -285,29 +297,6 @@ namespace fwcompiler {
          */
         DECLARE_NAT_RULE_PROCESSOR(ExpandAddressRanges);
 
-	/**
-	 * split rules with more than one service object, so that each
-	 * rule has services with the same protocol
-	 */
-        DECLARE_NAT_RULE_PROCESSOR(splitServices);
-
-	/**
-	 * separate TCP/UDP services with port ranges (can not be used with multiport)
-	 */
-        DECLARE_NAT_RULE_PROCESSOR(separatePortRanges);
-
-	/**
-	 * for TCP/UDP services separate objects with source ports
-         * from objects with destination ports
-	 */
-        DECLARE_NAT_RULE_PROCESSOR(separateSourcePorts);
-
-	/**
-	 * for TCP/UDP services separate objects that have both source
-	 * and destination ports
-	 */
-        DECLARE_NAT_RULE_PROCESSOR(separateSourceAndDestinationPorts);
-
 	/**
 	 * splits rules so multiport module can be used (only works for UDP and TCP)
 	 */
@@ -519,6 +508,8 @@ namespace fwcompiler {
             std::string version;
             void initializeMinusNTracker();
 
+            QString getInterfaceName(libfwbuilder::RuleElement *itf_re);
+
             /*
              * Prints single --option with argument and negation "!"
              * taking into account the change that happened in iptables 1.4.3.1
