@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: OSConfigurator_linux24_interfaces.cpp 3023 2010-06-25 19:48:28Z vadim $
+  $Id$
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -81,7 +81,10 @@ string OSConfigurator_linux24::printVerifyInterfacesCommands()
     for (i=interfaces.begin(); i!=interfaces.end(); ++i )
     {
         QString iface_name = (*i)->getName().c_str();
-        if (interfaces_to_check.indexOf(iface_name) == -1)
+/* if interface name ends with '*', this is a wildcard interface. Do
+ * not check if it exists.
+ */
+        if (!iface_name.contains("*") && interfaces_to_check.indexOf(iface_name) == -1)
         {
             interfaces_to_check.push_back((*i)->getName().c_str());
         }
@@ -486,10 +489,11 @@ string OSConfigurator_linux24::printDynamicAddressesConfigurationCommands()
 /*
  * get addresses of dynamic interfaces
  */
-    FWObjectTypedChildIterator j=fw->findByType(Interface::TYPENAME);
-    for ( ; j!=j.end(); ++j ) 
+    list<FWObject*> interfaces = fw->getByTypeDeep(Interface::TYPENAME);
+    list<FWObject*>::iterator j;
+    for (j=interfaces.begin(); j!=interfaces.end(); ++j )
     {
-        Interface *iface=Interface::cast(*j);
+        Interface *iface = Interface::cast(*j);
         
         if ( iface->isDyn() )
         {
