@@ -33,6 +33,10 @@
 #include "config.h"
 #include "Configlet.h"
 
+#include <QMap>
+#include <QString>
+
+
 namespace libfwbuilder
 {
     class Interface;
@@ -83,6 +87,8 @@ protected:
         // rules in different policy or nat objects)
         std::map<const std::string, bool> *minus_n_commands;
 
+        QMap<QString, libfwbuilder::FWObject*> regular_interfaces;
+
         static const std::list<std::string>& getStandardChains();
 
         void registerChain(const std::string &chain_name);
@@ -95,19 +101,6 @@ protected:
         bool isChainDescendantOfOutput(const std::string &chain_name);
         bool isChainDescendantOfInput(const std::string &chain_name);
 
-	void insertConntrackRule();
-	void insertFailoverRule();
-
-        libfwbuilder::PolicyRule* addMgmtRule(
-            libfwbuilder::Address* src,
-            libfwbuilder::Address* dst,
-            libfwbuilder::Service* service,
-            libfwbuilder::Interface* iface,
-            const libfwbuilder::PolicyRule::Direction direction,
-            const libfwbuilder::PolicyRule::Action action,
-            const std::string label,
-            const bool related = false);
-        
         std::string getInterfaceVarName(libfwbuilder::FWObject *iface,
                                         bool v6=false);
         std::string getAddressTableVarName(libfwbuilder::FWObject *iface);
@@ -762,24 +755,7 @@ protected:
         };
         friend class PolicyCompiler_ipt::checkForStatefulICMP6Rules;
 
-	/**
-	 * This processor separates TCP/UDP services with port ranges
-	 * (they can not be used with multiport). It also separates
-	 * rules using "Any UDP" and "Any TCP" objects (they have
-	 * all ports set to zero)
-	 */
-        DECLARE_POLICY_RULE_PROCESSOR(separatePortRanges);
-
-        DECLARE_POLICY_RULE_PROCESSOR(separateUserServices);
         
-	/**
-	 * separate TCP/UDP services that specify source port (can
-	 * not be used in combination with destination port with
-	 * multiport)
-	 */
-        DECLARE_POLICY_RULE_PROCESSOR(separateSrcPort);
-
-
 	/**
 	 * deals with special cases with some known custom services
 	 */
@@ -819,9 +795,11 @@ protected:
             std::map<std::string, bool> rules_seen_so_far;
             PolicyCompiler_ipt::PrintRule *printRule;
             public:
-            optimize3(const std::string &name) : PolicyRuleProcessor(name){
-                printRule=NULL;
+            optimize3(const std::string &name) : PolicyRuleProcessor(name)
+            {
+                printRule = NULL;
             }
+            virtual ~optimize3() { if (printRule) delete printRule; }
             virtual bool processNext();
         };
         friend class PolicyCompiler_ipt::optimize3;
