@@ -85,9 +85,12 @@
 #include "UserWorkflow.h"
 #include "ObjectManipulator.h"
 #include "FWWindow.h"
+#include "networkZoneManager.h"
 
 #include "IOSImporter.h"
 #include "IPTImporter.h"
+#include "PIXImporter.h"
+
 
 using namespace std;
 using namespace libfwbuilder;
@@ -95,6 +98,10 @@ using namespace libfwbuilder;
 DiscoveryDruid::DiscoveryDruid(QWidget *parent, bool start_with_import) :
     QDialog(parent)
 {
+    init = true;
+
+    discovered_fw = NULL;
+
     m_dialog = new Ui::DiscoveryDruid_q;
     m_dialog->setupUi(this);
 
@@ -105,6 +112,23 @@ DiscoveryDruid::DiscoveryDruid(QWidget *parent, bool start_with_import) :
                       m_dialog->cancelButton,
                       m_dialog->titleLabel);
 
+    QTextCursor cursor(m_dialog->discoverylog->textCursor());
+    normal_format = cursor.charFormat();
+
+    error_format = normal_format;
+    error_format.setForeground(QBrush(Qt::red));
+    error_format.setAnchorHref("http://somewhere.com");
+    error_format.setAnchor(true);
+    // weight must be between 0 and 99. Qt 4.4.1 does not seem to mind if
+    // it is >99 (just caps it) but older versions assert
+    error_format.setProperty(QTextFormat::FontWeight, 99);
+
+    warning_format = normal_format;
+    warning_format.setForeground(QBrush(Qt::blue));
+    warning_format.setProperty(QTextFormat::FontWeight, 99);
+    warning_format.setAnchor(true);
+    warning_format.setAnchorHref("http://somewhere.com");
+
     dm_method = new QButtonGroup;
     dm_method->addButton(m_dialog->dm_fromfile,0);
     dm_method->addButton(m_dialog->dm_importdns,1);
@@ -167,21 +191,23 @@ DiscoveryDruid::DiscoveryDruid(QWidget *parent, bool start_with_import) :
 
     importPlatformChanged(m_dialog->import_platform->currentIndex());
 
-    showPage(0);
-    setNextEnabled(0, true);
+    showPage(CHOOSE_METHOD_PAGE);
+    setNextEnabled(CHOOSE_METHOD_PAGE, true);
 
     if (start_with_import)
     {
         m_dialog->dm_import_config->setDown(true);
         setDiscoveryMethod_Import();
-        setAppropriate( 0, false );
+        setAppropriate( CHOOSE_METHOD_PAGE, false );
         // show the first page of the "import policy" track of the wizard
-        showPage( 2 );
-        setNextEnabled(2, false);
+        showPage( IMPORT_CONFIG_PAGE );
+        setNextEnabled(IMPORT_CONFIG_PAGE, false);
         cancelButton->show();
     }
 
     prg_timer->start(100);
+
+    init = false;
 }
 
 void DiscoveryDruid::nextClicked()
@@ -198,6 +224,62 @@ void DiscoveryDruid::backClicked()
 
 void DiscoveryDruid::finishClicked()
 {
+    if (current_task == BT_IMPORT && discovered_fw != NULL)
+    {
+        if (selectedPlatform() == "pix" && currentPage() == NETWORK_ZONES_PAGE)
+        {
+            // read and configure network zones
+            list<FWObject*> all_interfaces =
+                discovered_fw->getByTypeDeep(Interface::TYPENAME);
+            list<FWObject*>::iterator it;
+            for (it=all_interfaces.begin(); it!=all_interfaces.end(); ++it)
+            {
+                Interface *iface = Interface::cast(*it);
+
+                string  network_zone_str_id = "";
+
+                QList<QTableWidgetItem*> ltwi =
+                    m_dialog->iface_nz_list->findItems( iface->getName().c_str(),
+                                                        Qt::MatchExactly );
+                if ( ! ltwi.empty())
+                {
+                    QTableWidgetItem *itm2 = ltwi[0];
+                    assert(itm2!=NULL);
+                    int row = itm2->row();
+                    QComboBox *cb = dynamic_cast<QComboBox*>(
+                        m_dialog->iface_nz_list->cellWidget(row, 3));
+                    assert(cb!=NULL);
+                    int network_zone_int_id =
+                        cb->itemData(cb->currentIndex(), Qt::UserRole).toInt();
+                    if (network_zone_int_id != 0)
+                        network_zone_str_id = FWObjectDatabase::getStringId(
+                            network_zone_int_id);
+                    else
+                        network_zone_str_id = "";
+                }
+
+                // only set network zone if it is supported and is not
+                // empty. See #2014
+                if (!network_zone_str_id.empty())
+                    iface->setStr("network_zone", network_zone_str_id);
+
+            }
+        }
+
+        QCoreApplication::postEvent(
+            mw->activeProject(), new expandObjectInTreeEvent(
+                mw->activeProject()->getFileName(), discovered_fw->getId()));
+
+        QCoreApplication::postEvent(
+            mw->activeProject(), new showObjectInTreeEvent(
+                mw->activeProject()->getFileName(), discovered_fw->getId()));
+
+        QCoreApplication::postEvent(
+            mw, new openObjectInEditorEvent(
+                mw->activeProject()->getFileName(), discovered_fw->getId()));
+
+    }
+
     QDialog::accept();
 }
 
@@ -412,26 +494,28 @@ void DiscoveryDruid::dnsFinish(QHostInfo host)
 
 void DiscoveryDruid::changedSelected( const int &page )
 {
+    if (init) return;
+
     switch (page)
     {
 
-    case 1: // Reading file in hosts format
+    case READ_HOSTS_FILE_PAGE: // Reading file in hosts format
     {
-        setNextEnabled(page,false);
+        setNextEnabled(page, false);
         changedHostsFileName();
         m_dialog->filename->setFocus();
         break;
     }
 
-    case 2: // import config
+    case IMPORT_CONFIG_PAGE: // import config
     {
         m_dialog->obj_name->setFocus();
-        setBackEnabled(page,true);
-        setFinishEnabled(page,false);
+        setBackEnabled(page, true);
+        setFinishEnabled(page, false);
         break;
     }
 
-    case 3: // Import DNS zone
+    case IMPORT_DNS_ZONE_PAGE: // Import DNS zone
     {
         changedDomainName();
         m_dialog->domainname->setFocus();
@@ -439,7 +523,7 @@ void DiscoveryDruid::changedSelected( const int &page )
         break;
     }
 
-    case 4: // Name server
+    case NAME_SERVER_PAGE: // Name server
     {
         if (page>FromPage)
             getNameServers();
@@ -452,7 +536,7 @@ void DiscoveryDruid::changedSelected( const int &page )
         break;
     }
 
-    case 5: // Network discovery using SNMP
+    case SNMP_DISCOVERY_PAGE: // Network discovery using SNMP
     {
         disconnect(timer,SIGNAL(timeout()),0,0);
         connect(timer,SIGNAL(timeout()),this,SLOT(checkHostName()));
@@ -466,21 +550,21 @@ void DiscoveryDruid::changedSelected( const int &page )
         break;
     }
 
-    case 6: // Network scan options
+    case NETWORK_SCAN_OPTIONS_PAGE: // Network scan options
     {
         m_dialog->snmprecursive->setFocus();
         //setNextEnabled(page,false);
         break;
     }
 
-    case 7: // SNMP and DNS reverse lookup queries parameters
+    case SNMP_PARAMETERS_PAGE: // SNMP and DNS reverse lookup queries parameters
     {
         checkSNMPCommunity();
         m_dialog->snmpcommunity->setFocus();
         break;
     }
 
-    case 8: // Background process (import from hosts and from config file)
+    case BACKGROUND_PROCESS_PAGE: // Background process (import from hosts and from config file)
     {
         m_dialog->discoveryprogress->setValue(-1);
         m_dialog->discoverylog->clear();
@@ -501,7 +585,7 @@ void DiscoveryDruid::changedSelected( const int &page )
         break;
     }
 
-    case 9: // Networks
+    case CHOOSE_NETWORKS_PAGE: // Networks
     {
         fillListOfNetworks();
         fillNetworks();
@@ -510,7 +594,7 @@ void DiscoveryDruid::changedSelected( const int &page )
         break;
     }
 
-    case 10: // Objects
+    case CHOOSE_OBJECTS_PAGE: // Objects
     {
         if (Networks.size()==0)
             setBackEnabled(page,false);
@@ -521,32 +605,44 @@ void DiscoveryDruid::changedSelected( const int &page )
         break;
     }
 
-    case 11: // Adjust Object type
+    case ADJUST_OBJECT_TYPES_PAGE: // Adjust Object type
     {
         setBackEnabled(page,true);
         fillTypeChangingList();
         break;
     }
 
-    case 12: // Target library
+    case TARGET_LIB_PAGE: // Target library
     {
         break;
     }
 
-    case 13: // Objects creation ...
+    case CREATE_OBJECTS_PAGE: // Objects creation ...
     {
         setBackEnabled(page,false);
         cancelButton->hide();
         createRealObjects();
-        setFinishEnabled(page,true);
+        setNextEnabled(page, false);
+        setFinishEnabled(page, true);
         finishButton->setFocus();
         break;
     }
 
+    case NETWORK_ZONES_PAGE: // Network zones for PIX
+    {
+        setBackEnabled(page, false);
+        cancelButton->hide();
+        setNextEnabled(page, false);
+        setFinishEnabled(page, true);
+        finishButton->setFocus();
+        fillNetworkZones();
+        break;
+    }
+
     default : {}
 
     }
-    FromPage=page;
+    FromPage = page;
 }
 
 void DiscoveryDruid::startBackgroundProcess()
@@ -644,9 +740,9 @@ void DiscoveryDruid::getNameServers()
 
 void DiscoveryDruid::setDiscoveryMethod_file()
 {
-    current_task=BT_HOSTS;
     m_dialog->processname->setText(tr("Hosts file parsing ..."));
-    for (int i=0;i<WIZARD_PAGES;i++)
+    current_task = BT_HOSTS;
+    for (int i=0; i<WIZARD_PAGES; i++)
     {
         setAppropriate( i, WIZARD_FILE_PAGES[i]);
     }
@@ -655,8 +751,8 @@ void DiscoveryDruid::setDiscoveryMethod_file()
 void DiscoveryDruid::setDiscoveryMethod_DNS()
 {
     m_dialog->processname->setText(tr("DNS zone transfer ..."));
-    current_task=BT_DNS;
-    for (int i=0;i<WIZARD_PAGES;i++)
+    current_task = BT_DNS;
+    for (int i=0; i<WIZARD_PAGES; i++)
     {
         setAppropriate( i, WIZARD_DNS_PAGES[i]);
     }
@@ -665,8 +761,8 @@ void DiscoveryDruid::setDiscoveryMethod_DNS()
 void DiscoveryDruid::setDiscoveryMethod_SNMP()
 {
     m_dialog->processname->setText(tr("Network discovery using SNMP ..."));
-    current_task=BT_SNMP;
-    for (int i=0;i<WIZARD_PAGES;i++)
+    current_task = BT_SNMP;
+    for (int i=0; i<WIZARD_PAGES; i++)
     {
         setAppropriate( i, WIZARD_SNMP_PAGES[i]);
     }
@@ -675,8 +771,8 @@ void DiscoveryDruid::setDiscoveryMethod_SNMP()
 void DiscoveryDruid::setDiscoveryMethod_Import()
 {
     m_dialog->processname->setText(tr("Import configuration from file ..."));
-    current_task=BT_IMPORT;
-    for (int i=0;i<WIZARD_PAGES;i++)
+    current_task = BT_IMPORT;
+    for (int i=0; i<WIZARD_PAGES; i++)
     {
         setAppropriate( i, WIZARD_IMPORT_PAGES[i]);
     }
@@ -757,12 +853,7 @@ void DiscoveryDruid::startConfigImport()
 
         // need to pick right platform string based on
         // m_dialog->import_platform->currentItem()
-        string platform = "";
-        switch (m_dialog->import_platform->currentIndex())
-        {
-        case 0: platform = "iosacl"; break;
-        case 1: platform = "iptables"; break;
-        }
+        string platform = selectedPlatform();
 
         //
         // ConfigImport "owns" buffer - it is deleted
@@ -782,6 +873,18 @@ void DiscoveryDruid::startConfigImport()
     }
 }
 
+string DiscoveryDruid::selectedPlatform()
+{
+    string platform = "";
+    switch (m_dialog->import_platform->currentIndex())
+    {
+    case IMPORT_IOS: platform = "iosacl"; break;
+    case IMPORT_IPT: platform = "iptables"; break;
+    case IMPORT_PIX: platform = "pix"; break;
+    }
+    return platform;
+}
+
 InetAddr DiscoveryDruid::getNS()
 {
     string ns;
@@ -1001,7 +1104,7 @@ void DiscoveryDruid::stopBackgroundProcess()
 
     if (bop!=NULL && bop->isRunning())
     {
-        m_dialog->discoverylog->append("Terminating task. Please wait...");
+        addToLog("Terminating task. Please wait...");
 
         bop->stop_operation();
         m_dialog->discoveryStopButton->setEnabled(false);
@@ -1191,6 +1294,55 @@ void DiscoveryDruid::selectAllObjs()
     m_dialog->objectlist->selectAll();
 }
 
+void DiscoveryDruid::fillNetworkZones()
+{
+    m_dialog->iface_nz_list->clear();
+
+    QStringList labels;
+    labels << QObject::tr("Name") << QObject::tr("Label")
+           << QObject::tr("Address") << QObject::tr("Network Zone");
+    m_dialog->iface_nz_list->setHorizontalHeaderLabels(labels);
+
+    NetworkZoneManager netzone_manager;
+    netzone_manager.load(mw->activeProject()->db());
+
+    list<FWObject*> all_interfaces = discovered_fw->getByTypeDeep(Interface::TYPENAME);
+    list<FWObject*>::iterator it;
+    int row = 0;
+    for (it=all_interfaces.begin(); it!=all_interfaces.end(); ++it)
+    {
+        Interface *iface = Interface::cast(*it);
+
+        m_dialog->iface_nz_list->insertRow(row);
+
+        QTableWidgetItem* itm;
+
+        itm = new QTableWidgetItem(iface->getName().c_str());
+        itm->setFlags(itm->flags() & ~Qt::ItemIsEditable);
+        m_dialog->iface_nz_list->setItem(row, 0, itm);
+
+        itm = new QTableWidgetItem(iface->getLabel().c_str());
+        itm->setFlags(itm->flags() & ~Qt::ItemIsEditable);
+        m_dialog->iface_nz_list->setItem(row, 1, itm);
+
+        QString addr_str;
+        const InetAddr* addr = iface->getAddressPtr();
+        if (addr) addr_str = addr->toString().c_str();
+
+        itm = new QTableWidgetItem(addr_str);
+        itm->setFlags(itm->flags() & ~Qt::ItemIsEditable);
+        m_dialog->iface_nz_list->setItem(row, 2, itm);
+
+        QComboBox *widget = new QComboBox();
+        netzone_manager.packComboBox(widget, -1);
+        m_dialog->iface_nz_list->setCellWidget(row, 3, widget);
+
+        row++;
+    }
+
+    m_dialog->iface_nz_list->resizeColumnToContents(3);
+}
+
 void DiscoveryDruid::fillNetworks()
 {
     ObjectDescriptor buf;
@@ -1313,8 +1465,11 @@ void DiscoveryDruid::loadDataFromImporter()
     if (imp!=NULL)
     {
         Firewall *fw = imp->finalize();
+        qApp->processEvents(); // to flush the log
+
         if (fw) // fw can be NULL if import was uncussessful
         {
+            discovered_fw = fw;
 
             ProjectPanel *pp = mw->activeProject();
             QString filename = pp->getFileName();
@@ -1420,7 +1575,7 @@ void DiscoveryDruid::startSNMPScan()
     {
         logger = bop->start_operation();
         if (fwbdebug) logger->copyToStderr();
-        m_dialog->discoverylog->append("Collecting data ...");
+        addToLog("Collecting data ...");
 
         disconnect(timer, SIGNAL(timeout()), 0, 0);
         connect(timer, SIGNAL(timeout()), this, SLOT(updateLog()));
@@ -1508,10 +1663,8 @@ void DiscoveryDruid::loadDataFromCrawler()
                     od.sysname = hostName.toLatin1().constData();
             }
 
-            QString buf;
-
-            buf = QString(od.addr.toString().c_str()) + " : " + od.sysname.c_str();
-            m_dialog->discoverylog->append(buf);
+            addToLog(
+                QString(od.addr.toString().c_str()) + " : " + od.sysname.c_str());
 
         }
 
@@ -1631,9 +1784,11 @@ void DiscoveryDruid::customEvent(QEvent *event)
         case BT_HOSTS:
             loadDataFromFile();
             break;
+
         case BT_IMPORT:
             loadDataFromImporter();
             break;
+
         default:
             break;
         }
@@ -1659,10 +1814,22 @@ void DiscoveryDruid::customEvent(QEvent *event)
                 nextButton->setEnabled(false);
             }
             break;
+
         case BT_IMPORT:
-            setFinishEnabled(currentPage(),true);
-            finishButton->setFocus();
+        {
+            // if imported PIX, need to show one more page to select network zones
+            if (selectedPlatform() == "pix")
+            {
+                setNextEnabled(currentPage(), true);
+                setFinishEnabled(currentPage(), false);
+            } else
+            {
+                setNextEnabled(currentPage(), false);
+                setFinishEnabled(currentPage(), true);
+                finishButton->setFocus();
+            }
             break;
+        }
         default:
             break;
         }
@@ -1682,7 +1849,7 @@ void DiscoveryDruid::updateLog()
             while(thread->Log->ready())
             {
                 buf = thread->Log->getLine().c_str();
-                m_dialog->discoverylog->insertPlainText(buf);
+                addToLog(buf);
             }
         }
     }
@@ -1707,8 +1874,9 @@ void DiscoveryDruid::updateLog()
             FWException * ex=bop->get_latest_error();
             if (ex!=NULL)
             {
-                QMessageBox::critical(this,tr("Discovery error"), ex->toString().c_str());
-                //m_dialog->discoverylog->append(QString("\nLast exception: ")+ex->toString().c_str()+"\n");
+                QMessageBox::critical(this,
+                                      tr("Discovery error"),
+                                      ex->toString().c_str());
             }
             if (Objects.size()>0 || Networks.size()>0)
             {
@@ -1753,8 +1921,8 @@ void DiscoveryDruid::updateLog()
             FWException * ex=bop->get_latest_error();
             if (ex!=NULL)
             {
-                QMessageBox::critical(this,tr("Discovery error"), ex->toString().c_str());
-                //m_dialog->discoverylog->append(QString("\nLast exception: ")+ex->toString().c_str()+"\n");
+                QMessageBox::critical(this, tr("Discovery error"),
+                                      ex->toString().c_str());
             }
             if (Objects.size()>0)
             {
@@ -1779,6 +1947,38 @@ void DiscoveryDruid::updateLog()
     }
 }
 
+void DiscoveryDruid::addToLog(const QString &buf)
+{
+    if (buf.isEmpty()) return;
+
+    foreach(QString line, buf.trimmed().split("\n"))
+    {
+        QTextCharFormat format = normal_format;
+
+        if (line.contains("Parser error"))
+            format = error_format;
+
+        if (line.contains("Parser warning"))
+            format = warning_format;
+
+        if (line.contains("SNMP error, status 2 Timeout"))
+            format = warning_format;
+
+        QString txt = line;
+        while (!txt.isEmpty() && (txt.endsWith("\n") || txt.endsWith("\r")))
+            txt.chop(1);
+
+        if (format == error_format || format == warning_format)
+            format.setAnchorHref(txt);
+
+        QTextCursor cursor = m_dialog->discoverylog->textCursor();
+        cursor.insertBlock();
+        cursor.insertText(txt, format);
+    }
+
+    m_dialog->discoverylog->ensureCursorVisible();
+}
+
 void DiscoveryDruid::changedSeedHost()
 {
     m_dialog->seedhosterror_message->setText(" ");
@@ -1931,7 +2131,7 @@ int DiscoveryDruid::monitorOperation()
         if (buf.endsWith('\n'))
             buf = buf.left(buf.length() - 1);
 
-        m_dialog->discoverylog->append(buf);
+        addToLog(buf);
 
         /*if (fwbdebug) qDebug("monitorOperation  appending the following buf: (1)");
         if (fwbdebug) qDebug(buf.toAscii().constData());
@@ -1963,7 +2163,7 @@ int DiscoveryDruid::monitorOperation()
         if (buf.endsWith('\n'))
             buf = buf.left(buf.length() - 1);
 
-        m_dialog->discoverylog->append(buf);
+        addToLog(buf);
 
         /*if (fwbdebug) qDebug("monitorOperation  appending the following buf: (2)");
         if (fwbdebug) qDebug(buf.toAscii().constData());
@@ -2422,7 +2622,7 @@ void DiscoveryDruid::importPlatformChanged(int cp)
 
     switch (cp)
     {
-    case 0:
+    case IMPORT_IOS:
         m_dialog->import_text->setText(
             QObject::tr("Firewall Builder can import Cisco IOS access lists "
                         "from the router configuration saved using 'show run' "
@@ -2433,7 +2633,20 @@ void DiscoveryDruid::importPlatformChanged(int cp)
             )
         );
         break;
-    case 1:
+
+    case IMPORT_PIX:
+        m_dialog->import_text->setText(
+            QObject::tr("Firewall Builder can import Cisco PIX and ASA "
+                        "configuration saved with 'show run' command.  "
+                        "The name of the created firewall object, all of "
+                        "its interfaces and their addresses will be "
+                        "configured automatically if this information can "
+                        "be found in the configuration file."
+            )
+        );
+        break;
+
+    case IMPORT_IPT:
         m_dialog->import_text->setText(
             QObject::tr("Firewall Builder can import iptables rules "
                         "from a file in iptables-save format. Firewall "
@@ -2443,8 +2656,9 @@ void DiscoveryDruid::importPlatformChanged(int cp)
             )
         );
         break;
+            
     }
-
+    
 }
 
 //----------------------------------------------------------------------
@@ -2548,7 +2762,7 @@ HostsFileImport::HostsFileImport(const QString &f) :
 
 void HostsFileImport::run()
 {
-    *Log << "Discovery method:"
+    *Log << "Discovery method: "
          << "Read file in hosts format. \n";
 
     map<InetAddr, vector<string> > reverse_hosts;
@@ -2623,10 +2837,10 @@ ConfigImport::~ConfigImport()
 
 void ConfigImport::run()
 {
-    *Log << "Discovery method:"
-         << "Import firewall configuration. \n";
+    *Log << "Discovery method: Import firewall configuration.\n";
 
     std::istringstream instream(*buffer);
+
     imp = NULL;
     if (platform == "iosacl") imp = new IOSImporter(mw->getCurrentLib(),
                                                     instream,
@@ -2634,6 +2848,9 @@ void ConfigImport::run()
     if (platform == "iptables") imp = new IPTImporter(mw->getCurrentLib(),
                                                       instream,
                                                       Log, fwname);
+    if (platform == "pix") imp = new PIXImporter(mw->getCurrentLib(),
+                                                 instream,
+                                                 Log, fwname);
 
     // add other platforms here when available
 
@@ -2645,7 +2862,6 @@ void ConfigImport::run()
         } catch(ImporterException &e)
         {
             last_error = e.toString().c_str();
-            *Log << "Parser error:\n";
             *Log << e.toString() << "\n";
         }
 
