@@ -52,7 +52,6 @@
 #include "PrefsDialog.h"
 #include "LibExportDialog.h"
 #include "findDialog.h"
-#include "DiscoveryDruid.h"
 #include "FindObjectWidget.h"
 #include "FindWhereUsedWidget.h"
 #include "CompilerOutputPanel.h"
@@ -74,6 +73,10 @@
 
 #include "events.h"
 
+#include "importAddressListWizard/ImportAddressListWizard.h"
+#include "snmpNetworkDiscoveryWizard/SNMPNetworkDiscoveryWizard.h"
+#include "importFirewallConfigurationWizard/ImportFirewallConfigurationWizard.h"
+
 #include "fwbuilder/FWReference.h"
 #include "fwbuilder/Policy.h"
 #include "fwbuilder/NAT.h"
@@ -449,6 +452,8 @@ void FWWindow::showSub(ProjectPanel *pp)
     sub->setAttribute(Qt::WA_DeleteOnClose);
     m_mainWindow->m_space->addSubWindow(sub);
 
+    connect( sub, SIGNAL(aboutToActivate()), pp, SLOT(aboutToActivate()));
+
     if (fwbdebug)
         qDebug() << "Show subwindow maximized: " << windows_maximized;
 
@@ -858,12 +863,21 @@ void FWWindow::fileExit()
     app->quit();
 }
 
-void FWWindow::toolsDiscoveryDruid()
+void FWWindow::toolsImportAddressesFromFile()
 {
     if (activeProject())
     {
-        DiscoveryDruid druid(this);
-        druid.exec();
+        ImportAddressListWizard wiz(this);
+        wiz.exec();
+    }
+}
+
+void FWWindow::toolsSNMPDiscovery()
+{
+    if (activeProject())
+    {
+        SNMPNetworkDiscoveryWizard wiz(this);
+        wiz.exec();
     }
 }
 
@@ -873,9 +887,9 @@ void FWWindow::importPolicy()
     {
         if (!activeProject()->m_panel->om->isObjectAllowed(Firewall::TYPENAME))
             return;
-
-        DiscoveryDruid druid(this, true);
-        druid.exec();
+        
+        ImportFirewallConfigurationWizard wiz(this, db());
+        wiz.exec();
     }
 }
 
@@ -986,12 +1000,24 @@ void FWWindow::prepareFileMenu()
     m_mainWindow->filePropAction->setEnabled(real_file_opened);
     m_mainWindow->filePrintAction->setEnabled(real_file_opened);
     m_mainWindow->libExportAction->setEnabled(real_file_opened);
-    m_mainWindow->libImportAction->setEnabled(true);
-    m_mainWindow->policyImportAction->setEnabled(true);
+
+    FWObject *lib = activeProject()->getCurrentLib();
+    bool f = (
+        lib == NULL ||
+        lib->getId()==FWObjectDatabase::TEMPLATE_LIB_ID ||
+        lib->getId()==FWObjectDatabase::DELETED_OBJECTS_ID  ||
+        lib->isReadOnly()
+    );
+    bool new_object_op_possible = !f;
+
+    m_mainWindow->libImportAction->setEnabled(new_object_op_possible);
+    m_mainWindow->policyImportAction->setEnabled(new_object_op_possible);
 
     m_mainWindow->addToRCSAction->setEnabled(real_file_opened && !in_rcs);
-    m_mainWindow->fileCommitAction->setEnabled(real_file_opened && in_rcs && needs_saving);
-    m_mainWindow->fileDiscardAction->setEnabled(real_file_opened && in_rcs && needs_saving);
+    m_mainWindow->fileCommitAction->setEnabled(
+        real_file_opened && in_rcs && needs_saving);
+    m_mainWindow->fileDiscardAction->setEnabled(
+        real_file_opened && in_rcs && needs_saving);
 
     m_mainWindow->fileNewAction->setEnabled(true);
     m_mainWindow->fileOpenAction->setEnabled(true);
@@ -1000,7 +1026,11 @@ void FWWindow::prepareFileMenu()
 
 void FWWindow::prepareToolsMenu()
 {
-    m_mainWindow->DiscoveryDruidAction->setEnabled(db()!=NULL);
+#ifdef HAVE_LIBSNMP
+    m_mainWindow->SNMPDiscoveryAction->setEnabled(true);
+#else
+    m_mainWindow->SNMPDiscoveryAction->setEnabled(false);
+#endif
 }
 
 void FWWindow::prepareWindowsMenu()
@@ -1085,8 +1115,8 @@ QStringList FWWindow::getListOfOpenedFiles()
 
 void FWWindow::activatePreviousSubWindow()
 {
+    if (fwbdebug) qDebug() << "FWWindow::activatePreviousSubWindow()";
     m_mainWindow->m_space->setActiveSubWindow(previous_subwindow);
-    //previous_subwindow->raise();
 }
 
 
@@ -1103,39 +1133,23 @@ void FWWindow::subWindowActivated(QMdiSubWindow *subwindow)
     if (subwindow==NULL) return;
 
     if (fwbdebug)
-        qDebug() << "FWWindow::subWindowActivated subwindow="
-                 << subwindow
-                 << " "
-                 << subwindow->windowTitle()
+        qDebug() << "FWWindow::subWindowActivated"
+                 << "subwindow=" << subwindow
+                 << "(" << subwindow->windowTitle() << ")"
+                 << "previous_subwindow=" << previous_subwindow
+                 << "("
+                 << QString((previous_subwindow) ? previous_subwindow->windowTitle() : "")
+                 << ")"
                  << "isMaximized()=" << subwindow->isMaximized();
 
     if (previous_subwindow == subwindow) return;
 
-    // if (isEditorVisible() && !oe->validateAndSave())
-    // {
-    //     // editor has unsaved data and user clicked "Continue editing"
-    //     // Roll back switch of subwindows
-
-    //     if (fwbdebug)
-    //         qDebug() << "Activating previous subwindow "
-    //                  << previous_subwindow
-    //                  << " "
-    //                  << previous_subwindow->windowTitle();
-    //     QTimer::singleShot(0, this, SLOT(activatePreviousSubWindow()));
-    //     return;
-    // }
-
     previous_subwindow = subwindow;
 
     ProjectPanel *pp = dynamic_cast<ProjectPanel*>(subwindow->widget());
     if (pp)
     {
         QCoreApplication::postEvent(mw, new updateGUIStateEvent());
-
-        //prepareFileMenu();
-        //prepareRulesMenu();
-        //updateGlobalToolbar();
-
         pp->setActive();
         if (isEditorVisible()) openEditor(pp->getSelectedObject());
     }
@@ -1321,19 +1335,33 @@ bool FWWindow::event(QEvent *event)
             ev->accept();
             return true;
         }
+
+        case CLOSE_EDITOR_PANEL_EVENT:
+        {
+            hideEditor();
+            ev->accept();
+            return true;
+        }
+
+        case CLEAR_EDITOR_PANEL_EVENT:
+        {
+            clearEditorAndSearchPanels();
+            ev->accept();
+            return true;
+        }
+
         }
 
         // dispatch event to all projectpanel windows
         foreach(QMdiSubWindow* sw, m_mainWindow->m_space->subWindowList())
             QCoreApplication::sendEvent(sw->widget(), event);
 
-        // QList<QMdiSubWindow*> subWindowList = m_mainWindow->m_space->subWindowList();
-        // for (int i = 0 ; i < subWindowList.size(); i++)
-        //     QCoreApplication::sendEvent(subWindowList[i]->widget(), event);
-
         event->accept();
         return true;
     }
+
+    //if (fwbdebug) qDebug() << this << "event:" << event;
+
     return QMainWindow::event(event);
 }
 
@@ -1342,8 +1370,11 @@ void FWWindow::selectActiveSubWindow(/*const QString & text*/)
     QObject * sender_ = sender ();
     QAction * act = (QAction*) sender_ ;
     QString text = act->text();
-    if (text=="[Noname]")
-        text="";
+    if (text=="[Noname]") text="";
+
+    if (fwbdebug) qDebug() << "FWWindow::selectActiveSubWindow()"
+                           << "text=" << text;
+
     for (int i = 0 ; i < windowsTitles.size();i++)
     {
         if (windowsTitles[i]==text)
@@ -1499,7 +1530,16 @@ void FWWindow::help()
 
 void FWWindow::showReleaseNotes()
 {
-    QString file_name = QString("release_notes_%1.html").arg(VERSION);
+    QStringList version_components = QString(VERSION).split(".");
+    assert(version_components.size() >= 3);
+
+    QString version_no_build = QString("%1.%2.%3")
+        .arg(version_components[0])
+        .arg(version_components[1])
+        .arg(version_components[2]);
+
+    QString file_name = QString("release_notes_%1.html").arg(version_no_build);
+
     // Show "release notes" dialog only if corresponding file
     // exists.
     QString contents;
