@@ -86,30 +86,46 @@ bool FirewallInstaller::parseManifestLine(const QString &line,
         *main_script = true;
         workline = workline.remove(0, 1).trimmed();
     }
+
     QString local, remote;
     int i = 0;
+
     for(i=0; i<workline.size(); i++)
     {
+        // qDebug() << "1: " << workline.at(i);
+
         if (workline.at(i) == ' ' && workline.at(i-1) != '\\')
+        {
+            i++;
             break;
-        else
+        } else
             local.append(workline.at(i));
     }
+
+    //qDebug() << "local=" << local;
+
     for(; i<workline.size(); i++)
     {
+        // qDebug() << "2: " << workline.at(i);
+
         if (workline.at(i) == ' ' && workline.at(i-1) != '\\')
             break;
         else
             remote.append(workline.at(i));
     }
+
+    // qDebug() << "remote=" << remote;
+
     *local_file_name = fwcompiler::CompilerDriver::unescapeFileName(local);
     *remote_file_name = fwcompiler::CompilerDriver::unescapeFileName(remote);
 
     if (fwbdebug)
-        qDebug("local_name: '%s'  remote_name: '%s'  main_script: %d",
-               local_file_name->toAscii().constData(),
-               remote_file_name->toAscii().constData(),
-               *main_script);
+        qDebug() << "local_name:"
+                 << local_file_name->toAscii().constData()
+                 << "remote_name:"
+                 << remote_file_name->toAscii().constData()
+                 << "main_script:" 
+                 << *main_script;
 
     return true;
 }
@@ -373,6 +389,12 @@ void FirewallInstaller::packSCPArgs(const QString &local_name,
                                     const QString &remote_name, 
                                     QStringList &args)
 {
+    if (fwbdebug)
+        qDebug() << "packSCPArgs"
+                 << "local_name=" << local_name
+                 << "remote_name=" << remote_name;
+
+
     QString file_with_path = getFullPath(local_name);
     QString scp = st->getSCPPath();
 
@@ -476,28 +498,17 @@ void FirewallInstaller::packSCPArgs(const QString &local_name,
 
     if (!cnf->putty_session.isEmpty())
         target = QString("%1%2:%3").arg(user_spec)
-            .arg(cnf->putty_session).arg(fwcompiler::CompilerDriver::escapeFileName(remote_name));
+            .arg(cnf->putty_session)
+            .arg(fwcompiler::CompilerDriver::escapeFileName(remote_name));
     else
         target = QString("%1%2:%3").arg(user_spec)
-            .arg(mgmt_addr).arg(fwcompiler::CompilerDriver::escapeFileName(remote_name));
+            .arg(mgmt_addr)
+            .arg(fwcompiler::CompilerDriver::escapeFileName(remote_name));
 
     args.push_back(target);
 
-/*
-    if (!cnf->putty_session.isEmpty())
-    {
-        args.push_back(cnf->putty_session + ":" +
-                       fwcompiler::CompilerDriver::escapeFileName(remote_name));
-    } else
-    {
-        if (!cnf->user.isEmpty())
-            args.push_back(cnf->user + "@" + mgmt_addr + ":" +
-                           fwcompiler::CompilerDriver::escapeFileName(remote_name));
-        else
-            args.push_back(mgmt_addr + ":" +
-                           fwcompiler::CompilerDriver::escapeFileName(remote_name));
-    }
-*/
+    if (fwbdebug)
+        qDebug() << "args=" << args;
 }
 
 /*
@@ -590,8 +601,6 @@ void FirewallInstaller::executeExternalInstallScript(const QString &command,
     QStringList args;
     //args.push_back(command.trimmed());
 
-    args += script_args.trimmed().split(" ", QString::SkipEmptyParts);
-
     args.push_back("-f");
     args.push_back(db->getFileName().c_str());
 
@@ -600,6 +609,9 @@ void FirewallInstaller::executeExternalInstallScript(const QString &command,
         args.push_back("-d");
         args.push_back(wdir);
     }
+
+    args += script_args.trimmed().split(" ", QString::SkipEmptyParts);
+
     args.push_back(cnf->fwobj->getName().c_str());
 
     if (cnf->verbose) inst_dlg->displayCommand(args);
@@ -688,6 +700,8 @@ QString FirewallInstaller::getActivationCmd()
         return cnf->activationCmd;
     }
 
+    FWOptions *fwopt = cnf->fwobj->getOptionsObject();
+
     QString configlet_name = "installer_commands_";
     if (cnf->user=="root") configlet_name += "root";
     else                   configlet_name += "reg_user";
@@ -706,13 +720,31 @@ QString FirewallInstaller::getActivationCmd()
     Configlet configlet(host_os, os_family, configlet_name);
     configlet.removeComments();
     configlet.collapseEmptyStrings(true);
-    configlet.setVariable("test",  cnf->testRun);
-    configlet.setVariable("run", ! cnf->testRun);
-    configlet.setVariable("with_rollback",   cnf->rollback);
-    configlet.setVariable("no_rollback",   ! cnf->rollback);
+
+    // test run and rollback were deprecated in 4.2.0. On Linux, BSD
+    // and PIX rollback was implemented by rebooting firewall which is
+    // too heavy-handed and it did not work on BSD at all.
+    configlet.setVariable("test", false);
+    configlet.setVariable("run", true);
+    configlet.setVariable("with_rollback", false);
+    configlet.setVariable("no_rollback", true);
+
+    configlet.setVariable("firewall_name",
+                          QString::fromUtf8(cnf->fwobj->getName().c_str()));
+
     configlet.setVariable("with_compression",  cnf->compressScript);
     configlet.setVariable("no_compression",  ! cnf->compressScript);
 
+    // On FreeBSD where we can generate either shell script or rc.conf
+    // file, installation commands differ.
+    //
+    // TODO: find more generic way to do this so that GUI installer does not
+    // have to be aware of the differences in generated file format.
+    configlet.setVariable("rc_conf_format",
+                          fwopt->getBool("generate_rc_conf_file"));
+    configlet.setVariable("shell_script_format",
+                          ! fwopt->getBool("generate_rc_conf_file"));
+
     replaceMacrosInCommand(&configlet);
 
     return configlet.expand().trimmed();
@@ -744,10 +776,19 @@ void FirewallInstaller::replaceMacrosInCommand(Configlet *conf)
  * Unix.
  */
 
-    QString fwbscript = fwcompiler::CompilerDriver::escapeFileName(
+/*
+ * TODO: there must be a better place to fill cnd.fwscript than
+ * this.  All I need to do is fill it before calling summary() and
+ * before launching installer that uses it in
+ * FirewallInstaller::replaceMacrosInCommand()
+ */
+
+    QString fwscript = fwcompiler::CompilerDriver::escapeFileName(
         QFileInfo(cnf->remote_script).fileName());
 
-    if (fwbscript.indexOf(":")!=-1) fwbscript = fwbscript.section(':', 1, 1);
+    if (fwscript.indexOf(":")!=-1) fwscript = fwscript.section(':', 1, 1);
+
+    cnf->fwscript = fwscript;
 
     if (fwbdebug)
     {
@@ -755,56 +796,31 @@ void FirewallInstaller::replaceMacrosInCommand(Configlet *conf)
         qDebug() << "  $fwdir=" << cnf->fwdir;
         qDebug() << "  cnf->script=" << cnf->script;
         qDebug() << "  cnf->remote_script=" << cnf->remote_script;
-        qDebug() << "  $fwscript=" << fwbscript;
+        qDebug() << "  $fwscript=" << cnf->fwscript;
+        qDebug() << "  $firewall_name=" << QString::fromUtf8(
+            cnf->fwobj->getName().c_str());
     }
 
     conf->setVariable("fwbprompt", fwb_prompt);
     conf->setVariable("fwdir", cnf->fwdir);
-    conf->setVariable("fwscript", fwbscript);
-
-    conf->setVariable("rbtimeout", cnf->rollbackTime);
-    conf->setVariable("rbtimeout_sec", cnf->rollbackTime * 60);
+    conf->setVariable("fwscript", cnf->fwscript);
+    conf->setVariable("firewall_name",
+                      QString::fromUtf8(cnf->fwobj->getName().c_str()));
 }
 
 
 /*
- * Takes destination directory defined in the configlet (or XML
- * resource file) and substitutes {{$fwbdir}} macro with
- * @fwdir. Returned directory path always ends with separator ("/")
- *
- * Main purpose of this method is to get the right directory depending
- * on the setting of the "test install" option. In case of test
- * install we copy all files into a different directory and run them
- * from there. The directory is defined in the resource (or configlet)
- * file.
+ * Returned directory path always ends with separator ("/")
  */
 QString FirewallInstaller::getDestinationDir(const QString &fwdir)
 {
-    QString dir = "";
-
-    if (cnf->testRun)
-    {
-        string optpath = "activation/fwdir_test/";
-        dir = Resources::getTargetOptionStr(cnf->fwobj->getStr("host_OS"),
-                                            optpath).c_str();
-        // need to trim dir because it picks up '\n' and possibly spaces
-        // from XML element body text formatting
-        dir = dir.trimmed();
-    }
+    QString dir = fwdir;
 
     if (fwbdebug)
         qDebug() << "FirewallInstaller::getDestinationDir:  "
                  << "destination directory=" << dir
                  << "cnf->fwdir=" << cnf->fwdir;
     
-    // dir can contain macro %FWDIR% which should be replaced with cnf->fwdir
-    // empty dir is equivalent to just the value of cnf->fwdir
-
-    if (!dir.isEmpty())
-        dir.replace("{{$fwdir}}", fwdir);
-    else
-        dir = fwdir;
-
     if (!dir.endsWith(QDir::separator())) return dir + "/";
     return dir;
 }
@@ -815,6 +831,18 @@ QString FirewallInstaller::getGeneratedFileFullPath(Firewall *fw)
  * setting that user enters in the "Compiler" tab of fw advanced
  * dialog can be either local or absolute path. 
  */
+    QString generated_file = getGeneratedFileName(fw);
+    QFileInfo gen_file_info(generated_file);
+    if (!gen_file_info.isAbsolute())
+    {
+        QFileInfo fwb_file_info = QFileInfo(mw->getRCS()->getFileName());
+        generated_file = fwb_file_info.dir().path() + "/" + generated_file;
+    }
+    return QDir::toNativeSeparators(generated_file);
+}
+ 
+QString FirewallInstaller::getGeneratedFileName(Firewall *fw)
+{
     FWOptions  *fwopt = fw->getOptionsObject();
     QString generated_file;
     QString ofname = QString::fromUtf8(fwopt->getStr("output_file").c_str());
@@ -823,14 +851,7 @@ QString FirewallInstaller::getGeneratedFileFullPath(Firewall *fw)
         generated_file = ofname;
     } else
         generated_file = QString::fromUtf8(fw->getName().c_str()) + ".fw";
-
-    QFileInfo gen_file_info(generated_file);
-    if (!gen_file_info.isAbsolute())
-    {
-        QFileInfo fwb_file_info = QFileInfo(mw->getRCS()->getFileName());
-        generated_file = fwb_file_info.dir().path() + "/" + generated_file;
-    }
-    return QDir::toNativeSeparators(generated_file);
+    return generated_file;
 }
  
 void FirewallInstaller::terminate()
