@@ -74,6 +74,8 @@ using namespace libfwbuilder;
 InterfaceDialog::InterfaceDialog(QWidget *parent) :
         BaseObjectDialog(parent)
 {
+    netzone_manager = new NetworkZoneManager();
+
     m_dialog = new Ui::InterfaceDialog_q;
     m_dialog->setupUi(this);
 /*
@@ -85,6 +87,7 @@ InterfaceDialog::InterfaceDialog(QWidget *parent) :
 
 InterfaceDialog::~InterfaceDialog()
 {
+    delete netzone_manager;
     delete m_dialog;
 }
 
@@ -101,27 +104,30 @@ void InterfaceDialog::loadFWObject(FWObject *o)
 
     init = true;
 
-    /*
-     * guessSubInterfaceTypeAndAttributes() changes some attributes of
-     * the object (mostly compensating for missing functions in
-     * auto-upgrade scripts but also makes some guesses based on the
-     * interface name, such as sets its vlan ID if its name looks like
-     * it might be a vlan interface). Since we make changes in the
-     * object here, do it before loading it into the dialog so that it
-     * does not look like it has changed in applyChanges() even if the
-     * user hasn't touched it, which causes new undo command to be
-     * created out of nowhere.
-     *
-     * TODO: better way of course is to call
-     * guessSubInterfaceTypeAndAttributes in places where user changes
-     * something relevant in the interface to complement their changes
-     * and right after the interface has been created.
-     */
-    interfaceProperties *int_prop =
-        interfacePropertiesObjectFactory::getInterfacePropertiesObject(
-            s->getParentHost());
-    int_prop->guessSubInterfaceTypeAndAttributes(s);
-    delete int_prop;
+    if (st->getBool("Objects/Interface/autoconfigureInterfaces"))
+    {
+        /*
+         * guessSubInterfaceTypeAndAttributes() changes some attributes of
+         * the object (mostly compensating for missing functions in
+         * auto-upgrade scripts but also makes some guesses based on the
+         * interface name, such as sets its vlan ID if its name looks like
+         * it might be a vlan interface). Since we make changes in the
+         * object here, do it before loading it into the dialog so that it
+         * does not look like it has changed in applyChanges() even if the
+         * user hasn't touched it, which causes new undo command to be
+         * created out of nowhere.
+         *
+         * TODO: better way of course is to call
+         * guessSubInterfaceTypeAndAttributes in places where user changes
+         * something relevant in the interface to complement their changes
+         * and right after the interface has been created.
+         */
+        interfaceProperties *int_prop =
+            interfacePropertiesObjectFactory::getInterfacePropertiesObject(
+                s->getParentHost());
+        int_prop->guessSubInterfaceTypeAndAttributes(s);
+        delete int_prop;
+    }
 
     m_dialog->obj_name->setText( QString::fromUtf8(s->getName().c_str()) );
     m_dialog->label->setText( QString::fromUtf8(s->getLabel().c_str()) );
@@ -290,81 +296,12 @@ void InterfaceDialog::loadFWObject(FWObject *o)
             m_dialog->netzone->setEnabled(true);
             m_dialog->netzoneLabel->setEnabled(true);
 
-            netzoneObjectIDs.clear();
-            netzoneObjectNos.clear();
-
-            QStringList netzoneObjectNames;
-
-            int n = 0;
-
-            netzoneObjectIDs[0] = n;
-            netzoneObjectNos[n] = 0;
-            netzoneObjectNames.push_back(" None ");
-
-            ++n;
-
-            netzoneObjectIDs[FWObjectDatabase::ANY_ADDRESS_ID] = n;
-            netzoneObjectNos[n] = FWObjectDatabase::ANY_ADDRESS_ID;
-            netzoneObjectNames.push_back(" Any ");
-
-            ++n;
-
-/* TODO: try to make this widget show object with appropriate icon */
-
-            list<FWObject*> libs = m_project->db()->getByType( Library::TYPENAME );
-            for (list<FWObject*>::iterator l=libs.begin(); l!=libs.end(); ++l)
-            {
-                FWObject *library= *l;
-                FWObject *o1,*o2;
-
-                if ( library->getId()==FWObjectDatabase::DELETED_OBJECTS_ID ) continue;
-
-                o1=library->findObjectByName(ObjectGroup::TYPENAME,"Objects");
-                assert(o1!=NULL);
-                o2=o1->findObjectByName(ObjectGroup::TYPENAME,"Groups");
-                if (o2==NULL)
-                {
-                    if (fwbdebug)
-                        qDebug("InterfaceDialog::loadFWObject  missing Groups group in %s", FWObjectDatabase::getStringId(o1->getId()).c_str());
-                    continue;
-                }
-//                assert(o2!=NULL);
-
-                for (FWObject::iterator i=o2->begin(); i!=o2->end(); ++i)
-                {
-                    netzoneObjectIDs[(*i)->getId()] = n;
-                    netzoneObjectNos[n] =(*i)->getId();
-                    netzoneObjectNames.push_back(
-                        tr("Group: ")+ (*i)->getName().c_str() );
-                    ++n;
-                }
-
-                o2=o1->findObjectByName(ObjectGroup::TYPENAME,"Networks");
-                if (o2==NULL)
-                {
-                    if (fwbdebug)
-                        qDebug("InterfaceDialog::loadFWObject  missing Networks group in %s",
-                               FWObjectDatabase::getStringId(o1->getId()).c_str());
-                    continue;
-                }
-//                assert(o2!=NULL);
-
-                for (FWObject::iterator i1=o2->begin(); i1!=o2->end(); ++i1)
-                {
-                    netzoneObjectIDs[(*i1)->getId()] = n;
-                    netzoneObjectNos[n] = (*i1)->getId();
-                    netzoneObjectNames.push_back(
-                        tr("Network: ")+ (*i1)->getName().c_str() );
-                    ++n;
-                }
-            }
-
-            m_dialog->netzone->clear();
-            m_dialog->netzone->addItems( netzoneObjectNames );
+            netzone_manager->load(m_project->db());
 
             int id = FWObjectDatabase::getIntId(obj->getStr("network_zone"));
             if (id==-1) id = 0;
-            m_dialog->netzone->setCurrentIndex( netzoneObjectIDs[id] );
+
+            netzone_manager->packComboBox(m_dialog->netzone, id);
         }
         else
         {
@@ -419,7 +356,9 @@ void InterfaceDialog::validate(bool *res)
         return;
     }
 
-    if ( ! int_prop->validateInterface(obj->getParent(), obj_name, err))
+    // validateInterface() checks validity of vlan inetrface name
+    if (st->getBool("Objects/Interface/autoconfigureInterfaces") &&
+        ! int_prop->validateInterface(obj->getParent(), obj_name, err))
     {
         /*
          * Here is the annoying part: if user entered interface name
@@ -517,11 +456,14 @@ void InterfaceDialog::applyChanges()
     try 
     {
         supports_security_levels=
-            Resources::getTargetCapabilityBool(f->getStr("platform"), "security_levels");
+            Resources::getTargetCapabilityBool(f->getStr("platform"),
+                                               "security_levels");
         supports_network_zones=
-            Resources::getTargetCapabilityBool(f->getStr("platform"), "network_zones");
+            Resources::getTargetCapabilityBool(f->getStr("platform"),
+                                               "network_zones");
         supports_unprotected =
-            Resources::getTargetCapabilityBool(f->getStr("platform"), "unprotected_interfaces");
+            Resources::getTargetCapabilityBool(f->getStr("platform"),
+                                               "unprotected_interfaces");
     } catch (FWException &ex)  { }
 
 
@@ -534,10 +476,20 @@ void InterfaceDialog::applyChanges()
             new_state->setBool("unprotected",  m_dialog->unprotected->isChecked() );
 
         if (supports_network_zones)
+        {
             new_state->setStr("network_zone",
                         FWObjectDatabase::getStringId(
-                            netzoneObjectNos[
-                                m_dialog->netzone->currentIndex() ]));
+                            m_dialog->netzone->itemData(
+                                m_dialog->netzone->currentIndex(),
+                                Qt::UserRole).toInt()));
+
+            // new_state->setStr("network_zone",
+            //             FWObjectDatabase::getStringId(
+            //                 netzone_manager->getNetzoneIdByListIndex(
+            //                     m_dialog->netzone->currentIndex() )
+            //             )
+            // );
+        }
 
         intf->setManagement( m_dialog->management->isChecked() );
     }
@@ -547,13 +499,17 @@ void InterfaceDialog::applyChanges()
         // Complement changes made by the user with our guesses, but
         // do this only if user changed something.
 
-        // ticket #328: automatically assign vlan id to interface based on
-        // interface name
-        interfaceProperties *int_prop =
-            interfacePropertiesObjectFactory::getInterfacePropertiesObject(
-                Interface::cast(obj)->getParentHost());
-        int_prop->guessSubInterfaceTypeAndAttributes(intf);
-        delete int_prop;
+        if (st->getBool("Objects/Interface/autoconfigureInterfaces"))
+        {
+            // ticket #328: automatically assign vlan id to interface based on
+            // interface name
+            interfaceProperties *int_prop =
+                interfacePropertiesObjectFactory::getInterfacePropertiesObject(
+                    Interface::cast(obj)->getParentHost());
+            int_prop->setPerformVlanChecks(true);
+            int_prop->guessSubInterfaceTypeAndAttributes(intf);
+            delete int_prop;
+        }
 
         if (obj->isReadOnly()) return;
         m_project->undoStack->push(cmd.release());
