@@ -151,15 +151,6 @@ vector<QTreeWidget*> ObjectManipulator::getTreeWidgets()
     return res;
 }
 
-/*
- * This method decides what should be shown in the column #1 of the
- * tree for the given object
- */
-QString ObjectManipulator::getTreeLabel( FWObject *obj )
-{
-    return FWObjectPropertiesFactory::getObjectPropertiesBrief(obj);
-}
-
 void ObjectManipulator::showDeletedObjects(bool f)
 {
     try
@@ -372,8 +363,25 @@ void ObjectManipulator::contextMenuRequested(const QPoint &pos)
 
     popup_menu->clear();
 
-    QAction *edtID =
-        popup_menu->addAction(tr("Edit"), this, SLOT( editSelectedObject()));
+    if (item->childCount() > 0)
+    {
+        if (item->isExpanded())
+            popup_menu->addAction(tr("Collapse"), this,
+                                  SLOT(collapseCurrentTreeNode()));
+        else
+            popup_menu->addAction(tr("Expand"), this,
+                                  SLOT(expandCurrentTreeNode()));
+        popup_menu->addSeparator();
+    }
+
+    QAction *edtID;
+
+    if (currentObj->isReadOnly())
+        edtID = popup_menu->addAction(tr("Inspect"),
+                                      this, SLOT( editSelectedObject()));
+    else
+        edtID = popup_menu->addAction(tr("Edit"),
+                                      this, SLOT( editSelectedObject()));
 
     if (RuleSet::cast(currentObj))
         popup_menu->addAction(tr("Open"), this, SLOT( openSelectedRuleSet()));
@@ -743,13 +751,17 @@ void ObjectManipulator::contextMenuRequested(const QPoint &pos)
                   delMenuItem, newMenuItem, inDeletedObjects);
 
     if (dupID) dupID->setEnabled(dupMenuItem);
-//    movetargets->setEnabled(moveMenuItem);
+
     copyID->setEnabled(copyMenuItem);
     pasteID->setEnabled(pasteMenuItem);
-
+    
     cutID->setEnabled(delMenuItem);
     delID->setEnabled(delMenuItem);
 
+    // can not move object if can not delete it
+    if (movetargets)
+        movetargets->setEnabled(delMenuItem);
+
     QList<QAction*>::iterator iter;
     for (iter=AddObjectActions.begin(); iter!=AddObjectActions.end(); iter++)
         (*iter)->setEnabled(newMenuItem);
@@ -761,6 +773,8 @@ void ObjectManipulator::contextMenuRequested(const QPoint &pos)
 
 bool ObjectManipulator::getDeleteMenuState(FWObject *obj)
 {
+    if (obj->isReadOnly()) return false;
+
     QString objPath = obj->getPath(true).c_str();
     bool del_menu_item_state = FWBTree().getDeleteMenuState(objPath);
 
@@ -787,8 +801,6 @@ void ObjectManipulator::getMenuState(bool haveMoveTargets,
                                      bool &newMenuItem,
                                      bool &inDeletedObjects)
 {
-    if (fwbdebug) qDebug("ObjectManipulator::getMenuState");
-
     inDeletedObjects = false;
 
     if (m_project->db() == NULL)
@@ -812,12 +824,8 @@ void ObjectManipulator::getMenuState(bool haveMoveTargets,
     FWObject *del_obj_library =
         m_project->db()->findInIndex( FWObjectDatabase::DELETED_OBJECTS_ID);
 
-    if (fwbdebug) qDebug() << "ObjectManipulator::getMenuState del_obj_library=" << del_obj_library;
-
     FWObject *current_library = getCurrentLib();
 
-    if (fwbdebug) qDebug() << "ObjectManipulator::getMenuState current_library=" << current_library;
-
     if (getCurrentObjectTree()==NULL) return;
 
     // delete, cut and copy menu items will be enabled only if all
@@ -830,12 +838,6 @@ void ObjectManipulator::getMenuState(bool haveMoveTargets,
     {
         FWObject *obj= *i;
 
-        if (fwbdebug)
-            qDebug() << "ObjectManipulator::getMenuState"
-                     << "obj=" << obj
-                     << "root=" << obj->getRoot()
-                     << "current_library root=" << current_library->getRoot();
-
         QString object_path = obj->getPath(true).c_str();
 
         if (parent == NULL) parent = obj->getParent();
@@ -928,7 +930,6 @@ void ObjectManipulator::getMenuState(bool haveMoveTargets,
             newMenuItem = false;
 
     }
-    if (fwbdebug) qDebug("ObjectManipulator::getMenuState done");
 }
 
 bool ObjectManipulator::isCurrentObjectLockable()
@@ -1244,29 +1245,12 @@ void ObjectManipulator::showObjectInTree(ObjectTreeViewItem *otvi)
     m_objectManipulator->widgetStack->setCurrentWidget(otv);
 
     otvi->getTree()->clearSelection();
-    otvi->getTree()->scrollToItem(otvi,
-                                  QAbstractItemView::PositionAtTop); // QAbstractItemView::EnsureVisible );
-    otvi->getTree()->setCurrentItem( otvi );
-    otvi->setSelected( true );
+    otvi->getTree()->scrollToItem(otvi);
+    otvi->getTree()->setCurrentItem(otvi);
+    otvi->setSelected(true);
     otvi->getTree()->setFocus(Qt::OtherFocusReason);
 }
 
-void ObjectManipulator::expandObjectInTree(FWObject *obj)
-{
-    FWObject *o = obj;
-    if (FWReference::cast(o)!=NULL) o = FWReference::cast(o)->getPointer();
-
-    QTreeWidgetItem *it = allItems[o];
-    if (it==NULL) return;
-    it->setExpanded(true);
-
-    for (list<FWObject*>::iterator i=o->begin();  i!=o->end(); ++i)
-    {
-        FWObject *o1 = *i;
-        if (o1 && o1->size() > 0) expandObjectInTree(o1);
-    }
-}
-
 void ObjectManipulator::libChangedById(int obj_id)
 {
     for (int i=0; i<libs_model->rowCount(); ++i)
@@ -1369,11 +1353,6 @@ FWObject* ObjectManipulator::getCurrentLib()
 
     FWObject *lib = libs_model->getLibrary(idx);
 
-    if (fwbdebug)
-    {
-        qDebug("ObjectManipulator::getCurrentLib(): idx=%d  lib=%p", idx, lib);
-    }
-
     return lib;
 }
 
