@@ -380,7 +380,10 @@ FWObject* ObjectManipulator::actuallyCreateObject(FWObject *parent,
     assert(nobj!=NULL);
     if (copyFrom!=NULL) nobj->duplicate(copyFrom, true);
     if (nobj->isReadOnly()) nobj->setReadOnly(false);
-    nobj->setName( string(objName.toUtf8().constData()) );
+
+    QString new_name = makeNameUnique(parent, objName, objType);
+
+    nobj->setName( string(new_name.toUtf8().constData()) );
 
     if (objType == DNSName::TYPENAME)
     {
@@ -501,6 +504,7 @@ FWObject* ObjectManipulator::newFirewall(QUndoCommand* macro)
         FWCmdAddObject *cmd = new FWCmdAddObject(
             m_project, parent, NULL, QObject::tr("Create new Firewall"), macro);
         FWObject *new_state = cmd->getNewState();
+
         parent->remove(nfw, false);
         new_state->add(nfw);
     }
