@@ -503,12 +503,15 @@ void ObjectManipulator::deleteObject(FWObject *obj, QUndoCommand* macro)
     if (object_library->getId() == FWObjectDatabase::STANDARD_LIB_ID)
         return;
 
+    if (obj->isReadOnly()) return;
+
     if (obj->getId() == FWObjectDatabase::STANDARD_LIB_ID ||
         obj->getId() == FWObjectDatabase::DELETED_OBJECTS_ID) return;
     
     bool is_library = Library::isA(obj);
     bool is_firewall = Firewall::cast(obj) != NULL; // includes Cluster too
     bool is_deleted_object = (deleted_objects_lib!=NULL && obj->isChildOf(deleted_objects_lib));
+
     // ruleset_visible == true if 1) we delete firewall object and one of its
     // rulesets is visible in the project panel, or 2) we delete ruleset object
     // which is visible in the project panel
@@ -561,7 +564,8 @@ void ObjectManipulator::deleteObject(FWObject *obj, QUndoCommand* macro)
     catch (FWException &ex)
     {
         if (fwbdebug)
-            qDebug("ObjectManipulator::deleteObject: catch:  restoreOverrideCursor");
+            qDebug() << "ObjectManipulator::deleteObject:"
+                     << "catch:  restoreOverrideCursor";
         QApplication::restoreOverrideCursor();
         QMessageBox::warning(
             this,"Firewall Builder",
@@ -583,7 +587,8 @@ void ObjectManipulator::deleteObject(FWObject *obj, QUndoCommand* macro)
 void ObjectManipulator::actuallyDeleteObject(FWObject *obj, QUndoCommand* macro)
 {
     map<int, set<FWObject*> > reference_holders;
-    UsageResolver().findAllReferenceHolders(obj, m_project->db(), reference_holders);
+    UsageResolver().findAllReferenceHolders(obj, m_project->db(),
+                                            reference_holders);
 
     FWObject *deleted_objects_lib = m_project->db()->findInIndex(
         FWObjectDatabase::DELETED_OBJECTS_ID);
