@@ -83,6 +83,9 @@ ObjectTreeView::ObjectTreeView(ProjectPanel* project,
     setObjectName(name);
     this->setParent(parent, f);
     setFont(st->getTreeFont());
+
+    setExpandsOnDoubleClick(false);
+
 //    setAcceptDrops( TRUE );
     item_before_drag_started=NULL;
     lastSelected = NULL;
@@ -129,7 +132,7 @@ ObjectTreeView::ObjectTreeView(ProjectPanel* project,
 
     //header()->hide();
 
-    header()->setDefaultAlignment(Qt::AlignHCenter);
+    header()->setDefaultAlignment(Qt::AlignLeft);
     header()->setResizeMode(QHeaderView::Interactive);
 
     showOrHideAttributesColumn();
@@ -410,16 +413,6 @@ void ObjectTreeView::startDrag(Qt::DropActions supportedActions)
 
     FWObject *edit_obj = mw->getOpenedEditor();
 
-    if (mw->isEditorVisible() &&
-        dragobj.size()==1 &&
-        edit_obj!=NULL &&
-        current_obj->getLibrary()==edit_obj->getLibrary() )
-    {
-        if (fwbdebug) qDebug("ObjectTreeView::dragObject() reset selection");
-        otvi->setSelected(false);
-        resetSelection();
-    }
-
     if (fwbdebug) qDebug("ObjectTreeView::dragObject()  returns !NULL");
 
     drag->start(supportedActions);
@@ -461,6 +454,7 @@ void ObjectTreeView::dragMoveEvent( QDragMoveEvent *ev)
         ev->setAccepted(false);
         return;
     }
+
     list<FWObject*> dragol;
     if (!FWObjectDrag::decode(ev, dragol))
         ev->setAccepted(false);
@@ -475,6 +469,13 @@ void ObjectTreeView::dragMoveEvent( QDragMoveEvent *ev)
             ev->setAccepted(false);
             return;
         }
+
+        // see #1976 do not allow pasting object that has been deleted
+        if (dragobj->getLibrary()->getId() == FWObjectDatabase::DELETED_OBJECTS_ID)
+        {
+            ev->setAccepted(false);
+            return;
+        }
     }
 
     ev->setAccepted(true);
@@ -739,6 +740,9 @@ void ObjectTreeView::updateAfterPrefEdit()
 
 void ObjectTreeView::ExpandTreeItems(const set<int> &ids)
 {
+    if (fwbdebug)
+        qDebug() << "ObjectTreeView::ExpandTreeItems()";
+
     QTreeWidgetItemIterator it(this);
     while ( *it )
     {
