@@ -104,14 +104,6 @@ void ProjectPanel::initMain(FWWindow *main)
 
     main->undoGroup->addStack(undoStack);
 
-    connect(m_panel->treeDockWidget, SIGNAL(topLevelChanged(bool)),
-            this, SLOT(topLevelChangedForTreePanel(bool)));
-    connect(m_panel->treeDockWidget, SIGNAL(visibilityChanged(bool)),
-            this, SLOT(visibilityChangedForTreePanel(bool)));
-
-//    connect(m_panel->bottomDockWidget, SIGNAL(topLevelChanged(bool)),
-//            this, SLOT(topLevelChangedForBottomPanel(bool)));
-
     fd  = new findDialog(this, this);
     fd->hide();
 
@@ -166,14 +158,19 @@ ProjectPanel::ProjectPanel(QWidget *parent):
 
     connect(m_panel->topSplitter, SIGNAL(splitterMoved(int,int)),
             this, SLOT(splitterPositionChanged(int,int)));
+
 }
 
 ProjectPanel::~ProjectPanel()
 {
+    if (fwbdebug) qDebug() << "ProjectPanel::~ProjectPanel()";
+
     undoStack->clear();
     if (rcs) delete rcs;
     if (objdb) delete objdb;
     delete m_panel;
+
+    if (fwbdebug) qDebug() << "ProjectPanel::~ProjectPanel() done";
 }
 
 QString ProjectPanel::getPageTitle(bool file_path)
@@ -196,22 +193,6 @@ QString ProjectPanel::getPageTitle(bool file_path)
     else return default_caption;
 }
 
-RuleElement* ProjectPanel::getRE(Rule* r, int col)
-{
-    string ret;
-    switch (col)
-    {
-        case 0: ret=RuleElementSrc::TYPENAME; break;//Object
-        case 1: ret=RuleElementDst::TYPENAME; break;//Object
-        case 2: ret=RuleElementSrv::TYPENAME; break;//Object
-        case 3: ret=RuleElementItf::TYPENAME; break;//Object
-        case 4: ret=RuleElementInterval::TYPENAME; break;//Time
-        default: return NULL;
-    }
-
-    return RuleElement::cast( r->getFirstByType(ret) );
-}
-
 void ProjectPanel::restoreRuleSetTab()
 {
     if (fwbdebug) qDebug("ProjectPanel::()");
@@ -741,7 +722,7 @@ RCS * ProjectPanel::getRCS()
  */
 void ProjectPanel::addRule()
 {
-    if (visibleRuleSet==NULL) return ;
+    if (visibleRuleSet==NULL || getCurrentRuleSetView()==NULL) return ;
     getCurrentRuleSetView()->insertRule();
 }
 
@@ -936,7 +917,18 @@ void ProjectPanel::inspect(set<Firewall *> fws)
     foreach(Firewall *fw, fws)
     {
         if (first_fw == NULL) first_fw = fw;
+
+        /*
+         * get full path to the generated file. The path is built from
+         * the file name returned by
+         * FirewallInstaller::getGeneratedFileName() and directory
+         * path from the .fwb file. Note that we use the same
+         * algorithm when GUI launches policy compiler, except there
+         * the path is passed to it via "-d" command line option.
+         */
         QString mainFile = FirewallInstaller::getGeneratedFileFullPath(fw);
+//        QString mainFile = FirewallInstaller::getGeneratedFileName(fw);
+
         if (QFile::exists(mainFile))
         {
             instConf cnf;
@@ -1033,16 +1025,28 @@ void ProjectPanel::redrawRuleSets()
     reopenFirewall();
 }
 
+void ProjectPanel::aboutToActivate()
+{
+    if (fwbdebug) qDebug() << "ProjectPanel::aboutToActivate " << this;
+}
+
 void ProjectPanel::showEvent(QShowEvent *ev)
 {
     if (fwbdebug) qDebug() << "ProjectPanel::showEvent " << this
                            << "title " << mdiWindow->windowTitle();
 
+    connect(m_panel->treeDockWidget, SIGNAL(topLevelChanged(bool)),
+            this, SLOT(topLevelChangedForTreePanel(bool)));
+    connect(m_panel->treeDockWidget, SIGNAL(visibilityChanged(bool)),
+            this, SLOT(visibilityChangedForTreePanel(bool)));
+
     m_panel->treeDockWidget->raise();
     QWidget::showEvent(ev);
 
-    // we get this event when subsindow is maximized or restored
-    loadState();
+    // we get this event when MDI window is maximized or restored
+    // loadState();
+
+    // visibilityChangedForTreePanel(true);
 }
 
 void ProjectPanel::hideEvent(QHideEvent *ev)
@@ -1064,11 +1068,19 @@ void ProjectPanel::closeEvent(QCloseEvent * ev)
         return;
     }
 
+    disconnect(m_panel->treeDockWidget, SIGNAL(topLevelChanged(bool)),
+               this, SLOT(topLevelChangedForTreePanel(bool)));
+    disconnect(m_panel->treeDockWidget, SIGNAL(visibilityChanged(bool)),
+               this, SLOT(visibilityChangedForTreePanel(bool)));
+
     saveState();
     fileClose();
 
     mw->updateWindowTitle();
 
+    //QCoreApplication::postEvent(mw, new closeEditorPanelEvent());
+    QCoreApplication::postEvent(mw, new clearEditorPanelEvent());
+
     QTimer::singleShot( 0, mw, SLOT(projectWindowClosed()) );
 }
 
@@ -1107,7 +1119,9 @@ void ProjectPanel::registerObjectToUpdateInTree(FWObject *o, bool update_subtree
         qDebug() << "ProjectPanel::registerObjectToUpdateInTree()"
                  << "o=" << o->getName().c_str()
                  << "update_subtree=" << update_subtree
-                 << "updateObjectsInTreePool.size()=" << updateObjectsInTreePool.size();
+                 << "updateObjectsInTreePool.size()="
+                 << updateObjectsInTreePool.size();
+
     if (updateObjectsInTreePool.find(o->getId()) == updateObjectsInTreePool.end())
     {
         updateObjectsInTreePool[o->getId()] = update_subtree;
@@ -1119,7 +1133,8 @@ void ProjectPanel::updateObjectInTree()
 {
     if (fwbdebug)
         qDebug() << "ProjectPanel::updateObjectInTree()"
-                 << "updateObjectsInTreePool.size()=" << updateObjectsInTreePool.size();
+                 << "updateObjectsInTreePool.size()="
+                 << updateObjectsInTreePool.size();
 
     while (updateObjectsInTreePool.size() > 0)
     {
@@ -1135,12 +1150,42 @@ void ProjectPanel::registerModifiedObject(FWObject *o)
 {
     if (fwbdebug)
         qDebug() << "ProjectPanel::registerModifiedObject "
-                 << "lastModifiedTimestampChangePool.size()=" << lastModifiedTimestampChangePool.size()
-                 << "o=" << o->getName().c_str();
-    if (lastModifiedTimestampChangePool.find(o->getId()) == lastModifiedTimestampChangePool.end())
+                 << "lastModifiedTimestampChangePool.size()="
+                 << lastModifiedTimestampChangePool.size()
+                 << "o=" << o->getName().c_str()
+                 << "(" << o->getTypeName().c_str() << ")"
+                 << "id=" << o->getId();
+
+    FWObject *modified_object = o;
+
+    /*
+     *  a bit of optimization: the purpose of registering modified
+     *  object here is to update "last modified" timestamp in the
+     *  firewall object it belongs to. One of the frequent cases is
+     *  when @o is rule element because user made some change to
+     *  it. Massive find and replace operations can cause waves of
+     *  registrations of rule elements, all of which belong to the
+     *  same rule set. If I register rule set instead, there will be
+     *  just one object to register.
+     */
+
+    if (RuleElement::cast(o))
     {
-        lastModifiedTimestampChangePool.insert(o->getId());
-        QTimer::singleShot(0, this, SLOT(updateLastModifiedTimestampForAllFirewalls()));
+        while (RuleSet::cast(modified_object) == NULL)
+            modified_object = modified_object->getParent();
+    }
+
+    if (lastModifiedTimestampChangePool.find(modified_object->getId()) ==
+        lastModifiedTimestampChangePool.end())
+    {
+        if (fwbdebug)
+            qDebug() << "ProjectPanel::registerModifiedObject "
+                     << "Add object" << modified_object->getName().c_str()
+                     << "id=" << modified_object->getId();
+                     
+        lastModifiedTimestampChangePool.insert(modified_object->getId());
+        QTimer::singleShot(
+            0, this, SLOT(updateLastModifiedTimestampForAllFirewalls()));
     }
 }
 
@@ -1153,7 +1198,8 @@ void ProjectPanel::updateLastModifiedTimestampForAllFirewalls()
 
     if (lastModifiedTimestampChangePool.size() == 0) return;
 
-    mw->showStatusBarMessage(tr("Searching for firewalls affected by the change..."));
+    mw->showStatusBarMessage(
+        tr("Searching for firewalls affected by the change..."));
 
     //QApplication::processEvents(QEventLoop::ExcludeUserInputEvents,100);
 
@@ -1228,6 +1274,31 @@ void ProjectPanel::toggleViewTree(bool f)
     else m_panel->treeDockWidget->hide();
 }
 
+
+/* Make the objectTree dock zero width if the objectTree is floating,
+   or if it's not visible.  Then the rules editor will take up the
+   full width */
+void ProjectPanel::adjustDockWidths(bool makeZeroWidth)
+{
+    /* These can be static since they are the same for every window
+       (they come from the .ui file) */
+    static int dockMinSize = 0;
+    static int dockMaxSize = 0;
+    if (dockMinSize == 0 && dockMaxSize == 0) {
+        dockMinSize = m_panel->treeDockWidgetParentFrame->minimumWidth();
+        dockMaxSize = m_panel->treeDockWidgetParentFrame->maximumWidth();
+    }
+
+    if (makeZeroWidth) {
+        m_panel->treeDockWidgetParentFrame->setMinimumWidth(0);
+        m_panel->treeDockWidgetParentFrame->setMaximumWidth(0);
+    } else {
+        m_panel->treeDockWidgetParentFrame->setMinimumWidth(dockMinSize);
+        m_panel->treeDockWidgetParentFrame->setMaximumWidth(dockMaxSize);
+    }
+}
+
+
 /*
  * Signal QDockWidget::topLevelChanged is called after dock widget
  * is made floating or docked.
@@ -1248,6 +1319,8 @@ void ProjectPanel::topLevelChangedForTreePanel(bool f)
      * is docked.
      */
 
+    adjustDockWidths(f);
+
     m_panel->treeDockWidget->blockSignals(true);
 
     if (f)  // window becomes detached
@@ -1278,11 +1351,16 @@ void ProjectPanel::topLevelChangedForTreePanel(bool f)
     m_panel->treeDockWidget->setFloating(f);
 
     if (fwbdebug)
-        qDebug() << "ProjectPanel::topLevelChangedForTreePanel check 1";
-
-    if (!m_panel->treeDockWidget->isWindow())
     {
-#if QT_VERSION > 0x040500
+        qDebug() << "ProjectPanel::topLevelChangedForTreePanel check 1";
+        qDebug() << "m_panel->treeDockWidget->isWindow()="
+                 << m_panel->treeDockWidget->isWindow();
+    }
+
+    if ( ! m_panel->treeDockWidget->isWindow())
+    {
+#if QT_VERSION >= 0x040500
+        if (fwbdebug) qDebug() << "Calling loadMainSplitter()";
         loadMainSplitter();
 #endif
     } else
@@ -1305,6 +1383,10 @@ void ProjectPanel::visibilityChangedForTreePanel(bool f)
                  << "isWindow()=" << m_panel->treeDockWidget->isWindow()
                  << "mdiWindow->isMaximized()=" << mdiWindow->isMaximized();
 
+    adjustDockWidths(!m_panel->treeDockWidget->isVisible() ||
+                     m_panel->treeDockWidget->isFloating());
+
+#if 0
     if (m_panel->treeDockWidget->isVisible() &&
         ! m_panel->treeDockWidget->isWindow())  // visible and not floating
     {
@@ -1315,7 +1397,7 @@ void ProjectPanel::visibilityChangedForTreePanel(bool f)
         collapseTree();
         m_panel->treeDockWidget->widget()->update();
     }
-
+#endif
 }
 
 void ProjectPanel::setActive()
