@@ -631,7 +631,9 @@ void RuleSetModel::restoreRules(QList<Rule*> rules, bool topLevel)
     {
         Rule* rule = rules.first();
         pivotRule = ruleset->getRuleByNum(0);
+
         ruleset->insert_before(pivotRule, rule);
+
         pivotIndex = index(pivotRule, 0);
 
         if (topLevel && pivotIndex.parent().isValid())
@@ -1243,18 +1245,31 @@ int RuleSetModel::getRulePosition(QModelIndex index)
 
 void RuleSetModel::objectChanged(FWObject* object)
 {
+/*
+ * See #2373
+ *
+ * Signal dataChanged() is connected to the slot
+ * RuleSetView::updateAllColumnsSize() that just calls resizeColumns()
+ * and does not do anything different depending on the model
+ * index. Can just call it once using different signal.
+
     QModelIndexList relatedIndexes = findObject(object);
 
+    if (fwbdebug)
+        qDebug() << "RuleSetModel::objectChanged"
+                 << "object=" << object->getName().c_str()
+                 << "relatedIndexes.size()=" << relatedIndexes.size();
+
     foreach(QModelIndex index, relatedIndexes)
     {
         emit dataChanged(index, index);
     }
+*/
+    emit layoutChanged();
 }
 
-QModelIndexList RuleSetModel::findObject (FWObject* object)
+QModelIndexList RuleSetModel::findObject(FWObject* object)
 {
-//    qDebug() << "RuleSetModel::findObject (FWObject* object)";
-//    qDebug() << "object:" << ((object)?QString::fromUtf8(object->getName().c_str()):"null");
     QModelIndexList list;
 
     RuleSetModelIterator it = begin();
@@ -1267,13 +1282,11 @@ QModelIndexList RuleSetModel::findObject (FWObject* object)
 
         if (node->type == RuleNode::Group)
         {
-//            qDebug() << "Group: " << node->name;
             ++it;
             continue;
         }
 
         Rule* rule = node->rule;
-//        qDebug() << "Rule " << rule->getPosition();
 
         // iterate through columns
         int column = 0;
@@ -1284,7 +1297,8 @@ QModelIndexList RuleSetModel::findObject (FWObject* object)
             if (colDesc.type == ColDesc::Object || colDesc.type == ColDesc::Time)
             {
                 // try to find the object
-                RuleElement* re = getRuleElementByRole(rule, colDesc.origin.toStdString());
+                RuleElement* re = getRuleElementByRole(
+                    rule, colDesc.origin.toStdString());
                 if (re->isAny()) continue;
 
                 for (FWObject::iterator i=re->begin(); i!=re->end(); i++)
@@ -1311,14 +1325,12 @@ QModelIndexList RuleSetModel::findObject (FWObject* object)
                         if (pr->getBranch() == object)
                         {
                             list.append(this->index(rule, column));
-//                            qDebug() << "Branch column:" << column;
                         }
                     } else if (pr->getAction() == PolicyRule::Tag)
                     {
                         if (pr->getTagObject() == object)
                         {
                             list.append(this->index(rule, column));
-//                            qDebug() << "Tag column:" << column;
                         }
                     }
                 }
@@ -1350,32 +1362,36 @@ void PolicyModel::configure()
     supports_logging      = false;
     supports_rule_options = false;
     supports_time         = false;
+    
+    string platform;
 
     if (getFirewall())
     {
+        platform = getFirewall()->getStr("platform");
+
         try {
             supports_logging = Resources::getTargetCapabilityBool(
-                    getFirewall()->getStr("platform"), "logging_in_policy");
+                    platform, "logging_in_policy");
             supports_rule_options = Resources::getTargetCapabilityBool(
-                    getFirewall()->getStr("platform"), "options_in_policy");
+                    platform, "options_in_policy");
             supports_time = Resources::getTargetCapabilityBool(
-                    getFirewall()->getStr("platform"), "supports_time");
+                    platform, "supports_time");
         } catch(FWException &ex)    {    }
     }
 
-    header  << ColDesc(RuleElementSrc::TYPENAME, ColDesc::Object)   // 1
-            << ColDesc(RuleElementDst::TYPENAME, ColDesc::Object)   // 2
-            << ColDesc(RuleElementSrv::TYPENAME, ColDesc::Object)   // 3
-            << ColDesc(RuleElementItf::TYPENAME, ColDesc::Object)   // 4
-            << ColDesc("Direction", ColDesc::Direction)             // 5
-            << ColDesc("Action", ColDesc::Action);                  // 6
+    header  << ColDesc(platform, RuleElementSrc::TYPENAME, ColDesc::Object)   // 1
+            << ColDesc(platform, RuleElementDst::TYPENAME, ColDesc::Object)   // 2
+            << ColDesc(platform, RuleElementSrv::TYPENAME, ColDesc::Object)   // 3
+            << ColDesc(platform, RuleElementItf::TYPENAME, ColDesc::Object)   // 4
+            << ColDesc(platform, "Direction", ColDesc::Direction)             // 5
+            << ColDesc(platform, "Action", ColDesc::Action);                  // 6
     if (supports_time)
-        header << ColDesc(RuleElementInterval::TYPENAME, ColDesc::Time);  // 7
+        header << ColDesc(platform, RuleElementInterval::TYPENAME, ColDesc::Time);  // 7
 
     if (supports_logging && supports_rule_options)
-        header << ColDesc("Options", ColDesc::Options);
+        header << ColDesc(platform, "Options", ColDesc::Options);
 
-    header << ColDesc("Comment", ColDesc::Comment);
+    header << ColDesc(platform, "Comment", ColDesc::Comment);
 }
 
 QVariant PolicyModel::getRuleDataForDisplayRole(const QModelIndex &index, RuleNode* node) const
@@ -1463,27 +1479,42 @@ bool PolicyModel::checkRuleType(libfwbuilder::Rule *rule)
 void NatModel::configure()
 {
     supports_actions = false;
+    supports_inbound_interface = false;
+    supports_outbound_interface = false;
 
+    string platform;
+    
     if (getFirewall())
     {
+        platform = getFirewall()->getStr("platform");
+
         try {
             supports_actions = Resources::getTargetCapabilityBool(
-                getFirewall()->getStr("platform"), "actions_in_nat");
+                platform, "actions_in_nat");
+            supports_inbound_interface = Resources::getTargetCapabilityBool(
+                platform, "inbound_interface_in_nat");
+            supports_outbound_interface = Resources::getTargetCapabilityBool(
+                platform, "outbound_interface_in_nat");
         } catch(FWException &ex)    {    }
     }
 
-    header  << ColDesc(RuleElementOSrc::TYPENAME, ColDesc::Object)   // 1
-            << ColDesc(RuleElementODst::TYPENAME, ColDesc::Object)   // 2
-            << ColDesc(RuleElementOSrv::TYPENAME, ColDesc::Object)   // 3
-            << ColDesc(RuleElementTSrc::TYPENAME, ColDesc::Object)   // 4
-            << ColDesc(RuleElementTDst::TYPENAME, ColDesc::Object)   // 5
-            << ColDesc(RuleElementTSrv::TYPENAME, ColDesc::Object);   // 6
+    header  << ColDesc(platform, RuleElementOSrc::TYPENAME, ColDesc::Object)   // 1
+            << ColDesc(platform, RuleElementODst::TYPENAME, ColDesc::Object)   // 2
+            << ColDesc(platform, RuleElementOSrv::TYPENAME, ColDesc::Object)   // 3
+            << ColDesc(platform, RuleElementTSrc::TYPENAME, ColDesc::Object)   // 4
+            << ColDesc(platform, RuleElementTDst::TYPENAME, ColDesc::Object)   // 5
+            << ColDesc(platform, RuleElementTSrv::TYPENAME, ColDesc::Object);   // 6
+
+    if (supports_inbound_interface)
+        header << ColDesc(platform, RuleElementItfInb::TYPENAME, ColDesc::Object);
+    if (supports_outbound_interface)
+        header << ColDesc(platform, RuleElementItfOutb::TYPENAME, ColDesc::Object);
 
     if (supports_actions)
-        header << ColDesc("Action", ColDesc::Action);
+        header << ColDesc(platform, "Action", ColDesc::Action);
 
-    header << ColDesc("Options", ColDesc::Options)                  // 7
-           << ColDesc("Comment", ColDesc::Comment);                 // 8
+    header << ColDesc(platform, "Options", ColDesc::Options)                  // 7
+           << ColDesc(platform, "Comment", ColDesc::Comment);                 // 8
 }
 
 QVariant NatModel::getRuleDataForDisplayRole(const QModelIndex &index, RuleNode* node) const
@@ -1554,25 +1585,35 @@ void RoutingModel::configure()
 {
     //if (fwbdebug) qDebug() << "RoutingModel::configure";
     supports_routing_itf  = false;
+    supports_metric  = false;
+
+    string platform;
+    string host_os;
 
     if (getFirewall())
     {
+        platform = getFirewall()->getStr("platform");
+        host_os = getFirewall()->getStr("host_OS");
+
         try {
-            supports_routing_itf =
-                Resources::getTargetCapabilityBool(
-                        getFirewall()->getStr("platform"), "supports_routing_itf");
+            supports_routing_itf = Resources::getTargetCapabilityBool(
+                host_os, "supports_routing_itf");
+            supports_metric = Resources::getTargetCapabilityBool(
+                host_os, "supports_metric");
         } catch(FWException &ex)    {    }
     }
 
-    header  << ColDesc(RuleElementRDst::TYPENAME, ColDesc::Object)   // 1
-            << ColDesc(RuleElementRGtw::TYPENAME, ColDesc::Object);  // 2
+    header  << ColDesc(platform, RuleElementRDst::TYPENAME, ColDesc::Object)   // 1
+            << ColDesc(platform, RuleElementRGtw::TYPENAME, ColDesc::Object);  // 2
 
     if (supports_routing_itf)
-        header << ColDesc(RuleElementRItf::TYPENAME, ColDesc::Object);
+        header << ColDesc(platform, RuleElementRItf::TYPENAME, ColDesc::Object);
 
-    header  << ColDesc("Metric", ColDesc::Metric)
-            << ColDesc("Options", ColDesc::Options)
-            << ColDesc("Comment", ColDesc::Comment);
+    if (supports_metric)
+        header  << ColDesc(platform, "Metric", ColDesc::Metric);
+
+    header << ColDesc(platform, "Options", ColDesc::Options)
+           << ColDesc(platform, "Comment", ColDesc::Comment);
 }
 
 QVariant RoutingModel::getRuleDataForDisplayRole(const QModelIndex &index, RuleNode* node) const
