@@ -309,6 +309,11 @@ RuleSetView* RuleSetView::getRuleSetViewByType(ProjectPanel *project,
     return NULL;
 }
 
+void RuleSetView::makeCurrentRuleVisible()
+{
+    scrollTo( currentIndex(), QAbstractItemView::PositionAtCenter);
+}
+
 void RuleSetView::selectRE(QModelIndex index)
 {
     if (fwbdebug)
@@ -704,31 +709,48 @@ void RuleSetView::addColumnRelatedMenu(QMenu *menu, const QModelIndex &index,
                 FWObject *object = getObject(pos, index);
 
                 QAction *editID = menu->addAction(
-                        tr("Edit")   , this , SLOT( editSelected() ) );
+                    tr("Edit")   , this , SLOT( editSelected() ) );
+
                 menu->addSeparator();
+
                 QAction *copyID = menu->addAction(
-                        tr("Copy")   , this , SLOT( copySelectedObject() ) );
+                    tr("Copy")   , this , SLOT( copySelectedObject() ) );
+
                 QAction *cutID  = menu->addAction(
-                        tr("Cut")    , this , SLOT( cutSelectedObject() ) );
-                menu->addAction( tr("Paste")  , this , SLOT( pasteObject() ) );
+                    tr("Cut")    , this , SLOT( cutSelectedObject() ) );
+
+                QAction *pasteID = menu->addAction(
+                    tr("Paste")  , this , SLOT( pasteObject() ) );
 
                 QAction *delID  =menu->addAction(
-                        tr("Delete") ,  this , SLOT( deleteSelectedObject() ) );
-                menu->addSeparator();
-                QAction *fndID = menu->addAction(
-                        tr("Where used") , this , SLOT( findWhereUsedSlot()));
-                QAction *revID = menu->addAction(
-                        tr("Reveal in tree") ,this , SLOT( revealObjectInTree() ) );
-                menu->addSeparator();
-                QAction *negID  = menu->addAction(
-                        tr("Negate") , this , SLOT( negateRE() ) );
+                    tr("Delete") ,  this , SLOT( deleteSelectedObject() ) );
 
-                if (object == NULL || re->isAny())
-                    editID->setEnabled(false);
+                menu->addSeparator();
+
+                QAction *fndID = menu->addAction(
+                    tr("Where used") , this , SLOT( findWhereUsedSlot()));
+
+                QAction *revID = menu->addAction(
+                    tr("Reveal in tree") ,this , SLOT( revealObjectInTree() ) );
+
+                menu->addSeparator();
+
+                QAction *negID  = menu->addAction(
+                    tr("Negate") , this , SLOT( negateRE() ) );
+
+                if (object == NULL || re->isAny()) editID->setEnabled(false);
                 copyID->setEnabled(!re->isAny());
                 cutID->setEnabled(!re->isAny());
                 delID->setEnabled(!re->isAny());
 
+                // see #1976 do not allow pasting object that has been deleted
+                FWObject *obj_in_clipboard =
+                    FWObjectClipboard::obj_clipboard->getObject();
+                bool obj_deleted = (obj_in_clipboard &&
+                                    obj_in_clipboard->getLibrary()->getId() ==
+                                    FWObjectDatabase::DELETED_OBJECTS_ID);
+                pasteID->setEnabled(obj_in_clipboard!=NULL && !obj_deleted);
+
                 string cap_name;
                 if (Policy::cast(md->getRuleSet())!=NULL) cap_name="negation_in_policy";
                 if (NAT::cast(md->getRuleSet())!=NULL) cap_name="negation_in_nat";
@@ -879,9 +901,8 @@ void RuleSetView::editSelected()
 
 bool RuleSetView::switchObjectInEditor(const QModelIndex& index, bool validate)
 {
-//    qDebug() << "RuleSetView::switchObjectInEditor";
     RuleSetModel* md = ((RuleSetModel*)model());
-    if(!isTreeReadWrite(this,md->getRuleSet())) return false;
+    if (!isTreeReadWrite(this,md->getRuleSet())) return false;
 
     if ( index.column()<=0 || index.row()==-1 ) return false;
 
@@ -964,6 +985,7 @@ bool RuleSetView::switchObjectInEditor(const QModelIndex& index, bool validate)
         return true;
     }
 
+
     if (object == NULL)
     {
         mw->blankEditor();
@@ -1033,11 +1055,11 @@ void RuleSetView::removeRule()
             RuleNode* node = md->nodeFromIndex(index);
             if (node->type == RuleNode::Rule)
                 rulesToDelete << node->rule;
-
-
         }
 
-        //Special case - all rows are inside one group (excluding the first item of the group)
+        //Special case - all rows are inside one group (excluding the
+        //first item of the group)
+
         if (itemsInGroups.size() == 1 &&
             itemsInGroups[itemsInGroups.keys().first()].size() == rulesToDelete.size())
         {
@@ -1053,7 +1075,9 @@ void RuleSetView::removeRule()
 
             if (!containsFirstRow)
             {
-                FWCmdRuleDeleteFromGroup* cmd = new FWCmdRuleDeleteFromGroup(project, md->getRuleSet(), rulesToDelete);
+                FWCmdRuleDeleteFromGroup* cmd =
+                    new FWCmdRuleDeleteFromGroup(project,
+                                                 md->getRuleSet(), rulesToDelete);
                 project->undoStack->push(cmd);
                 return;
             }
@@ -2196,7 +2220,8 @@ bool RuleSetView::validateForInsertionToInterfaceRE(RuleElementItf *re,
 bool RuleSetView::validateForInsertion(QModelIndex index, FWObject *obj)
 {
     ColDesc colDesc = index.data(Qt::UserRole).value<ColDesc>();
-    if (colDesc.type != ColDesc::Object && colDesc.type != ColDesc::Time) return false;
+    if (colDesc.type != ColDesc::Object && colDesc.type != ColDesc::Time)
+        return false;
 
     RuleElement *re = (RuleElement *)index.data(Qt::DisplayRole).value<void *>();
     assert (re!=NULL);
@@ -2207,6 +2232,10 @@ bool RuleSetView::validateForInsertion(RuleElement *re, FWObject *obj, bool quie
 {
     if (RuleSet::cast(obj)!=NULL) return false;
 
+    // see #1976 do not allow pasting object that has been deleted
+    if (obj && obj->getLibrary()->getId() == FWObjectDatabase::DELETED_OBJECTS_ID)
+        return false;
+
     if (! re->validateChild(obj) )
     {
         if (!quiet)
@@ -2252,6 +2281,7 @@ bool RuleSetView::validateForInsertion(RuleElement *re, FWObject *obj, bool quie
         }
     }
 
+    // This includes RuleElementItfInb and RuleElementItfOutb of nat rules
     if (RuleElementItf::cast(re) || RuleElementRItf::cast(re))
         return validateForInsertionToInterfaceRE(RuleElementItf::cast(re), obj);
 
@@ -2501,18 +2531,22 @@ bool RuleSetView::showToolTip(QEvent *event)
     QModelIndex index = indexAt(pos);
     if (!index.isValid())
     {
-        if (st->getBool("UI/AdvancedTooltips"))
-            return false;
-        toolTip = "<html>\
-Policy, NAT and routing rules are shown here.\
-<ul>\
-<li><b>Rules use objects</b>, if you want to use an object like IP address in a rule,\
- you need to first create it in the object tree</li>\
-<li><b>Drag and drop</b> objects from the tree to the desired field (source, destination, etc.) in the rule.</li>\
-<li><b>To add a rule</b>, click the '+' button at the top of the window</li>\
-<li><b>To open menu of operations</b> such as 'add rule', 'remove rule' etc, click right mouse button</li>\
-</ul>\
-</html>";
+        if (st->getBool("UI/AdvancedTooltips")) return false;
+
+        toolTip = QObject::tr(
+            "<html>"
+            "Policy, NAT and routing rules are shown here. "
+            "<ul><li><b>Rules use objects</b>, if you want "
+            "to use an object like IP address in a rule, "
+            "you need to first create it in the object tree</li>"
+            "<li><b>Drag and drop</b> objects from the tree to "
+            "the desired field (source, destination, etc.) in "
+            "the rule.</li>"
+            "<li><b>To add a rule</b>, click the '+' button at "
+            "the top of the window</li>"
+            "<li><b>To open menu of operations</b> such as "
+            "'add rule', 'remove rule' etc, click right mouse button</li>"
+            "</ul></html>");
 
         QToolTip::showText(mapToGlobal( he->pos() ), toolTip, this);
         return true;
@@ -2529,11 +2563,16 @@ Policy, NAT and routing rules are shown here.\
 
         if (column == 0)
         {
+            if (st->getBool("UI/AdvancedTooltips")) return false;
+
             // rule number column
-            toolTip = "<html>\
-<b>To open menu of operations</b> such as 'add rule', 'remove rule' etc, click right mouse button<br>\
-<b>To compile the rule</b> and see generated firewall configuration, first select it by clicking inside of it and then hit 'X' on keyboard\
-</html>";
+            toolTip = QObject::tr(
+                "<html>"
+                "<b>To open menu of operations</b> such as 'add rule', "
+                "'remove rule' etc, click right mouse button.<br> "
+                "<b>To compile the rule</b> and see generated firewall "
+                "configuration, first select it by clicking inside of it "
+                "and then hit 'X' on keyboard html>");
             QToolTip::showText(mapToGlobal( he->pos() ), toolTip, this);
             return true;
         } else
@@ -2564,23 +2603,39 @@ Policy, NAT and routing rules are shown here.\
             break;
 
             case ColDesc::Direction:
-                if (st->getBool("UI/AdvancedTooltips"))
-                    return false;
-                toolTip = "<b>Direction:</b> " + v.value<QString>() +
-                    "<br><b>To change the direction</b>, click right mouse button to open the list of possible settings";
+                if (st->getBool("UI/AdvancedTooltips")) 
+                {
+                    toolTip = QObject::tr("<b>Direction:</b> %1 <br>")
+                        .arg(v.value<QString>());
+                } else
+                {
+                    toolTip = QObject::tr("<b>Direction:</b> %1 <br>"
+                                          "<b>To change the direction</b>, "
+                                          "click right mouse button to open "
+                                          "the list of possible settings")
+                        .arg(v.value<QString>());
+                }
                 break;
 
             case ColDesc::Action:
-                if (st->getBool("UI/AdvancedTooltips"))
-                    return false;
-                toolTip = v.value<ActionDesc>().tooltip +
-                    "<b>To change the action</b>, click right mouse button to open the list of possible settings";
+                if (st->getBool("UI/AdvancedTooltips")) 
+                {
+                    toolTip = v.value<ActionDesc>().tooltip;
+                } else
+                {
+                    toolTip = QObject::tr("%1 <b>To change the action</b>, "
+                                          "click right mouse button to open "
+                                          "the list of possible settings")
+                        .arg(v.value<ActionDesc>().tooltip);
+                }
                 break;
 
             default:
                 FWObject *object = getObject(pos, index);
                 if (object == 0) return true;
-                toolTip = FWObjectPropertiesFactory::getObjectPropertiesDetailed(object, true, true);
+                toolTip = FWObjectPropertiesFactory::getObjectPropertiesDetailed(
+                    object, true, true);
+
                 if (st->getBool("UI/AdvancedTooltips"))
                 {
                     if (object->getId() == FWObjectDatabase::ANY_ADDRESS_ID ||
@@ -2865,7 +2920,8 @@ void RuleSetView::keyPressEvent( QKeyEvent* ev )
 void RuleSetView::compileCurrentRule()
 {
     RuleSetModel* md = ((RuleSetModel*)model());
-    if(!isTreeReadWrite(this,md->getRuleSet())) return;
+
+    //if (!isTreeReadWrite(this, md->getRuleSet())) return;
     if (md->getFirewall()==NULL) return;
 
     QModelIndex index = currentIndex();
@@ -2873,10 +2929,6 @@ void RuleSetView::compileCurrentRule()
     RuleNode* node = static_cast<RuleNode *>(index.internalPointer());
     if (node == 0 || node->type != RuleNode::Rule || node->rule == 0) return;
 
-    // if (mw->isEditorVisible() &&
-    //     !mw->requestEditorOwnership(this, node->rule, ObjectEditor::optRuleCompile, true))
-    //     return;
-
     mw->singleRuleCompile(node->rule);
 }
 
