@@ -378,7 +378,8 @@ QPixmap RuleSetViewDelegate::getPixmap(QString name, PixmapAttr pmattr) const
     return pm;
 }
 
-QSize RuleSetViewDelegate::sizeHint(const QStyleOptionViewItem & option, const QModelIndex & index) const
+QSize RuleSetViewDelegate::sizeHint(const QStyleOptionViewItem & option,
+                                    const QModelIndex & index) const
 {
     QStyleOptionViewItem newOpt = option;
     QFont font = st->getRulesFont();
@@ -436,11 +437,13 @@ QSize RuleSetViewDelegate::getTextSize(QString s, int flag)
     return fontMetrics.size(flag,s);
 }
 
-QSize RuleSetViewDelegate::calculateCellSizeForRule(const QStyleOptionViewItem & option, const QModelIndex & index, RuleNode * node ) const
+QSize RuleSetViewDelegate::calculateCellSizeForRule(
+    const QStyleOptionViewItem & option,
+    const QModelIndex & index,
+    RuleNode * node ) const
 {
     Q_UNUSED(option);
     Q_UNUSED(node);
-    //if (fwbdebug) qDebug() << "RuleSetViewDelegate::calculateCellSizeForRule";
 
     QSize iconSize = getIconSize();
     int itemHeight = getItemHeight();
@@ -468,6 +471,9 @@ QSize RuleSetViewDelegate::calculateCellSizeForRule(const QStyleOptionViewItem &
         case ColDesc::Direction :
             calculated = calculateCellSizeForIconAndText(index);
             break;
+        case ColDesc::Options :
+            calculated = calculateCellSizeForIconAndText(index);
+            break;
         default :
             calculated = QSize(0,0);
     }
@@ -480,10 +486,10 @@ QSize RuleSetViewDelegate::calculateCellSizeForComment(const QModelIndex & index
 {
     QString text = index.data(Qt::DisplayRole).value<QString>();
     if (text.isNull() && text.isEmpty()) return QSize(0,0);
-    QSize res = getTextSize(text,0);
+    QSize res = getTextSize(text, 0);
     if (st->getClipComment())
         res.setHeight(0);
-    return res;
+    return res + QSize(10, 0);
 }
 
 QSize RuleSetViewDelegate::calculateCellSizeForObject(const QModelIndex & index) const
@@ -539,7 +545,8 @@ QString RuleSetViewDelegate::constructActionText(ActionDesc &actionDesc) const
     return text;
 }
 
-QSize RuleSetViewDelegate::calculateCellSizeForIconAndText(const QModelIndex & index) const
+QSize RuleSetViewDelegate::calculateCellSizeForIconAndText(
+    const QModelIndex & index) const
 {
     QVariant v = index.data(Qt::DisplayRole);
     ActionDesc actionDesc = v.value<ActionDesc>();
@@ -569,6 +576,9 @@ QString RuleSetViewDelegate::objectText(RuleElement *re,FWObject *obj) const
         if (RuleElementRGtw::isA(re) ||
             RuleElementRItf::isA(re))  return QString("");
         if (RuleElementItf::isA(re))   return QString(tr("All"));
+        if (RuleElementItfInb::isA(re))   return QString(tr("Auto"));
+        if (RuleElementItfOutb::isA(re))   return QString(tr("Auto"));
+
         return any_object_name;
     }
 
