@@ -359,6 +359,10 @@ void instDialog::setInProcessState(QTreeWidgetItem *item)
 
 void instDialog::opSuccess(Firewall *fw)
 {
+    if (fwbdebug)
+        qDebug() << "instDialog::opSuccess fw=" << fw->getName().c_str();
+
+    compile_status[fw] = fwcompiler::BaseCompiler::FWCOMPILER_SUCCESS;
     QTreeWidgetItem* itm = opListMapping[(fw)->getId()];
     if (itm)
     {
@@ -373,6 +377,10 @@ void instDialog::opSuccess(Firewall *fw)
 
 void instDialog::opError(Firewall *fw)
 {
+    if (fwbdebug)
+        qDebug() << "instDialog::opError fw=" << fw->getName().c_str();
+
+    compile_status[fw] = fwcompiler::BaseCompiler::FWCOMPILER_ERROR;
     QTreeWidgetItem* itm = opListMapping[(fw)->getId()];
     if (itm)
     {
@@ -384,6 +392,7 @@ void instDialog::opError(Firewall *fw)
 
 void instDialog::opCancelled(Firewall *fw)
 {
+    compile_status[fw] = fwcompiler::BaseCompiler::FWCOMPILER_ERROR;
     QTreeWidgetItem* itm = opListMapping[(fw)->getId()];
     // itm can be NULL, for example when we install to PIX cluster
     // where we skip one of the members
@@ -466,17 +475,9 @@ void instDialog::storeInstallerOptions()
     st->setValue(SETTINGS_PATH_PREFIX"/Installer/dryrun"  ,cnf.dry_run);
     st->setValue(SETTINGS_PATH_PREFIX"/Installer/quiet",   cnf.quiet);
     st->setValue(SETTINGS_PATH_PREFIX"/Installer/verbose", cnf.verbose);
-    st->setValue(SETTINGS_PATH_PREFIX"/Installer/stripComments",
-                 cnf.stripComments);
-    st->setValue(SETTINGS_PATH_PREFIX"/Installer/compressScript",
-                 cnf.compressScript);
+    st->setValue(SETTINGS_PATH_PREFIX"/Installer/stripComments", cnf.stripComments);
+    st->setValue(SETTINGS_PATH_PREFIX"/Installer/compressScript", cnf.compressScript);
     st->setValue(SETTINGS_PATH_PREFIX"/Installer/copyFWB",  cnf.copyFWB);
-    st->setValue(SETTINGS_PATH_PREFIX"/Installer/testRun",  cnf.testRun);
-    st->setValue(SETTINGS_PATH_PREFIX"/Installer/rollback", cnf.rollback);
-    st->setValue(SETTINGS_PATH_PREFIX"/Installer/rollbackTime",
-                 cnf.rollbackTime);
-    st->setValue(SETTINGS_PATH_PREFIX"/Installer/canceRollbackIfSuccess",
-                 cnf.cancelRollbackIfSuccess);
 }
 
 void instDialog::summary()
@@ -488,9 +489,14 @@ void instDialog::summary()
     str.append(QObject::tr("* Firewall name : %1")
                .arg(QString::fromUtf8(cnf.fwobj->getName().c_str())));
     str.append(QObject::tr("* Installer uses user name : %1").arg(cnf.user));
-    str.append(QObject::tr("* Management address : %1").arg(cnf.maddr));
+
+    // print destination machine address or name correctly, taking into
+    // account putty session if any
     if (!cnf.putty_session.isEmpty())
-        str.append(QObject::tr("* Using putty session : %1").arg(cnf.putty_session));
+        str.append(QObject::tr("* Using putty session: %1").arg(cnf.putty_session));
+    else
+        str.append(QObject::tr("* Management address : %1").arg(cnf.maddr));
+
     str.append(QObject::tr("* Platform : %1")
               .arg(cnf.fwobj->getStr("platform").c_str()));
     str.append(QObject::tr("* Host OS : %1")
@@ -505,6 +511,15 @@ void instDialog::summary()
     if (cnf.dry_run)
         str.append(QObject::tr("* Commands will not be executed on the firewall"));
 
+    if (fwbdebug)
+    {
+        str.append(QObject::tr("--------------------------------"));
+        str.append(QObject::tr("* Variables:"));
+        str.append(QObject::tr("* fwdir= %1") .arg(cnf.fwdir));
+        str.append(QObject::tr("* fwscript= %1") .arg(cnf.fwscript));
+        str.append(QObject::tr("* remote_script= %1") .arg(cnf.remote_script));
+    }
+
     str.append("");
 
     QTextCursor cursor = m_dialog->procLogDisplay->textCursor();
@@ -988,7 +1003,7 @@ void instDialog::fillInstallUIList()
     list<Firewall*>::iterator i;
     for(i=install_fw_list.begin(); i!=install_fw_list.end(); ++i)
     {
-        f=(*i);
+        f = (*i);
         item = new InstallFirewallViewItem(
             NULL,
             QString::fromUtf8(f->getName().c_str()),
@@ -1041,6 +1056,12 @@ bool instDialog::tableHasCheckedItems()
  */
 bool instDialog::getInstOptions(Firewall *fw, bool cancelAllVisible)
 {
+    if (fwbdebug)
+        qDebug() << "instDialog::getInstOptions() begin"
+                 << "cnf.user=" << cnf.user
+                 << "cnf.maddr=" << cnf.maddr
+                 << "fw=" << fw;
+
     cnf.fwobj = fw;
 
     readInstallerOptionsFromSettings();
@@ -1055,7 +1076,9 @@ bool instDialog::getInstOptions(Firewall *fw, bool cancelAllVisible)
     {
         // In non-batch mode installer options from the dialog
         // overwrite options set in the fw object itself.
-        instOptionsDialog *inst_opt_dlg = new instOptionsDialog(this, &cnf, cancelAllVisible);
+        instOptionsDialog *inst_opt_dlg = new instOptionsDialog(
+            this, &cnf, cancelAllVisible);
+
         int resultCode = inst_opt_dlg->exec();
         // 0 - rejected
         // 1 - accepted
@@ -1076,6 +1099,11 @@ bool instDialog::getInstOptions(Firewall *fw, bool cancelAllVisible)
         inst_opt_dlg->deleteLater();
     }
 
+    if (fwbdebug)
+        qDebug() << "instDialog::getInstOptions() end"
+                 << "cnf.user=" << cnf.user
+                 << "cnf.maddr=" << cnf.maddr;
+
     return verifyManagementAddress();
 }
 
@@ -1084,6 +1112,12 @@ bool instDialog::getInstOptions(Firewall *fw, bool cancelAllVisible)
  */
 bool instDialog::getBatchInstOptions(Firewall *first_fw)
 {
+    if (fwbdebug)
+        qDebug() << "instDialog::getBatchInstOptions() begin"
+                 << "cnf.user=" << cnf.user
+                 << "cnf.maddr=" << cnf.maddr
+                 << "first_fw=" << first_fw;
+
     cnf.fwobj = first_fw;   // NULL;
 
     readInstallerOptionsFromSettings();
@@ -1107,7 +1141,9 @@ bool instDialog::getBatchInstOptions(Firewall *first_fw)
     readInstallerOptionsFromDialog(NULL, batch_inst_opt_dlg);
 
     if (fwbdebug)
-        qDebug() << "instDialog::getBatchInstOptions():  cnf.user=" << cnf.user;
+        qDebug() << "instDialog::getBatchInstOptions() end"
+                 << "cnf.user=" << cnf.user
+                 << "cnf.maddr=" << cnf.maddr;
 
     return verifyManagementAddress();
 }
@@ -1128,15 +1164,15 @@ void instDialog::readInstallerOptionsFromSettings()
     cnf.stripComments  = st->value(SETTINGS_PATH_PREFIX"/Installer/stripComments").toBool();
     cnf.compressScript = st->value(SETTINGS_PATH_PREFIX"/Installer/compressScript").toBool();
     cnf.copyFWB = st->value(SETTINGS_PATH_PREFIX"/Installer/copyFWB").toBool();
-    cnf.testRun = st->value(SETTINGS_PATH_PREFIX"/Installer/testRun").toBool();
-    cnf.rollback = st->value(SETTINGS_PATH_PREFIX"/Installer/rollback").toBool();
-    cnf.rollbackTime = st->value(SETTINGS_PATH_PREFIX"/Installer/rollbackTime").toInt();
-    cnf.cancelRollbackIfSuccess =
-        st->value(SETTINGS_PATH_PREFIX"/Installer/canceRollbackIfSuccess").toBool();
 }
 
 void instDialog::readInstallerOptionsFromFirewallObject(Firewall *fw)
 {
+    if (fwbdebug)
+        qDebug() << "instDialog::readInstallerOptionsFromFirewallObject"
+                 << "fw=" << fw
+                 << QString( (fw) ? QString::fromUtf8(fw->getName().c_str()) : "");
+
     FWOptions *fwopt = NULL;
     if (fw)
     {
@@ -1158,9 +1194,16 @@ void instDialog::readInstallerOptionsFromFirewallObject(Firewall *fw)
             standard_management_addr = "";
 
         QString aaddr = fwopt->getStr("altAddress").c_str();
+
+        if (fwbdebug)
+            qDebug() << "    standard_management_addr=" 
+                     << standard_management_addr
+                     << "aaddr=" << aaddr;
+
         if (!aaddr.isEmpty()) cnf.maddr = aaddr;
         else cnf.maddr = standard_management_addr;
 
+        cnf.putty_session = "";
 
 #ifdef _WIN32
 
@@ -1198,6 +1241,8 @@ void instDialog::readInstallerOptionsFromFirewallObject(Firewall *fw)
 
         if (sessions.contains(aaddr))
         {
+            if (fwbdebug)
+                qDebug() << "Found matching putty session" << aaddr;
             cnf.maddr = standard_management_addr;
             cnf.putty_session = aaddr;
         }
@@ -1222,7 +1267,13 @@ void instDialog::readInstallerOptionsFromFirewallObject(Firewall *fw)
 
         cnf.fwdir = s;
 
+        /*
+         * Generated files should be saved in the same directory where
+         * the .fwb file is located, except if user specified full path
+         * in the advaced settings dialog.
+         */
         cnf.script = FirewallInstaller::getGeneratedFileFullPath(fw);
+//        cnf.script = FirewallInstaller::getGeneratedFileName(fw);
         cnf.remote_script = ""; // filled in FirewallInstaller::readManifest()
         cnf.fwbfile = project->db()->getFileName().c_str();
         cnf.wdir = getFileDir( project->getRCS()->getFileName() );
@@ -1257,9 +1308,10 @@ void instDialog::readInstallerOptionsFromFirewallObject(Firewall *fw)
 void instDialog::readInstallerOptionsFromDialog(Firewall *fw,
                                                 instOptionsDialog *dlg)
 {
-    if (fwbdebug) qDebug("instDialog::readInstallerOptionsFromDialog");
+    if (fwbdebug)
+        qDebug() << "instDialog::readInstallerOptionsFromDialog"
+                 << "fw=" << fw;
 
-    cnf.fwobj = fw;
     QString adm_user;
 
     FWOptions *fwopt = NULL;
@@ -1291,7 +1343,7 @@ void instDialog::readInstallerOptionsFromDialog(Firewall *fw,
  */
         cnf.maddr = aaddr;
         if (fwbdebug)
-            qDebug("alternative addr %s", aaddr.toAscii().constData());
+            qDebug() << "alternative addr:" << aaddr;
     }
 
     // user name set in the dialog overrides that set in the fw object
@@ -1308,11 +1360,6 @@ void instDialog::readInstallerOptionsFromDialog(Firewall *fw,
     cnf.stripComments = dlg->m_dialog->stripComments->isChecked();
     cnf.compressScript= dlg->m_dialog->compressScript->isChecked();
     cnf.copyFWB       = dlg->m_dialog->copyFWB->isChecked();
-    cnf.testRun       = dlg->m_dialog->testRun->isChecked();
-    cnf.rollback      = dlg->m_dialog->rollback->isChecked();
-    cnf.rollbackTime  = dlg->m_dialog->rollbackTime->value();
-    cnf.cancelRollbackIfSuccess =
-        dlg->m_dialog->cancelRollbackIfSuccess->isChecked();
 
     dlg->savePassword();
     storeInstallerOptions();
@@ -1323,7 +1370,7 @@ bool instDialog::verifyManagementAddress()
 /* check for a common error when none or multiple interfaces are marked as
  * 'management'
  */
-    if (cnf.fwobj)
+    if (cnf.maddr.isEmpty() && cnf.putty_session.isEmpty() && cnf.fwobj)
     {
         int nmi = 0;
         list<FWObject*> ll = cnf.fwobj->getByTypeDeep(Interface::TYPENAME);
@@ -1332,6 +1379,7 @@ bool instDialog::verifyManagementAddress()
             Interface *intf = Interface::cast( *i );
             if (intf->isManagement()) nmi++;
         }
+
         if (nmi>1)
         {
             QString err = QObject::tr("Only one interface of the firewall '%1' "
@@ -1343,10 +1391,16 @@ bool instDialog::verifyManagementAddress()
             addToLog(err);
             return false;
         }
+
         if (nmi==0)
         {
-            QString err = QObject::tr("One of the interfaces of the firewall '%1' "
-                                      "must be marked as management interface.\n")
+            QString err = QObject::tr(
+                "One of the interfaces of the firewall '%1' "
+                "must be marked as management interface. "
+                "To set the management interface, double click "
+                "on the interface of the firewall that you will "
+                "connect to and check the box called Management "
+                "interface in the Editor panel")
                 .arg(QString::fromUtf8(cnf.fwobj->getName().c_str()));
 
             QMessageBox::critical(this, "Firewall Builder", err,
@@ -1354,12 +1408,14 @@ bool instDialog::verifyManagementAddress()
             addToLog(err);
             return false;
         }
+
         if (cnf.maddr == "" ||
             cnf.maddr == QString(InetAddr::getAny().toString().c_str()))
         {
-            QString err = QObject::tr("Management interface does not have IP address, "
-                                      "can not communicate with the firewall.\n");
-
+            QString err = QObject::tr(
+                "Management interface does not have IP address, "
+                "can not communicate with the firewall.\n");
+            
             QMessageBox::critical(this, "Firewall Builder", err,
                                   tr("&Continue") );
             addToLog(err);
