@@ -2,12 +2,10 @@
 
                           Firewall Builder
 
-                 Copyright (C) 2003 NetCitadel, LLC
+                 Copyright (C) 2003-2011 NetCitadel, LLC
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id$
-
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
   of that license as published by the Free Software Foundation; either
@@ -38,6 +36,8 @@
 #include "UserWorkflow.h"
 #include "FWBApplication.h"
 #include "QDesktopWidget"
+#include "networkZoneManager.h"
+#include "ObjConflictResolutionDialog.h"
 
 #include "interfaceProperties.h"
 #include "interfacePropertiesObjectFactory.h"
@@ -49,24 +49,27 @@
 #include "fwbuilder/BackgroundOp.h"
 #include "fwbuilder/IPv4.h"
 #include "fwbuilder/IPv6.h"
+#include "fwbuilder/Constants.h"
 
-#include <qlineedit.h>
-#include <qtextedit.h>
-#include <qcombobox.h>
-#include <qpushbutton.h>
-#include <qtoolbutton.h>
-#include <qradiobutton.h>
-#include <qcheckbox.h>
-#include <qtreewidget.h>
-#include <qtextbrowser.h>
-#include <qmessagebox.h>
-#include <qtimer.h>
-#include <qlistwidget.h>
-#include <qapplication.h>
-#include <qcursor.h>
-#include <qpixmapcache.h>
-#include <qfiledialog.h>
-#include <qdir.h>
+#include <QLineEdit>
+#include <QTextEdit>
+#include <QSpinBox>
+#include <QComboBox>
+#include <QPushButton>
+#include <QToolButton>
+#include <QRadioButton>
+#include <QCheckBox>
+#include <QTreeWidget>
+#include <QTableWidget>
+#include <QTextBrowser>
+#include <QMessageBox>
+#include <QTimer>
+#include <QListWidget>
+#include <QApplication>
+#include <QCursor>
+#include <QPixmapCache>
+#include <QFileDialog>
+#include <QDir>
 #include <QtDebug>
 
 #include <algorithm>
@@ -79,12 +82,22 @@ using namespace libfwbuilder;
 using namespace std;
 
 
+#define NAME_AND_PLATFORM_PAGE 0
+#define INTERFACES_MANUAL_OR_SNMP 1
+#define CONFIGURE_INTERFACES_MANUALLY 2
+#define CONFIGURE_SECURITY_LEVELS 3
+#define CONFIGURE_NETWORK_ZONES 4
+#define CHOOSE_FW_TEMPLATE 5
+#define CONFIGURE_TEMPLATE_INTERFACES_MANUALLY 6
+
 
 newFirewallDialog::newFirewallDialog(QWidget *parentw, FWObject *_p) :
     QDialog(parentw)
 {
     parent = _p;
-    db = parent->getRoot();
+    db_orig = parent->getRoot();
+    db_copy = new FWObjectDatabase();
+    db_copy->duplicate(db_orig, false);
 
     m_dialog = new Ui::newFirewallDialog_q;
     m_dialog->setupUi(this);
@@ -126,7 +139,8 @@ newFirewallDialog::newFirewallDialog(QWidget *parentw, FWObject *_p) :
              this, SLOT(updateTemplatePanel()));
 
     m_dialog->useStandard->setChecked(true);
-    m_dialog->templateFilePath->setText(tempfname.c_str());
+    m_dialog->templateFilePath->setText(
+        Constants::getTemplatesObjectsFilePath().c_str());
     updateTemplatePanel();
 
     /* fill in platform. Since iptables is the most popular, start with
@@ -144,19 +158,9 @@ newFirewallDialog::newFirewallDialog(QWidget *parentw, FWObject *_p) :
     /* fill in host OS  */
     setHostOS(m_dialog->hostOS, readPlatform(m_dialog->platform), "");
 
-    setNextEnabled( 0, false );
+    setNextEnabled( NAME_AND_PLATFORM_PAGE, false );
 
-    /*for (int i=0; i<pageCount(); ++i)
-        setHelpEnabled( i, false );*/
-/*
-    //m_dialog->iface_list->setItemMargin( 1 );
-    m_dialog->iface_list->setAllColumnsShowFocus( true );
-
-    //m_dialog->iface_sl_list->setItemMargin( 1 );
-    m_dialog->iface_dyn->setToolTip(wordWrap(tr("Check option 'dynamic address' for the interface that gets its IP address dynamically via DHCP or PPP protocol.") ,80 ));
-    m_dialog->iface_unnum->setToolTip(wordWrap(tr("Check option 'Unnumbered interface' for the interface that does not have an IP address. Examples of interfaces of this kind are those used to terminate PPPoE or VPN tunnels.") ,80 ));
-*/
-    m_dialog->iface_sl_list->setAllColumnsShowFocus( true );
+    //m_dialog->iface_sl_list->setAllColumnsShowFocus( true );
     QTimer::singleShot(0, m_dialog->obj_name, SLOT(setFocus()));
 
     currentTemplate = NULL;
@@ -186,7 +190,7 @@ newFirewallDialog::newFirewallDialog(QWidget *parentw, FWObject *_p) :
     if (this->height() > maxheight)
         this->resize(this->width(), maxheight);
 
-    showPage(0);
+    showPage(NAME_AND_PLATFORM_PAGE);
 }
 
 void newFirewallDialog::browseTemplate()
@@ -207,7 +211,8 @@ void newFirewallDialog::browseTemplate()
 
 void newFirewallDialog::useStandardTemplate()
 {
-    m_dialog->templateFilePath->setText(tempfname.c_str());
+    m_dialog->templateFilePath->setText(
+        Constants::getTemplatesObjectsFilePath().c_str());
     updateTemplatePanel();
 }
 
@@ -229,7 +234,8 @@ void newFirewallDialog::updateTemplatePanel()
 
         if (using_std)
         {
-            m_dialog->templateFilePath->setText(tempfname.c_str());
+            m_dialog->templateFilePath->setText(
+                Constants::getTemplatesObjectsFilePath().c_str());
         }
     }
     else
@@ -245,6 +251,7 @@ newFirewallDialog::~newFirewallDialog()
 #ifdef HAVE_LIBSNMP
     if (q!=NULL) delete q;
 #endif
+    delete db_copy;
 }
 
 void newFirewallDialog::changed()
@@ -256,7 +263,7 @@ void newFirewallDialog::changed()
                  << "use_manual=" << m_dialog->use_manual->isChecked()
                  << "use_snmp=" << m_dialog->use_snmp->isChecked();
 
-    if (p==0)
+    if (p==NAME_AND_PLATFORM_PAGE)
     {
         setNextEnabled(p,
                        !m_dialog->obj_name->text().isEmpty() &&
@@ -268,7 +275,7 @@ void newFirewallDialog::changed()
         m_dialog->interfaceEditor2->setHostOS(host_os);
     }
 
-    if (p==1)
+    if (p==INTERFACES_MANUAL_OR_SNMP)
     {
 
         bool use_snmp = false;
@@ -293,6 +300,7 @@ void newFirewallDialog::changed()
         use_snmp = m_dialog->use_manual->isChecked() || snmpPollCompleted;
         setNextEnabled( 1, use_snmp );
     }
+
     if (fwbdebug)
         qDebug() << "newFirewallDialog::changed() done";
 }
@@ -340,6 +348,17 @@ void newFirewallDialog::monitor()
 
     timer->stop();
 
+    QString platform = readPlatform(m_dialog->platform);
+
+    guessOSAndPlatformFromSysDescr(q->getDescr().c_str(),
+                                   discovered_platform,
+                                   discovered_host_os,
+                                   discovered_version);
+
+    if (fwbdebug)
+        qDebug() << "Guessed version as " << discovered_version;
+
+
     map<int, InterfaceData>* intf = q->getInterfaces();
     map<int, InterfaceData>::iterator i;
     this->m_dialog->interfaceEditor1->clear();
@@ -348,9 +367,52 @@ void newFirewallDialog::monitor()
     {
         InterfaceData* idata = &(i->second);
 
-        if ( idata->ostatus )
+        if (fwbdebug)
         {
-            idata->guessLabel(readPlatform(m_dialog->platform).toStdString());
+            qDebug() << "------------------------------------------------";
+            qDebug() << "id=" << idata->id.c_str();
+            qDebug() << "name=" << idata->name.c_str();
+            qDebug() << "snmp_type=" << idata->snmp_type;
+            qDebug() << "ostatus=" << idata->ostatus;
+            qDebug() << "mac_addr=" << idata->mac_addr.c_str();
+            qDebug() << "interface_type=" << idata->interface_type.c_str();
+            qDebug() << "";
+        }
+
+        /*
+         * some special treatment of discovered interfaces for Cisco ASA devices:
+         * if mac address is reported as 00:00:00:00:00:00 or
+         * 00:00:00:anything, this is usually some kind of internal special
+         * interface and we can skip it. Examples: "_internal_loopback",
+         * "Internal-Data0/1"
+         *
+         * This is different from how Linux reports mac address of a
+         * loopback because Linux snmpd returns empty string for the
+         * loopback mac address.
+         *
+         * The name of the interface reported by ASA is like this:
+         * "Adaptive Security Appliance 'Ethernet0/0' interface"
+         *
+         * Need to strip all thie verbose description
+         */ 
+
+        if (idata->ostatus)
+        {
+            guessInterfaceLabel(idata);
+
+            if (platform == "pix" || platform == "fwsm")
+            {
+                if ( ! idata->mac_addr.empty() && idata->snmp_type == 1 &&
+                     idata->mac_addr.find("00:00:00")==0) continue;
+
+                QString name = idata->name.c_str();
+                name.replace("Adaptive Security Appliance '", "");
+                name.replace("Cisco PIX Security Appliance '", "");
+                name.replace("PIX Firewall '", "");
+                name.replace("' interface", "");
+                idata->name = name.toStdString();
+            }
+
             this->m_dialog->interfaceEditor1->addInterfaceFromData(idata);
         }
     }
@@ -363,7 +425,7 @@ void newFirewallDialog::monitor()
 #endif
 
     snmpPollCompleted=true;
-    setNextEnabled( 1, true );
+    setNextEnabled( INTERFACES_MANUAL_OR_SNMP, true );
 }
 
 void newFirewallDialog::getInterfacesViaSNMP()
@@ -416,12 +478,13 @@ void newFirewallDialog::getInterfacesViaSNMP()
         }
     }
 
-    logger=NULL;
+    logger = NULL;
     m_dialog->snmpProgress->clear();
 
     if (q!=NULL) delete q;
-    q=new SNMP_interface_query();
-    q->init(addr.toString(),rcomm,SNMP_DEFAULT_RETRIES,SNMP_DEFAULT_TIMEOUT);
+
+    q = new SNMP_interface_query();
+    q->init(addr.toString(), rcomm, SNMP_DEFAULT_RETRIES, SNMP_DEFAULT_TIMEOUT);
 
     timer->setSingleShot(false);
     timer->start(0);
@@ -448,17 +511,16 @@ bool interfaceCompare(libfwbuilder::Interface *first,
 
 bool newFirewallDialog::appropriate(const int page) const
 {
-    int p  = page;
-
-    switch (p)
+    switch (page)
     {
-    case 0:
-    case 4:
+    case NAME_AND_PLATFORM_PAGE:
+    case CHOOSE_FW_TEMPLATE:
         return true;
 
-    case 1:
-    case 2:
-    case 3:
+    case INTERFACES_MANUAL_OR_SNMP:
+    case CONFIGURE_INTERFACES_MANUALLY:
+    case CONFIGURE_SECURITY_LEVELS:
+    case CONFIGURE_NETWORK_ZONES:
         return (!m_dialog->useTemplate->isChecked());
     }
     return true;
@@ -466,7 +528,7 @@ bool newFirewallDialog::appropriate(const int page) const
 
 void newFirewallDialog::nextClicked()
 {
-    if ( currentPage() == 4 )
+    if ( currentPage() == CHOOSE_FW_TEMPLATE )
     {
         if (m_dialog->templateList->currentItem() == NULL)
         {
@@ -475,13 +537,18 @@ void newFirewallDialog::nextClicked()
                     tr("Please select template"),
                     tr("&Continue"), QString::null,QString::null,
                     0, 1 );
-            showPage(4);
+            showPage(CHOOSE_FW_TEMPLATE);
             return;
         }
     }
-    if ( currentPage() == 2 )
+
+    if ( currentPage() == CONFIGURE_INTERFACES_MANUALLY )
         if ( !this->m_dialog->interfaceEditor1->isValid() )
             return;
+
+    // FakeWizard::nextRelevant() finds next (in the numerical order)
+    // page that is permitted by function appropriate() and returns
+    // its number
     if (nextRelevant( currentPage() ) > -1)
         showPage(nextRelevant( currentPage() ));
 }
@@ -504,7 +571,7 @@ void newFirewallDialog::showPage(const int page)
 // p is a page number _after_ it changed
     switch (p)
     {
-    case 0:
+    case NAME_AND_PLATFORM_PAGE:
         // we get here if user hits "Back" on page 4 (where they
         // choose template object)
         if (tmpldb!=NULL)
@@ -517,7 +584,7 @@ void newFirewallDialog::showPage(const int page)
         m_dialog->obj_name->setFocus();
         break;
 
-    case 1:
+    case INTERFACES_MANUAL_OR_SNMP:
     {
 // page 1 is where we choose to configure interfaces manually or via snmp
         m_dialog->snmpIP->setText("");
@@ -526,7 +593,7 @@ void newFirewallDialog::showPage(const int page)
         break;
     }
 
-    case 2:
+    case CONFIGURE_INTERFACES_MANUALLY:
     {
         if (!Resources::getTargetCapabilityBool(
                 readPlatform(m_dialog->platform).toLatin1().constData(),
@@ -536,36 +603,54 @@ void newFirewallDialog::showPage(const int page)
 /* if chosen fw platform does not support security levels,
  * this is the last page
  */
-            setNextEnabled( 2, false );
-            setFinishEnabled( 2, true );
+            setNextEnabled( CONFIGURE_INTERFACES_MANUALLY, false );
+            setFinishEnabled( CONFIGURE_INTERFACES_MANUALLY, true );
             m_dialog->finishButton->setDefault(true);
         }
         break;
     }
 
-    case 3:
+    case CONFIGURE_SECURITY_LEVELS:
     {
         if (m_dialog->useTemplate->isChecked())
         {
-            showPage( 0 );
+            showPage( NAME_AND_PLATFORM_PAGE );
             return;
         }
 
         // Edit security levels
         fillInterfaceSLList();
 
-        setNextEnabled( 3, false );
-        setFinishEnabled( 3, true );
+        setNextEnabled( CONFIGURE_SECURITY_LEVELS, true );
+        setFinishEnabled( CONFIGURE_SECURITY_LEVELS, false );
+        m_dialog->nextButton->setDefault(true);
+        break;
+    }
+
+    case CONFIGURE_NETWORK_ZONES:
+    {
+        if (m_dialog->useTemplate->isChecked())
+        {
+            showPage( NAME_AND_PLATFORM_PAGE );
+            return;
+        }
+
+        // Edit network zones
+        fillInterfaceNZList();
+
+        setNextEnabled(CONFIGURE_NETWORK_ZONES, false );
+        setFinishEnabled(CONFIGURE_NETWORK_ZONES, true );
         m_dialog->finishButton->setDefault(true);
         break;
     }
 
-    case 4:
+    case CHOOSE_FW_TEMPLATE:
     {
         // Show firewall templates
-        setFinishEnabled( 4, false );
-        setNextEnabled( 4, true );
-/* load templates if not loaded */
+        setFinishEnabled( CHOOSE_FW_TEMPLATE, false );
+        setNextEnabled( CHOOSE_FW_TEMPLATE, true );
+
+        // load templates if not loaded
         if (tmpldb==NULL)
         {
 
@@ -576,7 +661,7 @@ void newFirewallDialog::showPage(const int page)
             {
                 tmpldb->load(
                     m_dialog->templateFilePath->text().toAscii().data(),
-                    &upgrade_predicate, librespath);
+                    &upgrade_predicate, Constants::getDTDDirectory());
             }
             catch (FWException &ex)
             {
@@ -589,10 +674,20 @@ void newFirewallDialog::showPage(const int page)
             }
         }
 
+        // nfw != NULL if user clicked Back on one of the subsequent
+        // pages because we create firewall object when they click
+        // Next on the page where they choose template ( see case
+        // CONFIGURE_TEMPLATE_INTERFACES_MANUALLY below)
+        if (nfw)
+        {
+            parent->remove(nfw, false);
+            delete nfw;
+            nfw = NULL;
+        }
+
         list<FWObject*> fl;
 
-        FWObjectTypedChildIterator libiter =
-            tmpldb->findByType(Library::TYPENAME);
+        FWObjectTypedChildIterator libiter = tmpldb->findByType(Library::TYPENAME);
         for ( ; libiter!=libiter.end(); ++libiter)
             findFirewalls(*libiter, fl, false);
 
@@ -600,10 +695,11 @@ void newFirewallDialog::showPage(const int page)
 
         m_dialog->templateList->clear();
 
-        int n=0;
+        int n = 0;
+        QListWidgetItem *first_template = NULL;
         for (list<FWObject*>::iterator m=fl.begin(); m!=fl.end(); m++,n++)
         {
-            FWObject *o=*m;
+            FWObject *o = *m;
 
 /*
   Ticket #1492 requested the change to only show the user templates
@@ -633,28 +729,26 @@ void newFirewallDialog::showPage(const int page)
             twi->setText( QString(o->getName().c_str()) );
 
             m_dialog->templateList->addItem( twi );
-            templates[ m_dialog->templateList->item( m_dialog->templateList->count()-1 ) ]=o;
+            templates[ m_dialog->templateList->item(
+                    m_dialog->templateList->count()-1 ) ] = o;
+
+            if (first_template == NULL) first_template = twi;
         }
+
         m_dialog->templateList->setFocus();
-        m_dialog->templateList->setCurrentItem(0);
+        m_dialog->templateList->setCurrentItem(first_template,
+                                               QItemSelectionModel::SelectCurrent);
         m_dialog->finishButton->setDefault(false);
         m_dialog->nextButton->setDefault(true);
         break;
     }
 
-    case 5:
+    case CONFIGURE_TEMPLATE_INTERFACES_MANUALLY:
     {
         // Edit interfaces of the template object
         createFirewallFromTemplate();
-        setFinishEnabled( 5, true );
-/*
-        this->m_dialog->interfaceEditor2->setExplanation(
-            tr("Here you can change IP address of the template interface "
-               "to match addresses used on your network. "
-               "Interface can have several IPv4 and "
-               "IPv6 addresses.")
-        );
-*/
+        setFinishEnabled( CONFIGURE_TEMPLATE_INTERFACES_MANUALLY, true );
+
         this->m_dialog->interfaceEditor2->clear();
         this->m_dialog->interfaceEditor2->closeTab();//->removeTab(0);
         this->m_dialog->interfaceEditor2->setCornerWidgetsVisible(false);
@@ -670,69 +764,165 @@ void newFirewallDialog::showPage(const int page)
     }
 }
 
+void newFirewallDialog::getInterfaceDataFromInterfaceEditor(
+    EditedInterfaceData &edata, InterfaceData &idata)
+{
+    idata.name  = edata.name.toStdString();
+    idata.label = edata.label.toStdString();
+    AddressInfo address;
+    bool gotIPv4 = false;
+    foreach(AddressInfo addr, edata.addresses.values())
+    {
+        if (addr.ipv4)
+        {
+            address = addr;
+            gotIPv4 = true;
+            break;
+        }
+    }
+
+    InetAddrMask *iam;// = new InetAddrMask();
+    if (edata.type == 0 && edata.addresses.size() != 0)
+    {
+        if (!gotIPv4) address = edata.addresses.values().first();
+        if ( address.ipv4 )
+            iam = new InetAddrMask(
+                InetAddr(address.address.toStdString()),
+                InetAddr(address.netmask.toStdString()));
+        else
+        {
+            iam = new InetAddrMask(
+                InetAddr(AF_INET6, address.address.toStdString()),
+                InetAddr(AF_INET6, address.netmask.toStdString()));
+        }
+        idata.addr_mask.push_back(iam);
+    }
+
+    if (gotIPv4)
+    {
+        try
+        {
+            guessSecurityLevel(
+                readPlatform(m_dialog->platform).toStdString(), &idata);
+        }
+        catch (FWException &ex)
+        {
+            QMessageBox::warning(
+                this,"Firewall Builder", ex.toString().c_str(),
+                "&Continue", QString::null, QString::null, 0, 1 );
+            showPage( CONFIGURE_INTERFACES_MANUALLY );
+            return;
+        }
+    }
+    else idata.securityLevel = 0;
+}
+
 void newFirewallDialog::fillInterfaceSLList()
 {
     m_dialog->iface_sl_list->clear();
+
+    QStringList labels;
+    labels << QObject::tr("Name") << QObject::tr("Label")
+           << QObject::tr("Address") << QObject::tr("Security Level");
+    m_dialog->iface_sl_list->setHorizontalHeaderLabels(labels);
+
+    int row = 0;
     foreach(EditedInterfaceData iface,
             this->m_dialog->interfaceEditor1->getData().values() +
             this->m_dialog->interfaceEditor1->getNewData())
     {
         InterfaceData idata;
 
-        idata.name  = iface.name.toStdString();
-        idata.label = iface.label.toStdString();
-        AddressInfo address;
-        bool gotIPv4 = false;
-        foreach(AddressInfo addr, iface.addresses.values())
+        getInterfaceDataFromInterfaceEditor(iface, idata);
+
+        m_dialog->iface_sl_list->insertRow(row);
+
+        QTableWidgetItem* itm;
+
+        itm = new QTableWidgetItem(iface.name);
+        itm->setFlags(itm->flags() & ~Qt::ItemIsEditable);
+        m_dialog->iface_sl_list->setItem(row, 0, itm);
+
+        itm = new QTableWidgetItem(iface.label);
+        itm->setFlags(itm->flags() & ~Qt::ItemIsEditable);
+        m_dialog->iface_sl_list->setItem(row, 1, itm);
+
+        QString addr_str;
+        if (iface.addresses.size() > 0)
         {
-            if (addr.ipv4)
-            {
-                address = addr;
-                gotIPv4 = true;
-                break;
-            }
+            AddressInfo addr = *(iface.addresses.begin());
+            addr_str = addr.address;
         }
 
-        InetAddrMask *iam;// = new InetAddrMask();
-        if (iface.type == 0 && iface.addresses.size() != 0)
-        {
-            if (!gotIPv4) address = iface.addresses.values().first();
-            if ( address.ipv4 )
-                iam = new InetAddrMask(InetAddr(address.address.toStdString()), InetAddr(address.netmask.toStdString()));
-            else
-            {
-                iam = new InetAddrMask(InetAddr(AF_INET6, address.address.toStdString()), InetAddr(AF_INET6, address.netmask.toStdString()));
-            }
-            idata.addr_mask.push_back(iam);
-        }
+        itm = new QTableWidgetItem(addr_str);
+        itm->setFlags(itm->flags() & ~Qt::ItemIsEditable);
+        m_dialog->iface_sl_list->setItem(row, 2, itm);
 
-        if (gotIPv4)
-        {
-            try
-            {
-                idata.guessSecurityLevel( readPlatform(m_dialog->platform).toStdString() );
-            }
-            catch (FWException &ex)
-            {
-                QMessageBox::warning( this,"Firewall Builder", ex.toString().c_str(),
-                                      "&Continue", QString::null, QString::null, 0, 1 );
-                showPage( 2 );
-                return;
-            }
-        }
-        else idata.securityLevel = 0;
+        //itm = new QTableWidgetItem(QString::number(idata.securityLevel));
+        QSpinBox *widget = new QSpinBox();
+        widget->setMaximum(100);
+        widget->setMinimum(0);
+        widget->setValue(idata.securityLevel);
+        m_dialog->iface_sl_list->setCellWidget(row, 3, widget);
 
-
-        QStringList qsl;
-        qsl << iface.name//idata.name.c_str()
-            << iface.label//idata.label.c_str()
-            << address.address
-            << QString::number(idata.securityLevel);
-
-        new QTreeWidgetItem(m_dialog->iface_sl_list, qsl);
+        row++;
     }
 }
 
+void newFirewallDialog::fillInterfaceNZList()
+{
+    m_dialog->iface_nz_list->clear();
+
+    QStringList labels;
+    labels << QObject::tr("Name") << QObject::tr("Label")
+           << QObject::tr("Address") << QObject::tr("Network Zone");
+    m_dialog->iface_nz_list->setHorizontalHeaderLabels(labels);
+
+    NetworkZoneManager netzone_manager;
+    netzone_manager.load(db_copy);
+
+    int row = 0;
+    foreach(EditedInterfaceData iface,
+            this->m_dialog->interfaceEditor1->getData().values() +
+            this->m_dialog->interfaceEditor1->getNewData())
+    {
+        InterfaceData idata;
+
+        getInterfaceDataFromInterfaceEditor(iface, idata);
+
+        m_dialog->iface_nz_list->insertRow(row);
+
+        QTableWidgetItem* itm;
+
+        itm = new QTableWidgetItem(iface.name);
+        itm->setFlags(itm->flags() & ~Qt::ItemIsEditable);
+        m_dialog->iface_nz_list->setItem(row, 0, itm);
+
+        itm = new QTableWidgetItem(iface.label);
+        itm->setFlags(itm->flags() & ~Qt::ItemIsEditable);
+        m_dialog->iface_nz_list->setItem(row, 1, itm);
+
+        QString addr_str;
+        if (iface.addresses.size() > 0)
+        {
+            AddressInfo addr = *(iface.addresses.begin());
+            addr_str = addr.address;
+        }
+
+        itm = new QTableWidgetItem(addr_str);
+        itm->setFlags(itm->flags() & ~Qt::ItemIsEditable);
+        m_dialog->iface_nz_list->setItem(row, 2, itm);
+
+        QComboBox *widget = new QComboBox();
+        netzone_manager.packComboBox(widget, -1);
+        m_dialog->iface_nz_list->setCellWidget(row, 3, widget);
+
+        row++;
+    }
+
+    m_dialog->iface_nz_list->resizeColumnToContents(3);
+}
+
 /*
  * this slot is connected to currentItemChanged signal of templateList
  * As a side effect, this slot is called when we clear templateList.
@@ -876,57 +1066,6 @@ bool newFirewallDialog::validateAddressAndMask(const QString &addr,
     return true;
 }
 
-void newFirewallDialog::adjustSL(QTreeWidgetItem *itm1)
-{
-// interface 1 is above 2. Adjust their security levels accordingly
-    int sl1 = itm1->text(3).toInt();
-
-    int index = itm1->treeWidget()->indexOfTopLevelItem(itm1);
-
-    QTreeWidgetItem *itm2 = itm1->treeWidget()->topLevelItem(index+1);
-    QTreeWidgetItem *itm3 = itm1->treeWidget()->topLevelItem(index-1);
-
-    if (itm2==NULL) sl1=100;
-    else
-    {
-        if (itm3==NULL) sl1=0;
-        else
-        {
-            int sl2 = itm2->text(3).toInt();
-            int sl3 = itm3->text(3).toInt();
-            sl1 = (sl2+sl3)/2;
-        }
-    }
-    itm1->setText( 3 , QString("%1").arg(sl1) );
-}
-
-void newFirewallDialog::upInterface()
-{
-    QTreeWidgetItem *itm1 = m_dialog->iface_sl_list->currentItem();
-    if (itm1==NULL) return;
-    int index = m_dialog->iface_sl_list->indexOfTopLevelItem(itm1);
-
-    QTreeWidgetItem *itm2 = m_dialog->iface_sl_list->topLevelItem(index-1);
-    if (itm2==NULL) return;
-    m_dialog->iface_sl_list->takeTopLevelItem(index);
-    m_dialog->iface_sl_list->insertTopLevelItem(index-1, itm1);
-    adjustSL(itm1);
-}
-
-void newFirewallDialog::downInterface()
-{
-
-    QTreeWidgetItem *itm1 = m_dialog->iface_sl_list->currentItem();
-    if (itm1==NULL) return;
-    int index = m_dialog->iface_sl_list->indexOfTopLevelItem(itm1);
-
-    QTreeWidgetItem *itm2 = m_dialog->iface_sl_list->topLevelItem(index+1);
-    if (itm2==NULL) return;
-    m_dialog->iface_sl_list->takeTopLevelItem(index);
-    m_dialog->iface_sl_list->insertTopLevelItem(index+1, itm1);
-    adjustSL(itm1);
-}
-
 void newFirewallDialog::cancelClicked()
 {
     if (nfw)
@@ -951,19 +1090,18 @@ void newFirewallDialog::finishClicked()
     // see #1594
     m_dialog->finishButton->setFocus(Qt::OtherFocusReason);
 
-    int p = currentPage();
-
     if (fwbdebug)
         qDebug() << "newFirewallDialog::finishClicked()"
-                 << "p=" << p;
+                 << "currentPage()=" << currentPage();
 
-    if ((!this->m_dialog->useTemplate->isChecked()) && p == 2)
+    if ((!this->m_dialog->useTemplate->isChecked()) &&
+        currentPage() == CONFIGURE_INTERFACES_MANUALLY)
     {
         if ( !this->m_dialog->interfaceEditor1->isValid() )
             return;
     }
 
-    if ( p == 5 )
+    if ( currentPage() == CONFIGURE_TEMPLATE_INTERFACES_MANUALLY )
         if ( !this->m_dialog->interfaceEditor2->isValid() )
             return;
 
@@ -972,9 +1110,10 @@ void newFirewallDialog::finishClicked()
 
     st->setNewFirewallPlatform(platform.c_str());
 
-    if (p==2)  fillInterfaceSLList();
+    if (currentPage()==CONFIGURE_INTERFACES_MANUALLY)  fillInterfaceSLList();
 
-    if (p==4 || p==5)
+    if (currentPage()==CHOOSE_FW_TEMPLATE ||
+        currentPage()==CONFIGURE_TEMPLATE_INTERFACES_MANUALLY)
     {
         // Creating from a template
         if (nfw==NULL) createFirewallFromTemplate();
@@ -990,7 +1129,7 @@ void newFirewallDialog::finishClicked()
             return;
 
         FWObject *o;
-        o = db->create(Firewall::TYPENAME);
+        o = db_copy->create(Firewall::TYPENAME);
 
         if (o==NULL)
         {
@@ -1009,24 +1148,55 @@ void newFirewallDialog::finishClicked()
         o->setStr("host_OS", host_os);
         Resources::setDefaultTargetOptions(host_os , nfw);
 
+        if ( ! discovered_version.isEmpty())
+            o->setStr("version", discovered_version.toStdString());
+
 /* create interfaces */
 
-        foreach(EditedInterfaceData iface, this->m_dialog->interfaceEditor1->getNewData())
+        foreach(EditedInterfaceData iface,
+                this->m_dialog->interfaceEditor1->getNewData())
         {
             QString name     =  iface.name;
             QString label    =  iface.label;
             bool    dyn      =  iface.type == 1;
             bool    unnum    =  iface.type == 2;
             QString physaddr =  iface.mac;
+            int     sec_level = 0;
+            string  network_zone_str_id = "";
 
-            QList<QTreeWidgetItem*> ltwi = m_dialog->iface_sl_list->findItems( name , Qt::MatchExactly );
-            assert(!ltwi.empty());
-            QTreeWidgetItem *itm2 = ltwi[0];
-            assert(itm2!=NULL);
+            QList<QTableWidgetItem*> ltwi =
+                m_dialog->iface_sl_list->findItems( name , Qt::MatchExactly );
 
-            int sl = itm2->text(3).toInt();
+            if ( ! ltwi.empty())
+            {
+                QTableWidgetItem *itm2 = ltwi[0];
+                assert(itm2!=NULL);
+                int row = itm2->row();
+                QSpinBox *sb = dynamic_cast<QSpinBox*>(
+                    m_dialog->iface_sl_list->cellWidget(row, 3));
+                assert(sb!=NULL);
+                sec_level = sb->value();
+            }
 
-            Interface *oi = Interface::cast(db->create(Interface::TYPENAME));
+            ltwi = m_dialog->iface_nz_list->findItems( name , Qt::MatchExactly );
+            if ( ! ltwi.empty())
+            {
+                QTableWidgetItem *itm2 = ltwi[0];
+                assert(itm2!=NULL);
+                int row = itm2->row();
+                QComboBox *cb = dynamic_cast<QComboBox*>(
+                    m_dialog->iface_nz_list->cellWidget(row, 3));
+                assert(cb!=NULL);
+                int network_zone_int_id =
+                    cb->itemData(cb->currentIndex(), Qt::UserRole).toInt();
+                if (network_zone_int_id != 0)
+                    network_zone_str_id = FWObjectDatabase::getStringId(
+                        network_zone_int_id);
+                else
+                    network_zone_str_id = "";
+            }
+
+            Interface *oi = Interface::cast(db_copy->create(Interface::TYPENAME));
             assert(oi!=NULL);
 
             nfw->add(oi);
@@ -1037,12 +1207,11 @@ void newFirewallDialog::finishClicked()
 
             oi->setDyn(dyn);
             oi->setUnnumbered(unnum);
-            oi->setSecurityLevel(sl);
+            oi->setSecurityLevel(sec_level);
 
-            //mw->insertObjectInTree(nfw, oi);
-            if (fwbdebug)
-                qDebug("Adding interface %s: security_level=%d",
-                       oi->getName().c_str(), sl);
+            // only set network zone if it is supported and is not empty. See #2014
+            if (!network_zone_str_id.empty())
+                oi->setStr("network_zone", network_zone_str_id);
 
             std::auto_ptr<interfaceProperties> int_prop(
                 interfacePropertiesObjectFactory::getInterfacePropertiesObject(nfw));
@@ -1065,7 +1234,7 @@ void newFirewallDialog::finishClicked()
                     if (address.ipv4)
                     {
                         string addrname = string( QString("%1:%2:ip").arg(QString(m_dialog->obj_name->text())).arg(name).toUtf8().constData() );
-                        IPv4 *oa = IPv4::cast(db->create(IPv4::TYPENAME));
+                        IPv4 *oa = IPv4::cast(db_copy->create(IPv4::TYPENAME));
                         oi->add(oa);
                         oa->setName(addrname);
                         oa->setAddress( InetAddr(address.address.toLatin1().constData()) );
@@ -1083,7 +1252,7 @@ void newFirewallDialog::finishClicked()
                     else
                     {
                         string addrname = string ( QString("%1:%2:ip6").arg(QString(m_dialog->obj_name->text())).arg(name).toUtf8().constData() );
-                        IPv6 *oa = IPv6::cast(db->create(IPv6::TYPENAME));
+                        IPv6 *oa = IPv6::cast(db_copy->create(IPv6::TYPENAME));
                         oi->add(oa);
                         oa->setName(addrname);
                         oa->setAddress(InetAddr(AF_INET6, address.address.toLatin1().constData()) );
@@ -1103,6 +1272,15 @@ void newFirewallDialog::finishClicked()
         }
     }
 
+    // merge dbcopy into db
+
+    CompareObjectsDialog cod(this);
+    db_orig->merge(db_copy, &cod);
+    db_orig->fixTree();
+
+    nfw = Firewall::cast(db_orig->findInIndex(nfw->getId()));
+
+
     if (tmpldb!=NULL)
     {
         delete tmpldb;
