@@ -43,6 +43,7 @@
 #include "fwbuilder/BackgroundOp.h"
 #include "fwbuilder/IPv4.h"
 #include "fwbuilder/IPv6.h"
+#include "fwbuilder/Constants.h"
 
 #include <qlineedit.h>
 #include <qtextedit.h>
@@ -104,7 +105,8 @@ newHostDialog::newHostDialog(QWidget *parentw, FWObject *_p) : QDialog(parentw)
              this,SLOT(useStandardTemplate()));
     connect( m_dialog->useTemplate, SIGNAL(released()),
              this,SLOT(updateTemplatePanel()));
-    m_dialog->templateFilePath->setText(tempfname.c_str());
+    m_dialog->templateFilePath->setText(
+        Constants::getTemplatesObjectsFilePath().c_str());
     updateTemplatePanel();
 
     setNextEnabled( OBJECT_NAME_PAGE, false );
@@ -131,7 +133,8 @@ void newHostDialog::browseTemplate()
 
 void newHostDialog::useStandardTemplate()
 {
-    m_dialog->templateFilePath->setText(tempfname.c_str());
+    m_dialog->templateFilePath->setText(
+        Constants::getTemplatesObjectsFilePath().c_str());
     updateTemplatePanel();
 }
 
@@ -140,7 +143,7 @@ void newHostDialog::updateTemplatePanel()
     if (m_dialog->useTemplate->checkState()==Qt::Checked)
     {
         QString fileName = m_dialog->templateFilePath->text();
-        bool using_std = fileName == tempfname.c_str();
+        bool using_std = fileName == Constants::getTemplatesObjectsFilePath().c_str();
 
         m_dialog->templateFrame->setVisible(true);
         m_dialog->templateFilePathLabel->setVisible(!using_std);
@@ -248,7 +251,7 @@ void  newHostDialog::monitor()
         InterfaceData* idata = &(i->second);
         if ( idata->ostatus )
         {
-            idata->guessLabel("");
+            guessInterfaceLabel(idata);
             this->m_dialog->interfaceEditor->addInterfaceFromData(idata);
         }
     }
@@ -391,7 +394,7 @@ void newHostDialog::showPage(const int page)
             tmpldb = new FWObjectDatabase();
             tmpldb->setReadOnly( false );
             tmpldb->load( m_dialog->templateFilePath->text().toAscii().data(),
-                          &upgrade_predicate, librespath);
+                          &upgrade_predicate, Constants::getDTDDirectory());
         }
         list<FWObject*> fl;
 
@@ -578,11 +581,8 @@ void newHostDialog::finishClicked()
         no->duplicate(o, true);
         no->setName(m_dialog->obj_name->text().toUtf8().constData());
 
-        if (no==NULL)
-        {
-          QDialog::accept();
-          return;
-        }
+        mw->activeProject()->m_panel->om->autorename(no);
+
         nhst = Host::cast(no);
     } else
     {
@@ -624,7 +624,8 @@ void newHostDialog::finishClicked()
             {
                 QString addrname=QString("%1:%2:mac")
                     .arg(m_dialog->obj_name->text()).arg(name);
-                physAddress* pa = physAddress::cast(db->create(physAddress::TYPENAME));
+                physAddress* pa = physAddress::cast(
+                    db->create(physAddress::TYPENAME));
                 pa->setName(addrname.toUtf8().constData());
                 oi->add(pa);
                 pa->setPhysAddress(physaddr.toLatin1().constData());
@@ -636,11 +637,14 @@ void newHostDialog::finishClicked()
                     if (address.address == "0.0.0.0") continue;
                     if (address.ipv4)
                     {
-                        string addrname = string( QString("%1:%2:ip").arg(m_dialog->obj_name->text()).arg(name).toUtf8() );
+                        string addrname = string(
+                            QString("%1:%2:ip")
+                            .arg(m_dialog->obj_name->text()).arg(name).toUtf8());
                         IPv4 *oa = IPv4::cast(db->create(IPv4::TYPENAME));
                         oi->add(oa);
                         oa->setName(addrname);
-                        oa->setAddress( InetAddr(address.address.toLatin1().constData()) );
+                        oa->setAddress(
+                            InetAddr(address.address.toLatin1().constData()) );
                         bool ok = false ;
                         int inetmask = address.netmask.toInt(&ok);
                         if (ok)
@@ -649,16 +653,21 @@ void newHostDialog::finishClicked()
                         }
                         else
                         {
-                            oa->setNetmask( InetAddr(address.netmask.toLatin1().constData()) );
+                            oa->setNetmask(
+                                InetAddr(address.netmask.toLatin1().constData()) );
                         }
                     }
                     else
                     {
-                        string addrname = string( QString("%1:%2:ip6").arg(m_dialog->obj_name->text()).arg(name).toUtf8() );
+                        string addrname = string(
+                            QString("%1:%2:ip6")
+                            .arg(m_dialog->obj_name->text()).arg(name).toUtf8() );
                         IPv6 *oa = IPv6::cast(db->create(IPv6::TYPENAME));
                         oi->add(oa);
                         oa->setName(addrname);
-                        oa->setAddress(InetAddr(AF_INET6, address.address.toLatin1().constData()) );
+                        oa->setAddress(
+                            InetAddr(AF_INET6,
+                                     address.address.toLatin1().constData()) );
                         bool ok = false ;
                         int inetmask = address.netmask.toInt(&ok);
                         if (ok)
@@ -667,7 +676,9 @@ void newHostDialog::finishClicked()
                         }
                         else
                         {
-                            oa->setNetmask(InetAddr(AF_INET6, address.netmask.toLatin1().constData()));
+                            oa->setNetmask(
+                                InetAddr(AF_INET6,
+                                         address.netmask.toLatin1().constData()));
                         }
                     }
                 }
