@@ -33,6 +33,8 @@
 #include "Help.h"
 #include "FWCmdChange.h"
 
+#include "CompilerDriver.h"
+
 #include "fwbuilder/Firewall.h"
 #include "fwbuilder/Management.h"
 #include "fwbuilder/Resources.h"
@@ -49,6 +51,7 @@
 #include <qregexp.h>
 #include <qtextedit.h>
 #include <QUndoStack>
+#include <QFileInfo>
 
 
 using namespace std;
@@ -71,7 +74,7 @@ pfAdvancedDialog::pfAdvancedDialog(QWidget *parent,FWObject *o)
     assert(mgmt!=NULL);
 
     if (fwbdebug)
-        qDebug("%s",Resources::getTargetOptionStr(
+        qDebug("%s", Resources::getTargetOptionStr(
                    obj->getStr("host_OS"),"user_can_change_install_dir").c_str());
 
     if (!Resources::getTargetOptionBool(
@@ -80,12 +83,58 @@ pfAdvancedDialog::pfAdvancedDialog(QWidget *parent,FWObject *o)
         m_dialog->pf_fw_dir->setEnabled(false);
         fwopt->setStr("firewall_dir","");
     }
-    data.registerOption(m_dialog->ipv4before, fwopt, "ipv4_6_order", QStringList() <<  "IPv4 before IPv6" <<"ipv4_first" << "IPv6 before IPv4" << "ipv6_first");
+
+    // see #1888: we now support rc.conf format for the output
+    // Set variables for backwards compatibility for users who configured
+    // custom name for the output .fw script before.
+
+    if (!fwopt->getBool("generate_shell_script") &&
+        !fwopt->getBool("generate_rc_conf_file"))
+    {
+        fwopt->setBool("generate_shell_script", true);
+    }
+
+    if (!Resources::getTargetOptionBool(obj->getStr("host_OS"),
+                                        "rc_conf_format_supported"))
+    {
+        fwopt->setBool("generate_shell_script", true);
+        fwopt->setBool("generate_rc_conf_file", false);
+    }
+
+    m_dialog->generateShellScript->setEnabled(
+        Resources::getTargetOptionBool(obj->getStr("host_OS"),
+                                       "rc_conf_format_supported"));
+    m_dialog->generateRcConfFile->setEnabled(
+        Resources::getTargetOptionBool(obj->getStr("host_OS"),
+                                       "rc_conf_format_supported"));
+
+
+    QString init_script_name = QString::fromUtf8(
+        fwopt->getStr("output_file").c_str());
+    QString conf_file_name = QString::fromUtf8(
+        fwopt->getStr("conf1_file").c_str());
+
+    if (!init_script_name.isEmpty() && conf_file_name.isEmpty())
+    {
+        conf_file_name =
+            fwcompiler::CompilerDriver::getConfFileNameFromFwFileName(
+                init_script_name, ".conf");
+        fwopt->setStr("conf1_file", conf_file_name.toUtf8().constData());
+    }
+
+    data.registerOption(m_dialog->ipv4before, fwopt,
+                        "ipv4_6_order",
+                        QStringList() <<  "IPv4 before IPv6"
+                        <<"ipv4_first"
+                        << "IPv6 before IPv4"
+                        << "ipv6_first");
 
     data.registerOption( m_dialog->pf_log_prefix,fwopt, "log_prefix");
     data.registerOption( m_dialog->pf_fallback_log,fwopt, "fallback_log");
-    data.registerOption( m_dialog->pf_do_timeout_interval,fwopt,"pf_do_timeout_interval");
-    data.registerOption( m_dialog->pf_timeout_interval,fwopt, "pf_timeout_interval");
+    data.registerOption( m_dialog->pf_do_timeout_interval, fwopt,
+                         "pf_do_timeout_interval");
+    data.registerOption( m_dialog->pf_timeout_interval, fwopt,
+                         "pf_timeout_interval");
     data.registerOption( m_dialog->pf_do_timeout_frag,fwopt, "pf_do_timeout_frag");
     data.registerOption( m_dialog->pf_timeout_frag,fwopt, "pf_timeout_frag");
     data.registerOption( m_dialog->pf_do_limit_frags,fwopt, "pf_do_limit_frags");
@@ -129,7 +178,50 @@ pfAdvancedDialog::pfAdvancedDialog(QWidget *parent,FWObject *o)
     m_dialog->pf_state_policy->addItems(getScreenNames(slm));
     data.registerOption( m_dialog->pf_state_policy, fwopt, "pf_state_policy", slm);
     m_dialog->pf_state_policy->setEnabled(
-        XMLTools::version_compare(version, "4.3") >= 0);
+        XMLTools::version_compare(version, "3.5") >= 0);
+
+// Prepare block_policy combo box
+    slm.clear();
+    slm.push_back("");
+    slm.push_back("");
+    slm.push_back(QObject::tr("Drop"));
+    slm.push_back("drop");
+    slm.push_back(QObject::tr("Return"));
+    slm.push_back("return");
+    m_dialog->pf_block_policy->clear();
+    m_dialog->pf_block_policy->addItems(getScreenNames(slm));
+    data.registerOption( m_dialog->pf_block_policy, fwopt, "pf_block_policy", slm);
+    m_dialog->pf_block_policy->setEnabled(
+        XMLTools::version_compare(version, "3.5") >= 0);
+
+// set debug combo box
+    slm.clear();
+    slm.push_back("");
+    slm.push_back("");
+    slm.push_back("emerg");
+    slm.push_back("emerg");
+    slm.push_back("alert");
+    slm.push_back("alert");
+    slm.push_back("crit");
+    slm.push_back("crit");
+    slm.push_back("err");
+    slm.push_back("err");
+    slm.push_back("warning");
+    slm.push_back("warning");
+    slm.push_back("notice");
+    slm.push_back("notice");
+    slm.push_back("info");
+    slm.push_back("info");
+    slm.push_back("debug");
+    slm.push_back("debug");
+
+    m_dialog->pf_set_debug->clear();
+    m_dialog->pf_set_debug->addItems(getScreenNames(slm));
+    data.registerOption( m_dialog->pf_set_debug, fwopt, "pf_set_debug", slm);
+    m_dialog->pf_set_debug->setEnabled(
+        XMLTools::version_compare(version, "3.5") >= 0);
+
+
 
     data.registerOption( m_dialog->pf_check_shadowing,fwopt, "check_shading");
     data.registerOption( m_dialog->pf_ignore_empty_groups,fwopt,
@@ -178,51 +270,84 @@ pfAdvancedDialog::pfAdvancedDialog(QWidget *parent,FWObject *o)
                          "configure_pfsync_interfaces");
     data.registerOption( m_dialog->pf_configure_vlan_interfaces, fwopt,
                          "configure_vlan_interfaces");
+    data.registerOption( m_dialog->pf_configure_bridge_interfaces, fwopt,
+                         "configure_bridge_interfaces");
 
     data.registerOption( m_dialog->pf_debug,fwopt, "debug");
     data.registerOption( m_dialog->pf_flush_states, fwopt, "pf_flush_states");
 
     data.registerOption( m_dialog->compiler,fwopt, "compiler");
     data.registerOption( m_dialog->compilerArgs,fwopt, "cmdline");
+
+    data.registerOption( m_dialog->generateShellScript, fwopt,
+                         "generate_shell_script");
+    data.registerOption( m_dialog->generateRcConfFile, fwopt,
+                         "generate_rc_conf_file");
+
     data.registerOption( m_dialog->outputFileName, fwopt, "output_file");
-    data.registerOption( m_dialog->fileNameOnFw, fwopt, "script_name_on_firewall");
-    data.registerOption( m_dialog->confFileNameOnFw, fwopt, "conf_file_name_on_firewall");
+    data.registerOption( m_dialog->confFileName, fwopt, "conf1_file");
+
+    data.registerOption( m_dialog->fileNameOnFw, fwopt,
+                         "script_name_on_firewall");
+    data.registerOption( m_dialog->confFileNameOnFw, fwopt,
+                         "conf_file_name_on_firewall");
 
     data.registerOption( m_dialog->mgmt_ssh,fwopt, "mgmt_ssh");
     data.registerOption( m_dialog->mgmt_addr,fwopt, "mgmt_addr");
 
     data.registerOption( m_dialog->pf_set_tcp_first, fwopt, "pf_set_tcp_first");
     data.registerOption( m_dialog->pf_tcp_first, fwopt, "pf_tcp_first");
-    data.registerOption( m_dialog->pf_set_tcp_opening, fwopt, "pf_set_tcp_opening");
+    data.registerOption( m_dialog->pf_set_tcp_opening, fwopt,
+                         "pf_set_tcp_opening");
     data.registerOption( m_dialog->pf_tcp_opening, fwopt, "pf_tcp_opening");
-    data.registerOption( m_dialog->pf_set_tcp_established, fwopt, "pf_set_tcp_established");
-    data.registerOption( m_dialog->pf_tcp_established, fwopt, "pf_tcp_established");
-    data.registerOption( m_dialog->pf_set_tcp_closing, fwopt, "pf_set_tcp_closing");
+    data.registerOption( m_dialog->pf_set_tcp_established, fwopt,
+                         "pf_set_tcp_established");
+    data.registerOption( m_dialog->pf_tcp_established, fwopt,
+                         "pf_tcp_established");
+    data.registerOption( m_dialog->pf_set_tcp_closing, fwopt,
+                         "pf_set_tcp_closing");
     data.registerOption( m_dialog->pf_tcp_closing, fwopt, "pf_tcp_closing");
-    data.registerOption( m_dialog->pf_set_tcp_finwait, fwopt, "pf_set_tcp_finwait");
-    data.registerOption( m_dialog->pf_tcp_finwait, fwopt, "pf_tcp_finwait");
-    data.registerOption( m_dialog->pf_set_tcp_closed, fwopt, "pf_set_tcp_closed");
-    data.registerOption( m_dialog->pf_tcp_closed, fwopt, "pf_tcp_closed");
-    data.registerOption( m_dialog->pf_set_udp_first, fwopt, "pf_set_udp_first");
-    data.registerOption( m_dialog->pf_udp_first, fwopt, "pf_udp_first");
-    data.registerOption( m_dialog->pf_set_udp_single, fwopt, "pf_set_udp_single");
+    data.registerOption( m_dialog->pf_set_tcp_finwait, fwopt,
+                         "pf_set_tcp_finwait");
+    data.registerOption( m_dialog->pf_tcp_finwait, fwopt,
+                         "pf_tcp_finwait");
+    data.registerOption( m_dialog->pf_set_tcp_closed, fwopt,
+                         "pf_set_tcp_closed");
+    data.registerOption( m_dialog->pf_tcp_closed, fwopt,
+                         "pf_tcp_closed");
+    data.registerOption( m_dialog->pf_set_udp_first, fwopt,
+                         "pf_set_udp_first");
+    data.registerOption( m_dialog->pf_udp_first, fwopt,
+                         "pf_udp_first");
+    data.registerOption( m_dialog->pf_set_udp_single, fwopt,
+                         "pf_set_udp_single");
     data.registerOption( m_dialog->pf_udp_single, fwopt, "pf_udp_single");
-    data.registerOption( m_dialog->pf_set_udp_multiple, fwopt, "pf_set_udp_multiple");
+    data.registerOption( m_dialog->pf_set_udp_multiple, fwopt,
+                         "pf_set_udp_multiple");
     data.registerOption( m_dialog->pf_udp_multiple, fwopt, "pf_udp_multiple");
-    data.registerOption( m_dialog->pf_set_icmp_first, fwopt, "pf_set_icmp_first");
+    data.registerOption( m_dialog->pf_set_icmp_first, fwopt,
+                         "pf_set_icmp_first");
     data.registerOption( m_dialog->pf_icmp_first, fwopt, "pf_icmp_first");
-    data.registerOption( m_dialog->pf_set_icmp_error, fwopt, "pf_set_icmp_error");
+    data.registerOption( m_dialog->pf_set_icmp_error, fwopt,
+                         "pf_set_icmp_error");
     data.registerOption( m_dialog->pf_icmp_error, fwopt, "pf_icmp_error");
-    data.registerOption( m_dialog->pf_set_other_first, fwopt, "pf_set_other_first");
+    data.registerOption( m_dialog->pf_set_other_first, fwopt,
+                         "pf_set_other_first");
     data.registerOption( m_dialog->pf_other_first, fwopt, "pf_other_first");
-    data.registerOption( m_dialog->pf_set_other_single, fwopt, "pf_set_other_single");
+    data.registerOption( m_dialog->pf_set_other_single, fwopt,
+                         "pf_set_other_single");
     data.registerOption( m_dialog->pf_other_single, fwopt, "pf_other_single");
-    data.registerOption( m_dialog->pf_set_other_multiple, fwopt, "pf_set_other_multiple");
-    data.registerOption( m_dialog->pf_other_multiple, fwopt, "pf_other_multiple");
+    data.registerOption( m_dialog->pf_set_other_multiple, fwopt,
+                         "pf_set_other_multiple");
+    data.registerOption( m_dialog->pf_other_multiple, fwopt,
+                         "pf_other_multiple");
 
-    data.registerOption( m_dialog->pf_set_adaptive, fwopt, "pf_set_adaptive");
-    data.registerOption( m_dialog->pf_adaptive_start, fwopt, "pf_adaptive_start");
-    data.registerOption( m_dialog->pf_adaptive_end, fwopt, "pf_adaptive_end");
+    data.registerOption( m_dialog->pf_set_adaptive, fwopt,
+                         "pf_set_adaptive");
+    data.registerOption( m_dialog->pf_adaptive_start, fwopt,
+                         "pf_adaptive_start");
+    data.registerOption( m_dialog->pf_adaptive_end, fwopt,
+                         "pf_adaptive_end");
 
     PolicyInstallScript *pis   = mgmt->getPolicyInstallScript();
 
