@@ -44,6 +44,8 @@
 #include "fwbuilder/Resources.h"
 #include "fwbuilder/Rule.h"
 #include "fwbuilder/Policy.h"
+#include <fwbuilder/FailoverClusterGroup.h>
+#include <fwbuilder/StateSyncClusterGroup.h>
 
 #include <algorithm>
 #include <iostream>
@@ -185,17 +187,23 @@ bool isUsingNetZone(Firewall *fw)
 
 bool isDefaultPolicyRuleOptions(FWOptions *opt)
 {
-    bool res=true;
+    bool res = true;
     FWObject *p;
     PolicyRule *rule = NULL;
 
-    p=opt;
+    p = opt;
     do {
-        p=p->getParent();
+        p = p->getParent();
         if (PolicyRule::cast(p)!=NULL) rule = PolicyRule::cast(p);
     } while ( p!=NULL && Firewall::cast(p)==NULL );
 
-    assert(p!=NULL);
+    if (p==NULL)
+    {
+        qDebug() << "isDefaultPolicyRuleOptions()"
+                 << "Can not locate parent Firewall object for the options object";
+        opt->dump(false, true);
+        return true;
+    }
 
     QString platform = p->getStr("platform").c_str();
 
@@ -360,6 +368,14 @@ bool isDefaultNATRuleOptions(FWOptions *opt)
                      ! opt->getBool("pf_round_robin")  &&
                      ! opt->getBool("pf_static_port") ) );
 	}
+
+	if (platform=="pix" || platform=="fwsm")
+        {
+            res = (! opt->getBool("asa8_nat_dns") &&
+                   ! opt->getBool("asa8_nat_static") &&
+                   ! opt->getBool("asa8_nat_dynamic"));
+
+        }
     }
     return res;
 }
@@ -549,44 +565,47 @@ void setInterfaceTypes(QComboBox *iface_type,
         getSubInterfaceTypes(Interface::cast(iface->getParent()), mapping);
     else getInterfaceTypes(iface, mapping);
 
-    // #335 : if interface name matches naming convention for vlan
-    // interfaces and vlan type is in the list that came from the
-    // resource file, then leave only vlan in the list we return.
-    // Note that if resource file says this subint can not be vlan, we
-    // dan't return vlan type on the list even if its name looks like
-    // it could be one.
-    FWObject *p = iface->getParentHost();
-    assert(p!=NULL);
-    QString host_os = p->getStr("host_OS").c_str();
-    QString obj_name = iface->getName().c_str();
-
-    Resources* os_res = Resources::os_res[p->getStr("host_OS")];
-    string os_family = p->getStr("host_OS");
-    if (os_res!=NULL)
-        os_family = os_res->getResourceStr("/FWBuilderResources/Target/family");
-
-    std::auto_ptr<interfaceProperties> int_prop(
-        interfacePropertiesObjectFactory::getInterfacePropertiesObject(
-            os_family));
-    if (int_prop->looksLikeVlanInterface(obj_name))
+    if (st->getBool("Objects/Interface/autoconfigureInterfaces"))
     {
-        QString parent_name = iface->getParent()->getName().c_str();
-        QString err;
-        if (int_prop->isValidVlanInterfaceName(obj_name, parent_name, err))
+        // #335 : if interface name matches naming convention for vlan
+        // interfaces and vlan type is in the list that came from the
+        // resource file, then leave only vlan in the list we return.
+        // Note that if resource file says this subint can not be vlan, we
+        // dan't return vlan type on the list even if its name looks like
+        // it could be one.
+        FWObject *p = iface->getParentHost();
+        assert(p!=NULL);
+        QString host_os = p->getStr("host_OS").c_str();
+        QString obj_name = iface->getName().c_str();
+
+        Resources* os_res = Resources::os_res[p->getStr("host_OS")];
+        string os_family = p->getStr("host_OS");
+        if (os_res!=NULL)
+            os_family = os_res->getResourceStr("/FWBuilderResources/Target/family");
+
+        std::auto_ptr<interfaceProperties> int_prop(
+            interfacePropertiesObjectFactory::getInterfacePropertiesObject(
+                os_family));
+        if (int_prop->looksLikeVlanInterface(obj_name))
         {
-            // iface can be valid vlan interface. Leave only vlan type
-            // in the list if it was there to begin with. 
-            for (list<QStringPair>::iterator it=mapping.begin();
-                 it!=mapping.end(); ++it)
+            QString parent_name = iface->getParent()->getName().c_str();
+            QString err;
+            if (int_prop->isValidVlanInterfaceName(obj_name, parent_name, err))
             {
-                QString itype = it->first;
-                QString rtype = it->second;
-                if (itype == "8021q")
+                // iface can be valid vlan interface. Leave only vlan type
+                // in the list if it was there to begin with. 
+                for (list<QStringPair>::iterator it=mapping.begin();
+                     it!=mapping.end(); ++it)
                 {
-                    mapping.clear();
-                    mapping.push_back(QStringPair(itype, rtype));
-                    mapping.push_back(QStringPair("unknown", "Unknown"));
-                    break;
+                    QString itype = it->first;
+                    QString rtype = it->second;
+                    if (itype == "8021q")
+                    {
+                        mapping.clear();
+                        mapping.push_back(QStringPair(itype, rtype));
+                        mapping.push_back(QStringPair("unknown", "Unknown"));
+                        break;
+                    }
                 }
             }
         }
@@ -753,35 +772,56 @@ bool getStatelessFlagForAction(PolicyRule *rule)
 /**
  * Returns translatable string - name of the corresponding rule element.
  */
-QString getReadableRuleElementName(const string &rule_element_type_name)
+QString getReadableRuleElementName(const string &platform,
+                                   const string &re_type_name)
 {
+    bool nat_intf_in = Resources::getTargetCapabilityBool(
+            platform, "inbound_interface_in_nat");
+    bool nat_intf_out = Resources::getTargetCapabilityBool(
+            platform, "outbound_interface_in_nat");
+
     // The following map TYPENAME of RuleElement classes to readable
     // translatable names.
-    if (rule_element_type_name == "Src") return QObject::tr("Source");
-    if (rule_element_type_name == "Dst") return QObject::tr("Destination");
-    if (rule_element_type_name == "Srv") return QObject::tr("Service");
-    if (rule_element_type_name == "Itf") return QObject::tr("Interface");
-    if (rule_element_type_name == "When") return QObject::tr("Time");
+    if (re_type_name == "Src") return QObject::tr("Source");
+    if (re_type_name == "Dst") return QObject::tr("Destination");
+    if (re_type_name == "Srv") return QObject::tr("Service");
+    if (re_type_name == "Itf") return QObject::tr("Interface");
+    if (re_type_name == "When") return QObject::tr("Time");
 
-    if (rule_element_type_name == "OSrc") return QObject::tr("Original Src");
-    if (rule_element_type_name == "ODst") return QObject::tr("Original Dst");
-    if (rule_element_type_name == "OSrv") return QObject::tr("Original Srv");
+    if (re_type_name == "OSrc") return QObject::tr("Original Src");
+    if (re_type_name == "ODst") return QObject::tr("Original Dst");
+    if (re_type_name == "OSrv") return QObject::tr("Original Srv");
 
-    if (rule_element_type_name == "TSrc") return QObject::tr("Translated Src");
-    if (rule_element_type_name == "TDst") return QObject::tr("Translated Dst");
-    if (rule_element_type_name == "TSrv") return QObject::tr("Translated Srv");
+    if (re_type_name == "TSrc") return QObject::tr("Translated Src");
+    if (re_type_name == "TDst") return QObject::tr("Translated Dst");
+    if (re_type_name == "TSrv") return QObject::tr("Translated Srv");
 
-    if (rule_element_type_name == "RDst") return QObject::tr("Destination");
-    if (rule_element_type_name == "RGtw") return QObject::tr("Gateway");
-    if (rule_element_type_name == "RItf") return QObject::tr("Interface");
+    if (nat_intf_in != nat_intf_out)
+    {
+        // For some platforms I only show one interface column in nat
+        // rules, in this case nat_intf_in and nat_intf_out have
+        // different values.  For example, for PF I hide inbound
+        // interface and show outbound interface column. Columns title
+        // should then be just "Interface"
+        if (re_type_name == "ItfInb") return QObject::tr("Interface");
+        if (re_type_name == "ItfOutb") return QObject::tr("Interface");
+    } else
+    {
+        if (re_type_name == "ItfInb") return QObject::tr("Interface In");
+        if (re_type_name == "ItfOutb") return QObject::tr("Interface Out");
+    }
+
+    if (re_type_name == "RDst") return QObject::tr("Destination");
+    if (re_type_name == "RGtw") return QObject::tr("Gateway");
+    if (re_type_name == "RItf") return QObject::tr("Interface");
 
     // as of v3.0.x the following are not real rule elements (not separate
     // classes with names) but just attributes of corresponding Rule class.
-    if (rule_element_type_name == "Direction") return QObject::tr("Direction");
-    if (rule_element_type_name == "Action") return QObject::tr("Action");
-    if (rule_element_type_name == "Options") return QObject::tr("Options");
-    if (rule_element_type_name == "Metric") return QObject::tr("Metric");
-    if (rule_element_type_name == "Comment") return QObject::tr("Comment");
+    if (re_type_name == "Direction") return QObject::tr("Direction");
+    if (re_type_name == "Action") return QObject::tr("Action");
+    if (re_type_name == "Options") return QObject::tr("Options");
+    if (re_type_name == "Metric") return QObject::tr("Metric");
+    if (re_type_name == "Comment") return QObject::tr("Comment");
 
     return QString();
 }
@@ -995,3 +1035,248 @@ void _repackStringList(list<string> &list1, list<QStringPair> &list2)
     }
 }
 
+void setDefaultStateSyncGroupAttributes(StateSyncClusterGroup *grp)
+{
+    FWObject *p = grp;
+    while (p && Cluster::cast(p)==NULL) p = p->getParent();
+    assert(p != NULL);
+    Cluster *cluster = Cluster::cast(p);
+    Resources *os_res = Resources::os_res[cluster->getStr("host_OS")];
+    assert(os_res != NULL);
+
+    list<string> protocols;
+    os_res->getResourceStrList("/FWBuilderResources/Target/protocols/state_sync",
+                               protocols);
+
+    QStringList protocol_names = QString(protocols.front().c_str()).split(",");
+
+    grp->setName(protocol_names[1].toStdString());
+    grp->setStr("type", protocol_names[0].toStdString());
+}
+
+void setDefaultFailoverGroupAttributes(FailoverClusterGroup *grp)
+{
+    FWObject *p = grp;
+    while (p && Cluster::cast(p)==NULL) p = p->getParent();
+    assert(p != NULL);
+    Cluster *cluster = Cluster::cast(p);
+    Resources *os_res = Resources::os_res[cluster->getStr("host_OS")];
+    assert(os_res != NULL);
+
+    FWOptions *gropt = grp-> getOptionsObject();
+    assert(gropt != NULL);
+    
+    string failover_protocol = grp->getStr("type");
+
+    if (failover_protocol == "carp")
+    {
+        gropt->setStr("carp_password", "");
+        gropt->setInt("carp_vhid", 1);
+        gropt->setInt("carp_advbase", 1);
+        gropt->setInt("carp_master_advskew", 10);
+        gropt->setInt("carp_default_advskew", 20);
+    }
+
+    if (failover_protocol == "vrrp")
+    {
+        gropt->setStr("vrrp_secret", "");
+        gropt->setInt("vrrp_vrid", 1);
+        gropt->setBool("vrrp_over_ipsec_ah", false);
+    }
+
+    if (failover_protocol == "heartbeat")
+    {
+        string default_address =
+            os_res->getResourceStr(
+                "/FWBuilderResources/Target/protocols/heartbeat/default_address");
+        string default_port =
+            os_res->getResourceStr(
+                "/FWBuilderResources/Target/protocols/heartbeat/default_port");
+
+        gropt->setStr("heartbeat_address", default_address);
+        gropt->setStr("heartbeat_port", default_port);
+        gropt->setBool("heartbeat_unicast", false);
+    }
+
+    if (failover_protocol == "openais")
+    {
+        string default_address =
+            os_res->getResourceStr(
+                "/FWBuilderResources/Target/protocols/openais/default_address");
+        string default_port =
+            os_res->getResourceStr(
+                "/FWBuilderResources/Target/protocols/openais/default_port");
+        gropt->setStr("openais_address", default_address);
+        gropt->setStr("openais_port", default_port);
+    }
+
+    if (failover_protocol == "pix_failover")
+    {
+        gropt->setStr("pix_failover_key", "");
+    }
+}
+
+void guessInterfaceLabel(InterfaceData *idata)
+{
+/*
+ *  some firewalls report fairly regular names for interfaces through
+ *  their built-in SNMP agent. We can use this to assign labels
+ *  automatically.
+ *
+ *  in PIX interfaces have names like "PIX Firewall 'inside' interface"
+ *
+ */
+    QString qs_name = idata->name.c_str();
+    QString qs_label;
+
+    QRegExp pat1("Adaptive Security Appliance '(.*)' interface");
+    QRegExp pat2("Cisco PIX Security Appliance '(.*)' interface");
+    QRegExp pat3("PIX Firewall '(.*)' interface");
+
+    if (pat1.indexIn(qs_name) > -1) qs_label = pat1.cap(1);
+    if (pat2.indexIn(qs_name) > -1) qs_label = pat2.cap(1);
+    if (pat3.indexIn(qs_name) > -1) qs_label = pat3.cap(1);
+
+    idata->label = qs_label.toStdString();
+
+    if ( ! idata->isDyn &&
+         ! idata->isUnnumbered &&
+         ! idata->isBridgePort &&
+         idata->addr_mask.size()!=0 &&
+         idata->addr_mask.front()->getAddressPtr()->toString() == InetAddr::getLoopbackAddr().toString())
+        idata->label = "loopback";
+}
+
+void guessSecurityLevel(const string&, InterfaceData *idata)
+{
+    InetAddrMask n10(InetAddr("10.0.0.0"), InetAddr("255.0.0.0"));
+    InetAddrMask n172(InetAddr("172.16.0.0"), InetAddr("255.240.0.0"));
+    InetAddrMask n192(InetAddr("192.168.0.0"), InetAddr("255.255.0.0"));
+
+    idata->securityLevel = -1;
+
+    string llbl = idata->label;
+
+    for (string::size_type i=0; i<llbl.length(); i++)
+        llbl[i] = tolower( llbl[i] );
+
+    if ( llbl=="out" ||
+         llbl=="ext" ||
+         llbl=="internet" ||
+         llbl=="wan" ||
+         llbl=="dsl" ||
+         llbl=="cable" ||
+         llbl.find("outside")!=string::npos ||
+         llbl.find("external")!=string::npos) idata->securityLevel = 0;
+
+    if ( llbl=="lan" ||
+         llbl=="in" ||
+         llbl.find("inside")!=string::npos ||
+         llbl.find("internal")!=string::npos ) idata->securityLevel = 100;
+
+    if ( llbl.find("dmz")!=string::npos ) idata->securityLevel = 50;
+
+    if ((*(idata->addr_mask.front()->getAddressPtr()))==InetAddr::getLoopbackAddr())
+        idata->securityLevel = 100; 
+
+    if (idata->name=="Null0") idata->securityLevel = 100; 
+
+    if (idata->securityLevel==-1 &&
+        ! idata->isDyn && ! idata->isUnnumbered && ! idata->isBridgePort)
+    {
+        if (n10.belongs(InetAddr(*(idata->addr_mask.front()->getAddressPtr()))))
+            idata->securityLevel = 100;
+        
+        if (n172.belongs(InetAddr(*(idata->addr_mask.front()->getAddressPtr()))))
+            idata->securityLevel = 100;
+
+        if (n192.belongs(InetAddr(*(idata->addr_mask.front()->getAddressPtr()))))
+            idata->securityLevel = 100;
+    }
+
+    if (idata->isDyn || idata->isUnnumbered || idata->isBridgePort)
+        idata->securityLevel = 0;
+
+    if (idata->securityLevel==-1) idata->securityLevel = 0;
+}
+
+void guessOSAndPlatformFromSysDescr(
+    const QString &sysDescr, QString &platform, QString &hostOS, QString &version)
+{
+    QList<QRegExp> pix_re;
+    pix_re << QRegExp("Cisco PIX Firewall Version ([0-9\\.]+)")
+           << QRegExp("Cisco PIX Security Appliance Version ([0-9\\.]+)")
+           << QRegExp("Cisco Adaptive Security Appliance Version ([0-9\\.]+)");
+
+    QList<QRegExp> ios_re;
+    ios_re << QRegExp("Cisco Internetwork Operating System Software .* Version ([0-9\\.]+)");
+
+    platform = "";
+    hostOS = "";
+    version = "";
+
+    if (fwbdebug)
+        qDebug() << "guessOSAndPlatformFromSysDescr:"
+                 << "sysdescr=" << sysDescr;
+
+    list<QStringPair> allowed_versions;
+    QString version_from_sysdescr;
+
+    foreach (QRegExp re, pix_re)
+    {
+        if (re.indexIn(sysDescr) > -1)
+        {
+            platform = "pix";
+            hostOS = "pix_os";
+            version_from_sysdescr = re.cap(1);
+        }
+    }
+
+
+    foreach (QRegExp re, ios_re)
+    {
+        if (re.indexIn(sysDescr) > -1)
+        {
+            platform = "iosacl";
+            hostOS = "ios";
+            version_from_sysdescr = re.cap(1);
+        }
+    }
+
+    if (fwbdebug)
+        qDebug() << "guessOSAndPlatformFromSysDescr:"
+                 << "platform=" << platform
+                 << "hostOS=" << hostOS
+                 << "version=" << version_from_sysdescr;
+
+    if ( ! platform.isEmpty())
+        version = findBestVersionMatch(platform, version_from_sysdescr);
+
+}
+
+QString findBestVersionMatch(const QString &platform,
+                             const QString &discovered_version)
+{
+    list<QStringPair> allowed_versions;
+
+    getVersionsForPlatform(platform, allowed_versions);
+
+    if ( ! discovered_version.isEmpty())
+    {
+        QString version_fit;
+        list<QStringPair>::iterator it;
+        foreach (QStringPair p, allowed_versions)
+        {
+            QString vers = p.first;
+            if (XMLTools::version_compare(vers.toStdString(),
+                                          discovered_version.toStdString())>0)
+                break;
+            version_fit = vers;
+        }
+        return version_fit;
+    }
+    return "";
+}
+
+
+
