@@ -1,5 +1,5 @@
-/* $ANTLR 2.7.7 (20100319): "iptables.g" -> "IPTCfgLexer.cpp"$ */
-#line 42 "iptables.g"
+/* $ANTLR 2.7.7 (20090306): "iptables.g" -> "IPTCfgLexer.cpp"$ */
+#line 55 "iptables.g"
 
     // gets inserted before the antlr generated includes in the cpp
     // file
@@ -14,24 +14,21 @@
 #include <antlr/CharStreamIOException.hpp>
 #include <antlr/NoViableAltForCharException.hpp>
 
-#line 48 "iptables.g"
+#line 61 "iptables.g"
 
     // gets inserted after the antlr generated includes in the cpp
     // file
 #include <antlr/Token.hpp>
 #include <antlr/TokenBuffer.hpp>
 
-#include "../libgui/IPTImporter.h"
-
 #include "fwbuilder/TCPService.h"
 #include "fwbuilder/Logger.h"
 
 #include <algorithm>
 
-
-#line 33 "IPTCfgLexer.cpp"
+#line 30 "IPTCfgLexer.cpp"
 #line 1 "iptables.g"
-#line 35 "IPTCfgLexer.cpp"
+#line 32 "IPTCfgLexer.cpp"
 IPTCfgLexer::IPTCfgLexer(ANTLR_USE_NAMESPACE(std)istream& in)
 	: ANTLR_USE_NAMESPACE(antlr)CharScanner(new ANTLR_USE_NAMESPACE(antlr)CharBuffer(in),true)
 {
@@ -52,39 +49,39 @@ IPTCfgLexer::IPTCfgLexer(const ANTLR_USE_NAMESPACE(antlr)LexerSharedInputState&
 
 void IPTCfgLexer::initLiterals()
 {
-	literals["FIN"] = 106;
-	literals["RST"] = 107;
-	literals["NEW"] = 60;
-	literals["comment"] = 93;
-	literals["ESTABLISHED"] = 61;
-	literals["recent"] = 71;
-	literals["limit"] = 68;
-	literals["POSTROUTING"] = 13;
-	literals["INPUT"] = 9;
-	literals["ACK"] = 105;
-	literals["URG"] = 108;
-	literals["mark"] = 66;
-	literals["iprange"] = 72;
-	literals["multiport"] = 92;
-	literals["pkttype"] = 87;
-	literals["broadcast"] = 89;
-	literals["NONE"] = 111;
-	literals["multicast"] = 90;
-	literals["ALL"] = 110;
-	literals["length"] = 85;
-	literals["PSH"] = 109;
-	literals["SYN"] = 104;
-	literals["udp"] = 31;
-	literals["PREROUTING"] = 12;
-	literals["FORWARD"] = 10;
-	literals["INVALID"] = 59;
-	literals["OUTPUT"] = 11;
-	literals["state"] = 63;
-	literals["unicast"] = 91;
-	literals["RELATED"] = 62;
-	literals["icmp"] = 32;
-	literals["tcp"] = 30;
-	literals["COMMIT"] = 6;
+	literals["FIN"] = 108;
+	literals["RST"] = 109;
+	literals["NEW"] = 62;
+	literals["comment"] = 95;
+	literals["ESTABLISHED"] = 63;
+	literals["recent"] = 73;
+	literals["limit"] = 70;
+	literals["POSTROUTING"] = 16;
+	literals["INPUT"] = 12;
+	literals["ACK"] = 107;
+	literals["URG"] = 110;
+	literals["mark"] = 68;
+	literals["iprange"] = 74;
+	literals["multiport"] = 94;
+	literals["pkttype"] = 89;
+	literals["broadcast"] = 91;
+	literals["NONE"] = 113;
+	literals["multicast"] = 92;
+	literals["ALL"] = 112;
+	literals["length"] = 87;
+	literals["PSH"] = 111;
+	literals["SYN"] = 106;
+	literals["udp"] = 33;
+	literals["PREROUTING"] = 15;
+	literals["FORWARD"] = 13;
+	literals["INVALID"] = 61;
+	literals["OUTPUT"] = 14;
+	literals["state"] = 65;
+	literals["unicast"] = 93;
+	literals["RELATED"] = 64;
+	literals["icmp"] = 34;
+	literals["tcp"] = 32;
+	literals["COMMIT"] = 9;
 }
 
 ANTLR_USE_NAMESPACE(antlr)RefToken IPTCfgLexer::nextToken()
@@ -188,6 +185,12 @@ ANTLR_USE_NAMESPACE(antlr)RefToken IPTCfgLexer::nextToken()
 				theRetToken=_returnToken;
 				break;
 			}
+			case 0x23 /* '#' */ :
+			{
+				mNUMBER_SIGN(true);
+				theRetToken=_returnToken;
+				break;
+			}
 			case 0x25 /* '%' */ :
 			{
 				mPERCENT(true);
@@ -549,8 +552,8 @@ ANTLR_USE_NAMESPACE(antlr)RefToken IPTCfgLexer::nextToken()
 					mROUTE_GW(true);
 					theRetToken=_returnToken;
 				}
-				else if ((LA(1) == 0x23 /* '#' */ ) && ((LA(2) >= 0x3 /* '\3' */  && LA(2) <= 0xff))) {
-					mLINE_COMMENT(true);
+				else if ((LA(1) == 0x20 /* ' ' */ ) && (LA(2) == 0x47 /* 'G' */ )) {
+					mIPTABLES_SAVE_HEADER(true);
 					theRetToken=_returnToken;
 				}
 				else if ((LA(1) == 0x2d /* '-' */ ) && (LA(2) == 0x41 /* 'A' */ )) {
@@ -589,14 +592,10 @@ ANTLR_USE_NAMESPACE(antlr)RefToken IPTCfgLexer::nextToken()
 					mOPT_FRAGM(true);
 					theRetToken=_returnToken;
 				}
-				else if ((_tokenSet_0.member(LA(1)))) {
+				else if ((_tokenSet_0.member(LA(1))) && (true)) {
 					mWhitespace(true);
 					theRetToken=_returnToken;
 				}
-				else if ((LA(1) == 0x23 /* '#' */ ) && (true)) {
-					mNUMBER_SIGN(true);
-					theRetToken=_returnToken;
-				}
 				else if ((LA(1) == 0x2d /* '-' */ ) && (true)) {
 					mMINUS(true);
 					theRetToken=_returnToken;
@@ -631,60 +630,12 @@ tryAgain:;
 	}
 }
 
-void IPTCfgLexer::mLINE_COMMENT(bool _createToken) {
+void IPTCfgLexer::mIPTABLES_SAVE_HEADER(bool _createToken) {
 	int _ttype; ANTLR_USE_NAMESPACE(antlr)RefToken _token; ANTLR_USE_NAMESPACE(std)string::size_type _begin = text.length();
-	_ttype = LINE_COMMENT;
+	_ttype = IPTABLES_SAVE_HEADER;
 	ANTLR_USE_NAMESPACE(std)string::size_type _saveIndex;
 	
-	match("#");
-	{ // ( ... )*
-	for (;;) {
-		if ((_tokenSet_1.member(LA(1)))) {
-			{
-			match(_tokenSet_1);
-			}
-		}
-		else {
-			goto _loop166;
-		}
-		
-	}
-	_loop166:;
-	} // ( ... )*
-	mNEWLINE(false);
-	if ( _createToken && _token==ANTLR_USE_NAMESPACE(antlr)nullToken && _ttype!=ANTLR_USE_NAMESPACE(antlr)Token::SKIP ) {
-	   _token = makeToken(_ttype);
-	   _token->setText(text.substr(_begin, text.length()-_begin));
-	}
-	_returnToken = _token;
-	_saveIndex=0;
-}
-
-void IPTCfgLexer::mNEWLINE(bool _createToken) {
-	int _ttype; ANTLR_USE_NAMESPACE(antlr)RefToken _token; ANTLR_USE_NAMESPACE(std)string::size_type _begin = text.length();
-	_ttype = NEWLINE;
-	ANTLR_USE_NAMESPACE(std)string::size_type _saveIndex;
-	
-	{
-	if ((LA(1) == 0xd /* '\r' */ ) && (LA(2) == 0xa /* '\n' */ )) {
-		match("\r\n");
-	}
-	else if ((LA(1) == 0xd /* '\r' */ ) && (true)) {
-		match('\r' /* charlit */ );
-	}
-	else if ((LA(1) == 0xa /* '\n' */ )) {
-		match('\n' /* charlit */ );
-	}
-	else {
-		throw ANTLR_USE_NAMESPACE(antlr)NoViableAltForCharException(LA(1), getFilename(), getLine(), getColumn());
-	}
-	
-	}
-	if ( inputState->guessing==0 ) {
-#line 1182 "iptables.g"
-		newline(); resetText();
-#line 687 "IPTCfgLexer.cpp"
-	}
+	match(" Generated by iptables-save v");
 	if ( _createToken && _token==ANTLR_USE_NAMESPACE(antlr)nullToken && _ttype!=ANTLR_USE_NAMESPACE(antlr)Token::SKIP ) {
 	   _token = makeToken(_ttype);
 	   _token->setText(text.substr(_begin, text.length()-_begin));
@@ -762,9 +713,42 @@ void IPTCfgLexer::mWhitespace(bool _createToken) {
 	}
 	}
 	if ( inputState->guessing==0 ) {
-#line 1180 "iptables.g"
+#line 1243 "iptables.g"
 		_ttype = ANTLR_USE_NAMESPACE(antlr)Token::SKIP;
-#line 768 "IPTCfgLexer.cpp"
+#line 719 "IPTCfgLexer.cpp"
+	}
+	if ( _createToken && _token==ANTLR_USE_NAMESPACE(antlr)nullToken && _ttype!=ANTLR_USE_NAMESPACE(antlr)Token::SKIP ) {
+	   _token = makeToken(_ttype);
+	   _token->setText(text.substr(_begin, text.length()-_begin));
+	}
+	_returnToken = _token;
+	_saveIndex=0;
+}
+
+void IPTCfgLexer::mNEWLINE(bool _createToken) {
+	int _ttype; ANTLR_USE_NAMESPACE(antlr)RefToken _token; ANTLR_USE_NAMESPACE(std)string::size_type _begin = text.length();
+	_ttype = NEWLINE;
+	ANTLR_USE_NAMESPACE(std)string::size_type _saveIndex;
+	
+	{
+	if ((LA(1) == 0xd /* '\r' */ ) && (LA(2) == 0xa /* '\n' */ )) {
+		match("\r\n");
+	}
+	else if ((LA(1) == 0xd /* '\r' */ ) && (true)) {
+		match('\r' /* charlit */ );
+	}
+	else if ((LA(1) == 0xa /* '\n' */ )) {
+		match('\n' /* charlit */ );
+	}
+	else {
+		throw ANTLR_USE_NAMESPACE(antlr)NoViableAltForCharException(LA(1), getFilename(), getLine(), getColumn());
+	}
+	
+	}
+	if ( inputState->guessing==0 ) {
+#line 1245 "iptables.g"
+		newline(); resetText();
+#line 752 "IPTCfgLexer.cpp"
 	}
 	if ( _createToken && _token==ANTLR_USE_NAMESPACE(antlr)nullToken && _ttype!=ANTLR_USE_NAMESPACE(antlr)Token::SKIP ) {
 	   _token = makeToken(_ttype);
@@ -787,6 +771,19 @@ void IPTCfgLexer::mIPV4(bool _createToken) {
 	_saveIndex=0;
 }
 
+void IPTCfgLexer::mTHREE_COMPONENT_VERSION(bool _createToken) {
+	int _ttype; ANTLR_USE_NAMESPACE(antlr)RefToken _token; ANTLR_USE_NAMESPACE(std)string::size_type _begin = text.length();
+	_ttype = THREE_COMPONENT_VERSION;
+	ANTLR_USE_NAMESPACE(std)string::size_type _saveIndex;
+	
+	if ( _createToken && _token==ANTLR_USE_NAMESPACE(antlr)nullToken && _ttype!=ANTLR_USE_NAMESPACE(antlr)Token::SKIP ) {
+	   _token = makeToken(_ttype);
+	   _token->setText(text.substr(_begin, text.length()-_begin));
+	}
+	_returnToken = _token;
+	_saveIndex=0;
+}
+
 void IPTCfgLexer::mIPV6(bool _createToken) {
 	int _ttype; ANTLR_USE_NAMESPACE(antlr)RefToken _token; ANTLR_USE_NAMESPACE(std)string::size_type _begin = text.length();
 	_ttype = IPV6;
@@ -960,17 +957,17 @@ void IPTCfgLexer::mNUM_HEX_4DIGIT(bool _createToken) {
 	
 	mHEXDIGIT(false);
 	{
-	if ((_tokenSet_2.member(LA(1)))) {
+	if ((_tokenSet_1.member(LA(1)))) {
 		{
 		mHEXDIGIT(false);
 		}
 		{
-		if ((_tokenSet_2.member(LA(1)))) {
+		if ((_tokenSet_1.member(LA(1)))) {
 			{
 			mHEXDIGIT(false);
 			}
 			{
-			if ((_tokenSet_2.member(LA(1)))) {
+			if ((_tokenSet_1.member(LA(1)))) {
 				mHEXDIGIT(false);
 			}
 			else {
@@ -1000,10 +997,10 @@ void IPTCfgLexer::mNUMBER(bool _createToken) {
 	_ttype = NUMBER;
 	ANTLR_USE_NAMESPACE(std)string::size_type _saveIndex;
 	
-	bool synPredMatched192 = false;
-	if ((((LA(1) >= 0x30 /* '0' */  && LA(1) <= 0x39 /* '9' */ )) && (_tokenSet_3.member(LA(2))) && (_tokenSet_3.member(LA(3))) && (_tokenSet_3.member(LA(4))) && (_tokenSet_3.member(LA(5))) && (_tokenSet_3.member(LA(6))) && (_tokenSet_3.member(LA(7))) && (true) && (true) && (true) && (true) && (true) && (true) && (true) && (true) && (true) && (true) && (true) && (true) && (true))) {
-		int _m192 = mark();
-		synPredMatched192 = true;
+	bool synPredMatched186 = false;
+	if ((((LA(1) >= 0x30 /* '0' */  && LA(1) <= 0x39 /* '9' */ )) && (_tokenSet_2.member(LA(2))) && (_tokenSet_2.member(LA(3))) && (_tokenSet_2.member(LA(4))) && (_tokenSet_2.member(LA(5))) && (_tokenSet_2.member(LA(6))) && (_tokenSet_2.member(LA(7))) && (true) && (true) && (true) && (true) && (true) && (true) && (true) && (true) && (true) && (true) && (true) && (true) && (true))) {
+		int _m186 = mark();
+		synPredMatched186 = true;
 		inputState->guessing++;
 		try {
 			{
@@ -1014,12 +1011,12 @@ void IPTCfgLexer::mNUMBER(bool _createToken) {
 			}
 		}
 		catch (ANTLR_USE_NAMESPACE(antlr)RecognitionException& pe) {
-			synPredMatched192 = false;
+			synPredMatched186 = false;
 		}
-		rewind(_m192);
+		rewind(_m186);
 		inputState->guessing--;
 	}
-	if ( synPredMatched192 ) {
+	if ( synPredMatched186 ) {
 		{
 		mNUM_3DIGIT(false);
 		match('.' /* charlit */ );
@@ -1029,62 +1026,96 @@ void IPTCfgLexer::mNUMBER(bool _createToken) {
 		match('.' /* charlit */ );
 		mNUM_3DIGIT(false);
 		if ( inputState->guessing==0 ) {
-#line 1245 "iptables.g"
+#line 1312 "iptables.g"
 			_ttype = IPV4;
-#line 1035 "IPTCfgLexer.cpp"
+#line 1032 "IPTCfgLexer.cpp"
 		}
 		}
 	}
-	else if ((LA(1) == 0x30 /* '0' */ ) && (LA(2) == 0x78 /* 'x' */ )) {
-		{
-		match('0' /* charlit */ );
-		match('x' /* charlit */ );
-		{ // ( ... )+
-		int _cnt196=0;
-		for (;;) {
-			if ((_tokenSet_2.member(LA(1)))) {
-				mHEXDIGIT(false);
-			}
-			else {
-				if ( _cnt196>=1 ) { goto _loop196; } else {throw ANTLR_USE_NAMESPACE(antlr)NoViableAltForCharException(LA(1), getFilename(), getLine(), getColumn());}
-			}
-			
-			_cnt196++;
+	else {
+		bool synPredMatched189 = false;
+		if ((((LA(1) >= 0x30 /* '0' */  && LA(1) <= 0x39 /* '9' */ )) && (_tokenSet_2.member(LA(2))) && (_tokenSet_2.member(LA(3))) && (_tokenSet_2.member(LA(4))) && (_tokenSet_2.member(LA(5))) && (true) && (true) && (true) && (true) && (true) && (true) && (true) && (true) && (true) && (true) && (true) && (true) && (true) && (true) && (true))) {
+			int _m189 = mark();
+			synPredMatched189 = true;
+			inputState->guessing++;
+			try {
+				{
+				mNUM_3DIGIT(false);
+				match('.' /* charlit */ );
+				mNUM_3DIGIT(false);
+				match('.' /* charlit */ );
+				}
+			}
+			catch (ANTLR_USE_NAMESPACE(antlr)RecognitionException& pe) {
+				synPredMatched189 = false;
+			}
+			rewind(_m189);
+			inputState->guessing--;
 		}
-		_loop196:;
-		}  // ( ... )+
+		if ( synPredMatched189 ) {
+			{
+			mNUM_3DIGIT(false);
+			match('.' /* charlit */ );
+			mNUM_3DIGIT(false);
+			match('.' /* charlit */ );
+			mNUM_3DIGIT(false);
+			if ( inputState->guessing==0 ) {
+#line 1316 "iptables.g"
+				_ttype = THREE_COMPONENT_VERSION;
+#line 1066 "IPTCfgLexer.cpp"
+			}
+			}
 		}
-		if ( inputState->guessing==0 ) {
-#line 1248 "iptables.g"
-			_ttype = HEX_CONST;
-#line 1061 "IPTCfgLexer.cpp"
+		else if ((LA(1) == 0x30 /* '0' */ ) && (LA(2) == 0x78 /* 'x' */ )) {
+			{
+			match('0' /* charlit */ );
+			match('x' /* charlit */ );
+			{ // ( ... )+
+			int _cnt193=0;
+			for (;;) {
+				if ((_tokenSet_1.member(LA(1)))) {
+					mHEXDIGIT(false);
+				}
+				else {
+					if ( _cnt193>=1 ) { goto _loop193; } else {throw ANTLR_USE_NAMESPACE(antlr)NoViableAltForCharException(LA(1), getFilename(), getLine(), getColumn());}
+				}
+				
+				_cnt193++;
+			}
+			_loop193:;
+			}  // ( ... )+
+			}
+			if ( inputState->guessing==0 ) {
+#line 1320 "iptables.g"
+				_ttype = HEX_CONST;
+#line 1092 "IPTCfgLexer.cpp"
+			}
 		}
-	}
-	else if (((LA(1) >= 0x30 /* '0' */  && LA(1) <= 0x39 /* '9' */ )) && (true) && (true) && (true) && (true) && (true) && (true) && (true) && (true) && (true) && (true) && (true) && (true) && (true) && (true) && (true) && (true) && (true) && (true) && (true)) {
-		{ // ( ... )+
-		int _cnt198=0;
-		for (;;) {
-			if (((LA(1) >= 0x30 /* '0' */  && LA(1) <= 0x39 /* '9' */ ))) {
-				mDIGIT(false);
-			}
-			else {
-				if ( _cnt198>=1 ) { goto _loop198; } else {throw ANTLR_USE_NAMESPACE(antlr)NoViableAltForCharException(LA(1), getFilename(), getLine(), getColumn());}
-			}
-			
-			_cnt198++;
-		}
-		_loop198:;
-		}  // ( ... )+
-		if ( inputState->guessing==0 ) {
-#line 1250 "iptables.g"
-			_ttype = INT_CONST;
-#line 1082 "IPTCfgLexer.cpp"
+		else if (((LA(1) >= 0x30 /* '0' */  && LA(1) <= 0x39 /* '9' */ )) && (true) && (true) && (true) && (true) && (true) && (true) && (true) && (true) && (true) && (true) && (true) && (true) && (true) && (true) && (true) && (true) && (true) && (true) && (true)) {
+			{ // ( ... )+
+			int _cnt195=0;
+			for (;;) {
+				if (((LA(1) >= 0x30 /* '0' */  && LA(1) <= 0x39 /* '9' */ ))) {
+					mDIGIT(false);
+				}
+				else {
+					if ( _cnt195>=1 ) { goto _loop195; } else {throw ANTLR_USE_NAMESPACE(antlr)NoViableAltForCharException(LA(1), getFilename(), getLine(), getColumn());}
+				}
+				
+				_cnt195++;
+			}
+			_loop195:;
+			}  // ( ... )+
+			if ( inputState->guessing==0 ) {
+#line 1322 "iptables.g"
+				_ttype = INT_CONST;
+#line 1113 "IPTCfgLexer.cpp"
+			}
 		}
-	}
 	else {
 		throw ANTLR_USE_NAMESPACE(antlr)NoViableAltForCharException(LA(1), getFilename(), getLine(), getColumn());
 	}
-	
+	}
 	_ttype = testLiteralsTable(_ttype);
 	if ( _createToken && _token==ANTLR_USE_NAMESPACE(antlr)nullToken && _ttype!=ANTLR_USE_NAMESPACE(antlr)Token::SKIP ) {
 	   _token = makeToken(_ttype);
@@ -1331,11 +1362,11 @@ void IPTCfgLexer::mWORD(bool _createToken) {
 		}
 		default:
 		{
-			goto _loop202;
+			goto _loop199;
 		}
 		}
 	}
-	_loop202:;
+	_loop199:;
 	} // ( ... )*
 	if ( _createToken && _token==ANTLR_USE_NAMESPACE(antlr)nullToken && _ttype!=ANTLR_USE_NAMESPACE(antlr)Token::SKIP ) {
 	   _token = makeToken(_ttype);
@@ -1353,15 +1384,15 @@ void IPTCfgLexer::mSTRING(bool _createToken) {
 	match('\"' /* charlit */ );
 	{ // ( ... )*
 	for (;;) {
-		if ((_tokenSet_4.member(LA(1)))) {
+		if ((_tokenSet_3.member(LA(1)))) {
 			matchNot('\"' /* charlit */ );
 		}
 		else {
-			goto _loop205;
+			goto _loop202;
 		}
 		
 	}
-	_loop205:;
+	_loop202:;
 	} // ( ... )*
 	match('\"' /* charlit */ );
 	if ( _createToken && _token==ANTLR_USE_NAMESPACE(antlr)nullToken && _ttype!=ANTLR_USE_NAMESPACE(antlr)Token::SKIP ) {
@@ -1392,9 +1423,9 @@ void IPTCfgLexer::mRSOURCE(bool _createToken) {
 	
 	match("--rsource");
 	if ( inputState->guessing==0 ) {
-#line 1274 "iptables.g"
+#line 1346 "iptables.g"
 		_ttype = UNSUPPORTED_OPTION;
-#line 1398 "IPTCfgLexer.cpp"
+#line 1429 "IPTCfgLexer.cpp"
 	}
 	if ( _createToken && _token==ANTLR_USE_NAMESPACE(antlr)nullToken && _ttype!=ANTLR_USE_NAMESPACE(antlr)Token::SKIP ) {
 	   _token = makeToken(_ttype);
@@ -2055,9 +2086,9 @@ void IPTCfgLexer::mULOG_QTHR(bool _createToken) {
 	
 	match("--ulog-qthreshold");
 	if ( inputState->guessing==0 ) {
-#line 1341 "iptables.g"
+#line 1413 "iptables.g"
 		_ttype = UNSUPPORTED_OPTION;
-#line 2061 "IPTCfgLexer.cpp"
+#line 2092 "IPTCfgLexer.cpp"
 	}
 	if ( _createToken && _token==ANTLR_USE_NAMESPACE(antlr)nullToken && _ttype!=ANTLR_USE_NAMESPACE(antlr)Token::SKIP ) {
 	   _token = makeToken(_ttype);
@@ -2074,9 +2105,9 @@ void IPTCfgLexer::mULOG_NLG(bool _createToken) {
 	
 	match("--ulog-nlgroup");
 	if ( inputState->guessing==0 ) {
-#line 1342 "iptables.g"
+#line 1414 "iptables.g"
 		_ttype = UNSUPPORTED_OPTION;
-#line 2080 "IPTCfgLexer.cpp"
+#line 2111 "IPTCfgLexer.cpp"
 	}
 	if ( _createToken && _token==ANTLR_USE_NAMESPACE(antlr)nullToken && _ttype!=ANTLR_USE_NAMESPACE(antlr)Token::SKIP ) {
 	   _token = makeToken(_ttype);
@@ -2093,9 +2124,9 @@ void IPTCfgLexer::mULOG_CPR(bool _createToken) {
 	
 	match("--ulog-cprange");
 	if ( inputState->guessing==0 ) {
-#line 1343 "iptables.g"
+#line 1415 "iptables.g"
 		_ttype = UNSUPPORTED_OPTION;
-#line 2099 "IPTCfgLexer.cpp"
+#line 2130 "IPTCfgLexer.cpp"
 	}
 	if ( _createToken && _token==ANTLR_USE_NAMESPACE(antlr)nullToken && _ttype!=ANTLR_USE_NAMESPACE(antlr)Token::SKIP ) {
 	   _token = makeToken(_ttype);
@@ -2670,27 +2701,20 @@ const unsigned long IPTCfgLexer::_tokenSet_0_data_[] = { 4294958072UL, 1UL, 0UL,
 // 0x3 0x4 0x5 0x6 0x7 0x8 0x9 0xb 0xc 0xe 0xf 0x10 0x11 0x12 0x13 0x14 
 // 0x15 0x16 0x17 0x18 0x19 0x1a 0x1b 0x1c 0x1d 0x1e 0x1f   0x7f 0x80 0x81 
 // 0x82 0x83 0x84 0x85 0x86 0x87 0x88 0x89 0x8a 0x8b 0x8c 0x8d 0x8e 0x8f 
+// 0x90 
 const ANTLR_USE_NAMESPACE(antlr)BitSet IPTCfgLexer::_tokenSet_0(_tokenSet_0_data_,16);
-const unsigned long IPTCfgLexer::_tokenSet_1_data_[] = { 4294958072UL, 4294967295UL, 4294967295UL, 4294967295UL, 4294967295UL, 4294967295UL, 4294967295UL, 4294967295UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL };
-// 0x3 0x4 0x5 0x6 0x7 0x8 0x9 0xb 0xc 0xe 0xf 0x10 0x11 0x12 0x13 0x14 
-// 0x15 0x16 0x17 0x18 0x19 0x1a 0x1b 0x1c 0x1d 0x1e 0x1f   ! \" # $ % 
-// & \' ( ) * + , - . / 0 1 2 3 4 5 6 7 8 9 : ; < = > ? @ A B C D E F G 
-// H I J K L M N O P Q R S T U V W X Y Z [ 0x5c ] ^ _ ` a b c d e f g h 
-// i j k l m n o p q r s t u v w x y z { | } ~ 0x7f 0x80 0x81 0x82 0x83 
-// 0x84 0x85 0x86 0x87 0x88 0x89 0x8a 0x8b 0x8c 0x8d 0x8e 0x8f 
-const ANTLR_USE_NAMESPACE(antlr)BitSet IPTCfgLexer::_tokenSet_1(_tokenSet_1_data_,16);
-const unsigned long IPTCfgLexer::_tokenSet_2_data_[] = { 0UL, 67043328UL, 126UL, 126UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL };
+const unsigned long IPTCfgLexer::_tokenSet_1_data_[] = { 0UL, 67043328UL, 126UL, 126UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL };
 // 0 1 2 3 4 5 6 7 8 9 A B C D E F a b c d e f 
-const ANTLR_USE_NAMESPACE(antlr)BitSet IPTCfgLexer::_tokenSet_2(_tokenSet_2_data_,10);
-const unsigned long IPTCfgLexer::_tokenSet_3_data_[] = { 0UL, 67059712UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL };
+const ANTLR_USE_NAMESPACE(antlr)BitSet IPTCfgLexer::_tokenSet_1(_tokenSet_1_data_,10);
+const unsigned long IPTCfgLexer::_tokenSet_2_data_[] = { 0UL, 67059712UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL };
 // . 0 1 2 3 4 5 6 7 8 9 
-const ANTLR_USE_NAMESPACE(antlr)BitSet IPTCfgLexer::_tokenSet_3(_tokenSet_3_data_,10);
-const unsigned long IPTCfgLexer::_tokenSet_4_data_[] = { 4294967288UL, 4294967291UL, 4294967295UL, 4294967295UL, 4294967295UL, 4294967295UL, 4294967295UL, 4294967295UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL };
+const ANTLR_USE_NAMESPACE(antlr)BitSet IPTCfgLexer::_tokenSet_2(_tokenSet_2_data_,10);
+const unsigned long IPTCfgLexer::_tokenSet_3_data_[] = { 4294967288UL, 4294967291UL, 4294967295UL, 4294967295UL, 4294967295UL, 4294967295UL, 4294967295UL, 4294967295UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL };
 // 0x3 0x4 0x5 0x6 0x7 0x8 0x9 0xa 0xb 0xc 0xd 0xe 0xf 0x10 0x11 0x12 0x13 
 // 0x14 0x15 0x16 0x17 0x18 0x19 0x1a 0x1b 0x1c 0x1d 0x1e 0x1f   ! # $ 
 // % & \' ( ) * + , - . / 0 1 2 3 4 5 6 7 8 9 : ; < = > ? @ A B C D E F 
 // G H I J K L M N O P Q R S T U V W X Y Z [ 0x5c ] ^ _ ` a b c d e f g 
 // h i j k l m n o p q r s t u v w x y z { | } ~ 0x7f 0x80 0x81 0x82 0x83 
-// 0x84 0x85 0x86 0x87 0x88 0x89 0x8a 0x8b 0x8c 0x8d 0x8e 0x8f 
-const ANTLR_USE_NAMESPACE(antlr)BitSet IPTCfgLexer::_tokenSet_4(_tokenSet_4_data_,16);
+// 0x84 0x85 0x86 0x87 0x88 0x89 0x8a 0x8b 0x8c 0x8d 0x8e 0x8f 0x90 
+const ANTLR_USE_NAMESPACE(antlr)BitSet IPTCfgLexer::_tokenSet_3(_tokenSet_3_data_,16);
 
