@@ -43,42 +43,45 @@
 #include "NATCompiler_pf.h"
 #include "TableFactory.h"
 #include "Preprocessor_pf.h"
+#include "RoutingCompiler_openbsd.h"
+#include "RoutingCompiler_freebsd.h"
+#include "AutomaticRules_pf.h"
 
 #include "OSConfigurator_openbsd.h"
 #include "OSConfigurator_freebsd.h"
 #include "OSConfigurator_solaris.h"
 
-#include "fwbuilder/Resources.h"
-#include "fwbuilder/FWObjectDatabase.h"
-#include "fwbuilder/XMLTools.h"
-#include "fwbuilder/FWException.h"
-#include "fwbuilder/Firewall.h"
-#include "fwbuilder/Interface.h"
-#include "fwbuilder/Policy.h"
-#include "fwbuilder/NAT.h"
-
-#include "fwcompiler/Preprocessor.h"
-
-#include "fwbuilder/Resources.h"
-#include "fwbuilder/FWObjectDatabase.h"
-#include "fwbuilder/FWException.h"
 #include "fwbuilder/Cluster.h"
 #include "fwbuilder/ClusterGroup.h"
+#include "fwbuilder/FWException.h"
+#include "fwbuilder/FWObjectDatabase.h"
+#include "fwbuilder/FailoverClusterGroup.h"
 #include "fwbuilder/Firewall.h"
 #include "fwbuilder/Interface.h"
+#include "fwbuilder/Library.h"
+#include "fwbuilder/NAT.h"
 #include "fwbuilder/Policy.h"
+#include "fwbuilder/Resources.h"
+#include "fwbuilder/Routing.h"
 #include "fwbuilder/StateSyncClusterGroup.h"
-#include "fwbuilder/FailoverClusterGroup.h"
+#include "fwbuilder/XMLTools.h"
+
+#include "fwcompiler/Preprocessor.h"
+#include "fwcompiler/exceptions.h"
 
 #include <QStringList>
 #include <QFileInfo>
 #include <QFile>
 #include <QTextStream>
+#include <QtDebug>
+
 
 using namespace std;
 using namespace libfwbuilder;
 using namespace fwcompiler;
 
+// #define DEBUG_FILE_NAMES 1
+
 
 QString CompilerDriver_pf::composeActivationCommand(Firewall *fw,
                                                     const string &pfctl_debug,
@@ -86,7 +89,11 @@ QString CompilerDriver_pf::composeActivationCommand(Firewall *fw,
                                                     const string &pf_version,
                                                     const string &remote_file_name)
 {
-    Configlet act(fw, "pf", "activation");
+    FWOptions* options = fw->getOptionsObject();
+    Configlet act(fw, "pf",
+                  options->getBool("generate_rc_conf_file") ?
+                  "rc_conf_activation" : "activation");
+    
     act.removeComments();
     act.setVariable("pfctl_debug", pfctl_debug.c_str());
     act.setVariable("anchor", !anchor_name.empty());
@@ -110,55 +117,47 @@ QString CompilerDriver_pf::printActivationCommands(Firewall *fw)
     bool debug = options->getBool("debug");
     string pfctl_dbg = (debug)?"-v ":"";
 
+    QString remote_file_name = escapeFileName(remote_file_names[CONF1_FILE]);
+
+    return composeActivationCommand(
+        fw, pfctl_dbg, "",
+        fw->getStr("version"), remote_file_name.toUtf8().constData());
+
+#if 0
     QStringList activation_commands;
-    QString remote_file = remote_conf_files["__main__"].c_str();
-    if (remote_file.isEmpty()) remote_file = conf_files["__main__"].c_str();
-    if (remote_file[0] != '/') remote_file = "${FWDIR}/" + remote_file;
-    remote_file = this->escapeFileName(remote_file);
 
-    activation_commands.push_back(
-        composeActivationCommand(
-            fw, pfctl_dbg, "", fw->getStr("version"), remote_file.toStdString()));
-
-    for (map<string,string>::iterator i=conf_files.begin();
-         i!=conf_files.end(); ++i)
+    // skip first item in the list since it is .fw script
+    for(int idx=1; idx<file_names.size(); idx++)
     {
-        QString remote_file = remote_conf_files[i->first].c_str();
-        if (remote_file.isEmpty()) remote_file = i->second.c_str();
-        if (remote_file[0] != '/') remote_file = "${FWDIR}/" + remote_file;
-        remote_file = this->escapeFileName(remote_file);
+        QString remote_file_name = escapeFileName(remote_file_names[idx]);
 
-        if (i->first != "__main__")
-            activation_commands.push_back(
-                composeActivationCommand(
-                    fw, pfctl_dbg, i->first, fw->getStr("version"), remote_file.toStdString()));
+        activation_commands.push_back(
+            composeActivationCommand(
+                fw, pfctl_dbg, anchor_names[idx].toUtf8().constData(),
+                fw->getStr("version"), remote_file_name.toUtf8().constData()));
     }
 
     return activation_commands.join("\n");
+#endif
 }
 
-QString CompilerDriver_pf::assembleManifest(Cluster*, Firewall* fw, bool )
+QString CompilerDriver_pf::assembleManifest(Cluster*, Firewall* , bool )
 {
-    QFileInfo fw_file_info(fw_file_name);
     QString script_buffer;
     QTextStream script(&script_buffer, QIODevice::WriteOnly);
 
-    script << MANIFEST_MARKER << "* " << this->escapeFileName(fw_file_info.fileName());
-    string remote_name = fw->getOptionsObject()->getStr("script_name_on_firewall");
-    if (!remote_name.empty()) script << " " << this->escapeFileName(remote_name.c_str());
-    script << "\n";
-
-    for (map<string,string>::iterator i=conf_files.begin();
-         i!=conf_files.end(); ++i)
+    for(int idx=0; idx<file_names.size(); idx++)
     {
-        string ruleset_name = i->first;
-        QString file_name = QFileInfo(i->second.c_str()).fileName();
-        QString remote_file_name = remote_conf_files[ruleset_name].c_str();
-        script << MANIFEST_MARKER << "  " << this->escapeFileName(file_name);
-        if (!remote_file_name.isEmpty() && remote_file_name != file_name)
-            script << " " << this->escapeFileName(remote_file_name);
+        QString master_file_marker = (idx==0) ? "* " : "  ";
+        QString local_file_name = file_names[idx];
+        QString remote_file_name = remote_file_names[idx];
+        script << MANIFEST_MARKER << master_file_marker
+               << escapeFileName(local_file_name);
+        if (!remote_file_name.isEmpty() && remote_file_name != local_file_name)
+            script << " " << escapeFileName(remote_file_name);
         script << "\n";
     }
+
     return script_buffer;
 }
 
@@ -168,17 +167,31 @@ QString CompilerDriver_pf::assembleFwScript(Cluster *cluster,
                                             OSConfigurator *oscnf)
 {
     FWOptions* options = fw->getOptionsObject();
-    Configlet script_skeleton(fw, "pf", "script_skeleton");
-    Configlet top_comment(fw, "pf", "top_comment");
+
+    Configlet script_skeleton(
+        fw, "pf",
+        options->getBool("generate_rc_conf_file") ?
+            "rc_conf_skeleton" : "script_skeleton");
+
+    Configlet top_comment(fw, "pf",
+                          options->getBool("generate_rc_conf_file") ?
+                          "rc_conf_top_comment" : "top_comment");
+
+    script_skeleton.setVariable("routing_script", 
+                                QString::fromUtf8(routing_script.c_str()));
 
     assembleFwScriptInternal(
-        cluster, fw, cluster_member, oscnf, &script_skeleton, &top_comment, "#");
+        cluster, fw, cluster_member, oscnf,
+        &script_skeleton, &top_comment, "#",
+        !options->getBool("generate_rc_conf_file"));
 
     if (fw->getStr("platform") == "pf")
     {
-        script_skeleton.setVariable("pf_flush_states", options->getBool("pf_flush_states"));
-        script_skeleton.setVariable("pf_version_ge_4_x", // fw->getStr("version")=="4.x");
-                                    XMLTools::version_compare(fw->getStr("version"), "4.0")>=0);
+        script_skeleton.setVariable(
+            "pf_flush_states", options->getBool("pf_flush_states"));
+        script_skeleton.setVariable(
+            "pf_version_ge_4_x", // fw->getStr("version")=="4.x");
+            XMLTools::version_compare(fw->getStr("version"), "4.0")>=0);
 
     } else
     {
@@ -194,16 +207,14 @@ QString CompilerDriver_pf::run(const std::string &cluster_id,
                                const std::string &single_rule_id)
 {
     Cluster *cluster = NULL;
-    if (!cluster_id.empty())
-        cluster = Cluster::cast(
-            objdb->findInIndex(objdb->getIntId(cluster_id)));
+    Firewall *fw = NULL;
 
-    Firewall *fw = Firewall::cast(
-        objdb->findInIndex(objdb->getIntId(firewall_id)));
-    assert(fw);
+    getFirewallAndClusterObjects(cluster_id, firewall_id, &cluster, &fw);
 
     try
     {
+        clearReadOnly(fw);
+
         // Copy rules from the cluster object
         populateClusterElements(cluster, fw);
 
@@ -213,9 +224,7 @@ QString CompilerDriver_pf::run(const std::string &cluster_id,
 
         // Note that fwobjectname may be different from the name of the
         // firewall fw This happens when we compile a member of a cluster
-        current_firewall_name = fw->getName().c_str();
-
-        fw_file_name = determineOutputFileName(cluster, fw, !cluster_id.empty(), ".fw");
+        current_firewall_name = QString::fromUtf8(fw->getName().c_str());
 
         string firewall_dir = options->getStr("firewall_dir");
         if (firewall_dir=="") firewall_dir="/etc/fw";
@@ -236,8 +245,8 @@ QString CompilerDriver_pf::run(const std::string &cluster_id,
         string platform = fw->getStr("platform");
         string fw_version = fw->getStr("version");
         string host_os = fw->getStr("host_OS");
-        string family = Resources::os_res[host_os
-        ]->Resources::getResourceStr("/FWBuilderResources/Target/family");
+        string family = Resources::os_res[host_os]->
+            Resources::getResourceStr("/FWBuilderResources/Target/family");
 
         if (host_os == "solaris")
             oscnf = std::auto_ptr<OSConfigurator_bsd>(new OSConfigurator_solaris(
@@ -259,8 +268,10 @@ QString CompilerDriver_pf::run(const std::string &cluster_id,
 
         oscnf->prolog();
 
-        string remote_fw_name = options->getStr("script_name_on_firewall");
-        string remote_conf_name = options->getStr("conf_file_name_on_firewall");
+        QString remote_fw_name = QString::fromUtf8(
+            options->getStr("script_name_on_firewall").c_str());
+        QString remote_conf_name = QString::fromUtf8(
+            options->getStr("conf_file_name_on_firewall").c_str());
 
         list<FWObject*> all_policies = fw->getByType(Policy::TYPENAME);
         list<FWObject*> all_nat = fw->getByType(NAT::TYPENAME);
@@ -268,6 +279,126 @@ QString CompilerDriver_pf::run(const std::string &cluster_id,
         findImportedRuleSets(fw, all_policies);
         findImportedRuleSets(fw, all_nat);
 
+        try
+        {
+            AutomaticRules_pf auto_rules(fw, persistent_objects);
+            auto_rules.addSshAccessRule();
+            auto_rules.addCarpRules();
+            auto_rules.addPfsyncRules();
+            auto_rules.addFallbackRule();
+        } catch (FWException &ex)
+        {
+            abort(ex.toString());
+        }
+
+        // assign unique rule ids that later will be used to generate
+        // chain names.  This should be done after calls to
+        // findImportedRuleSets()
+        // NB: these ids are not really used by compiler for PF
+
+        assignUniqueRuleIds(all_policies);
+        assignUniqueRuleIds(all_nat);
+
+        list<FWObject*> all_rulesets;
+        all_rulesets.insert(
+            all_rulesets.begin(), all_policies.begin(), all_policies.end());
+        all_rulesets.insert(
+            all_rulesets.begin(), all_nat.begin(), all_nat.end());
+
+        // establish mapping of rule sets to file names so it can be used
+        // for "load anchor" commands
+        
+        QMap<QString, QString> rulesets_to_file_names;
+        QMap<QString, QString> rulesets_to_remote_file_names;
+        QMap<QString, int> rulesets_to_indexes;
+        QStringList file_extensions;
+        QStringList remote_file_options;
+
+        anchor_names.clear();
+
+        anchor_names << ""; // for fw_file
+        anchor_names << ""; // for main .conf file (both policy and nat top rule sets)
+
+        // Can not make extension .conf when generating rc.conf file
+        // because the second file also has extension .conf and this
+        // causes conflict if both names are generated using default
+        // algorithm from the fw name
+        //
+
+        file_extensions << "fw";
+        file_extensions << "conf";
+
+        remote_file_options << "script_name_on_firewall";
+        remote_file_options << "conf_file_name_on_firewall";
+
+        rulesets_to_indexes["__main__"] = CONF1_FILE;
+
+        int idx = CONF2_FILE;
+        for (list<FWObject*>::iterator p=all_rulesets.begin();
+             p!=all_rulesets.end(); ++p)
+        {
+            RuleSet *rs = RuleSet::cast(*p);
+            QString ruleset_name = QString::fromUtf8(rs->getName().c_str());
+
+            if (ruleset_name.endsWith("/*"))
+            {
+                QString err("The name of the %1 ruleset %2"
+                            " ends with '/*', assuming it is externally"
+                            " controlled and skipping it.");
+                warning(fw, rs, NULL,
+                        err.arg(rs->getTypeName().c_str())
+                        .arg(ruleset_name).toStdString());
+                rs->setBool(".skip_ruleset", true);
+                continue;
+            }
+
+            if (rs->isTop()) continue;
+
+            // record index of this ruleset in file_names and remote_file_names
+            if (rulesets_to_indexes.count(ruleset_name) == 0)
+            {
+                anchor_names << ruleset_name;
+                file_extensions << "conf";
+                remote_file_options << ""; // to make sure it has right number of items
+                rulesets_to_indexes[ruleset_name] = idx;
+                idx++;
+            }
+        }
+
+#ifdef DEBUG_FILE_NAMES
+        qDebug() << "anchor_names=" << anchor_names;
+        qDebug() << "file_extensions=" << file_extensions;
+        qDebug() << "remote_file_options=" << remote_file_options;
+#endif
+
+        // The order of file names in file_names and remote_file_names
+        // is the same as the order of rule sets in all_rulesets
+        determineOutputFileNames(cluster, fw, !cluster_id.empty(),
+                                 anchor_names, file_extensions,
+                                 remote_file_options);
+
+
+        for (list<FWObject*>::iterator p=all_rulesets.begin();
+             p!=all_rulesets.end(); ++p)
+        {
+            RuleSet *rs = RuleSet::cast(*p);
+            if (rs->getBool(".skip_ruleset")) continue;
+            QString ruleset_name = QString::fromUtf8(rs->getName().c_str());
+            if (rs->isTop()) ruleset_name = "__main__";
+            int idx = rulesets_to_indexes[ruleset_name];
+            rulesets_to_file_names[ruleset_name] = file_names[idx];
+            rulesets_to_remote_file_names[ruleset_name] = remote_file_names[idx];
+        }
+
+#ifdef DEBUG_FILE_NAMES
+        qDebug() << "file_names=" << file_names;
+        qDebug() << "remote_file_names=" << remote_file_names;
+        qDebug() << "rulesets_to_file_names=" << rulesets_to_file_names;
+        qDebug() << "rulesets_to_remote_file_names=" << rulesets_to_remote_file_names;
+#endif
+
+        int routing_rules_count = 0;
+
         vector<int> ipv4_6_runs;
 
         // command line options -4 and -6 control address family for which
@@ -332,32 +463,24 @@ QString CompilerDriver_pf::run(const std::string &cluster_id,
                 NAT *nat = NAT::cast(*p);
 
                 if (!nat->matchingAddressFamily(policy_af)) continue;
+                if (nat->getBool(".skip_ruleset")) continue;
 
-                string ruleset_name = nat->getName();
-
-                if (ruleset_name.find("/*")!=string::npos)
-                {
-                    QString err("The name of the policy ruleset %1"
-                                " ends with '/*', assuming it is externally"
-                                " controlled and skipping it.");
-                    warning(fw, nat, NULL,
-                            err.arg(ruleset_name.c_str()).toStdString());
-                    continue;
-                }
-
-                if (nat->isTop())
-                    ruleset_name = "__main__";
+                QString ruleset_name = QString::fromUtf8(nat->getName().c_str());
+                if (nat->isTop()) ruleset_name = "__main__";
 
                 if (table_factories.count(ruleset_name) == 0)
                 {
-                    table_factories[ruleset_name] = new fwcompiler::TableFactory(this);
+                    table_factories[ruleset_name] =
+                        new fwcompiler::TableFactory(this, persistent_objects);
                 }
 
                 NATCompiler_pf n( objdb, fw, ipv6_policy, oscnf.get(),
-                                  table_factories[ruleset_name] );
+                                  table_factories[ruleset_name]
+                );
 
                 n.setSourceRuleSet( nat );
                 n.setRuleSetName(nat->getName());
+                n.setPersistentObjects(persistent_objects);
 
                 n.setSingleRuleCompileMode(single_rule_id);
                 n.setDebugLevel( dl );
@@ -402,56 +525,38 @@ QString CompilerDriver_pf::run(const std::string &cluster_id,
 
                 all_errors.push_back(n.getErrors("").c_str());
 
-                conf_files[ruleset_name] = getConfFileName(
-                    ruleset_name,
-                    current_firewall_name.toUtf8().constData(),
-                    fw_file_name.toUtf8().constData());
-
-
-                remote_conf_files[ruleset_name] = getRemoteConfFileName(
-                    ruleset_name,
-                    conf_files[ruleset_name],
-                    remote_fw_name,
-                    remote_conf_name);
-
                 const list<NATCompiler_pf::redirectRuleInfo> lst = 
                     n.getRedirRulesInfo();
                 redirect_rules_info.insert(redirect_rules_info.begin(),
                                            lst.begin(), lst.end());
             }
 
+
             for (list<FWObject*>::iterator p=all_policies.begin();
                  p!=all_policies.end(); ++p )
             {
                 Policy *policy = Policy::cast(*p);
-                string ruleset_name = policy->getName();
-
-                if (ruleset_name.find("/*")!=string::npos)
-                {
-                    QString err("The name of the policy ruleset %1"
-                                " ends with '/*', assuming it is externally"
-                                " controlled and skipping it.");
-                    warning(fw, policy, NULL,
-                            err.arg(ruleset_name.c_str()).toStdString());
-                    continue;
-                }
 
                 if (!policy->matchingAddressFamily(policy_af)) continue;
+                if (policy->getBool(".skip_ruleset")) continue;
 
-                if (policy->isTop())
-                    ruleset_name = "__main__";
+                QString ruleset_name = QString::fromUtf8(policy->getName().c_str());
+                if (policy->isTop()) ruleset_name = "__main__";
 
                 if (table_factories.count(ruleset_name) == 0)
                 {
-                    table_factories[ruleset_name] = new fwcompiler::TableFactory(this);
+                    table_factories[ruleset_name] =
+                        new fwcompiler::TableFactory(this, persistent_objects);
                 }
 
                 PolicyCompiler_pf c( objdb, fw, ipv6_policy, oscnf.get(),
                                      &redirect_rules_info,
-                                     table_factories[ruleset_name] );
+                                     table_factories[ruleset_name]
+                );
 
                 c.setSourceRuleSet( policy );
                 c.setRuleSetName(policy->getName());
+                c.setPersistentObjects(persistent_objects);
 
                 c.setSingleRuleCompileMode(single_rule_id);
                 c.setDebugLevel( dl );
@@ -470,7 +575,7 @@ QString CompilerDriver_pf::run(const std::string &cluster_id,
 
                 if (policy->isTop())
                 {
-                    generated_scripts[ruleset_name] = main_str;
+                    generated_scripts["__main__"] = main_str;
                 } else
                 {
                     generated_scripts[ruleset_name] = new ostringstream();
@@ -494,20 +599,59 @@ QString CompilerDriver_pf::run(const std::string &cluster_id,
 
                 all_errors.push_back(c.getErrors("").c_str());
 
-                conf_files[ruleset_name] = getConfFileName(
-                    ruleset_name,
-                    current_firewall_name.toUtf8().constData(),
-                    fw_file_name.toUtf8().constData());
-
-                remote_conf_files[ruleset_name] = getRemoteConfFileName(
-                    ruleset_name,
-                    conf_files[ruleset_name],
-                    remote_fw_name,
-                    remote_conf_name);
-
             }
+
         }
 
+        std::auto_ptr<RoutingCompiler> routing_compiler;
+
+        if (host_os == "openbsd")
+            routing_compiler = std::auto_ptr<RoutingCompiler>(
+                new RoutingCompiler_openbsd(objdb, fw, false, oscnf.get()));
+
+        if (host_os == "freebsd")
+            routing_compiler = std::auto_ptr<RoutingCompiler>(
+                new RoutingCompiler_freebsd(objdb, fw, false, oscnf.get()));
+
+        if (routing_compiler.get() == NULL)
+        {
+            abort("Unrecognized host OS " + host_os + "  (family " + family + ")");
+            return "";
+        }
+
+        RuleSet *routing = RuleSet::cast(fw->getFirstByType(Routing::TYPENAME));
+        if (routing)
+        {
+            routing_compiler->setSourceRuleSet(routing);
+            routing_compiler->setRuleSetName(routing->getName());
+            routing_compiler->setPersistentObjects(persistent_objects);
+
+            routing_compiler->setSingleRuleCompileMode(single_rule_id);
+            routing_compiler->setDebugLevel( dl );
+            if (rule_debug_on) routing_compiler->setDebugRule(drp);
+            routing_compiler->setVerbose( verbose );
+            if (inTestMode()) routing_compiler->setTestMode();
+            if (inEmbeddedMode()) routing_compiler->setEmbeddedMode();
+
+            if ( (routing_rules_count=routing_compiler->prolog()) > 0 )
+            {
+                routing_compiler->compile();
+                routing_compiler->epilog();
+            }
+
+            if (routing_compiler->haveErrorsAndWarnings())
+                all_errors.push_back(routing_compiler->getErrors("").c_str());
+
+            routing_script += routing_compiler->getCompiledScript();
+        }
+
+        /*
+         * compilers detach persistent objects when they finish, this
+         * means at this point library persistent_objects is not part
+         * of any object tree.
+         */
+        objdb->reparent(persistent_objects);
+
         if (haveErrorsAndWarnings())
         {
             all_errors.push_front(getErrors("").c_str());
@@ -519,13 +663,14 @@ QString CompilerDriver_pf::run(const std::string &cluster_id,
             QString buffer;
             QTextStream pf_str(&buffer);
 
-            for (map<string, ostringstream*>::iterator fi=generated_scripts.begin();
+            for (map<QString, ostringstream*>::iterator fi=generated_scripts.begin();
                  fi!=generated_scripts.end(); fi++)
             {
-                string ruleset_name = fi->first;
+                QString ruleset_name = fi->first;
                 ostringstream *strm = fi->second;
                 pf_str << table_factories[ruleset_name]->PrintTables();
                 pf_str << QString::fromUtf8(strm->str().c_str());
+                pf_str << QString::fromUtf8(routing_script.c_str());
             }
 
             // clear() calls destructors of all elements in the container
@@ -535,17 +680,39 @@ QString CompilerDriver_pf::run(const std::string &cluster_id,
             return formSingleRuleCompileOutput(buffer);
         }
 
-/*
- * now write generated scripts to files
- */
-        for (map<string, ostringstream*>::iterator fi=generated_scripts.begin();
+        /* add commands to load anchors to the bottom of the main .conf file */
+        QMap<QString, QString>::iterator it;
+        for (it=rulesets_to_remote_file_names.begin();
+             it!=rulesets_to_remote_file_names.end(); ++it)
+        {
+            QString ruleset_name = it.key();
+            if (ruleset_name == "__main__") continue;
+            QString remote_file_name = it.value();
+            ostringstream *ostr = generated_scripts["__main__"];
+            // note that ostr can be NULL if the firewall we are
+            // trying to compile has no top-level rule sets
+            if (ostr == NULL) continue;
+            *ostr << QString("load anchor %1 from \"%2\"")
+                .arg(ruleset_name).arg(remote_file_name).toUtf8().constData()
+                                             << endl;
+        }
+
+
+        /*
+         * now write generated scripts to files
+         */
+
+        idx = CONF1_FILE;
+        for (map<QString, ostringstream*>::iterator fi=generated_scripts.begin();
              fi!=generated_scripts.end(); fi++)
         {
-            string ruleset_name = fi->first;
-            QString file_name = conf_files[ruleset_name].c_str();
+            QString ruleset_name = fi->first;
+            QString file_name = rulesets_to_file_names[ruleset_name]; // file_names[idx];
             ostringstream *strm = fi->second;
 
-            if (ruleset_name.find("/*")!=string::npos) continue;
+            if (strm==NULL) continue;
+
+            if (ruleset_name.contains("/*")) continue;
 
             file_name = getAbsOutputFileName(file_name);
 
@@ -558,7 +725,13 @@ QString CompilerDriver_pf::run(const std::string &cluster_id,
                 if (ruleset_name == "__main__")
                 {
                     printStaticOptions(pf_str, fw);
+
+                    // attach persistent_tables subtree inside TableFactory object
+                    // to the object tree
+                    table_factories[ruleset_name]->init(objdb);
+
                     pf_str << table_factories[ruleset_name]->PrintTables();
+
                     if (prolog_place == "pf_file_after_tables")
                         printProlog(pf_str, pre_hook);
                 } else 
@@ -574,10 +747,14 @@ QString CompilerDriver_pf::run(const std::string &cluster_id,
                 table_factories.clear();
                 generated_scripts.clear();
 
-                QString err(" Failed to open file %1 for writing: %2; Current dir: %3");
-                abort(err.arg(pf_file.fileName()).arg(pf_file.error()).arg(QDir::current().path()).toStdString());
+                QString err("Failed to open file %1 for writing: %2; "
+                            "Current dir: %3");
+                abort(err.arg(pf_file.fileName())
+                      .arg(pf_file.error())
+                      .arg(QDir::current().path()).toStdString());
             }
 
+            idx++;
         }
 /*
  * assemble the script and then perhaps post-process it if needed
@@ -589,10 +766,11 @@ QString CompilerDriver_pf::run(const std::string &cluster_id,
         table_factories.clear();
         generated_scripts.clear();
 
-        fw_file_name = getAbsOutputFileName(fw_file_name);
 
-        info("Output file name: " + fw_file_name.toStdString());
-        QFile fw_file(fw_file_name);
+        file_names[FW_FILE] = getAbsOutputFileName(file_names[FW_FILE]);
+
+        info("Output file name: " + file_names[FW_FILE].toStdString());
+        QFile fw_file(file_names[FW_FILE]);
         if (fw_file.open(QIODevice::WriteOnly))
         {
             QTextStream fw_str(&fw_file);
@@ -608,12 +786,14 @@ QString CompilerDriver_pf::run(const std::string &cluster_id,
         } else
         {
             QString err(" Failed to open file %1 for writing: %2; Current dir: %3");
-            abort(err.arg(fw_file.fileName()).arg(fw_file.error()).arg(QDir::current().path()).toStdString());
+            abort(err.arg(fw_file.fileName())
+                  .arg(fw_file.error()).arg(QDir::current().path()).toStdString());
         }
     }
-    catch (FatalErrorInSingleRuleCompileMode &ex)
+    catch (FWException &ex)
     {
-        return QString::fromUtf8(getErrors("").c_str());
+        status = BaseCompiler::FWCOMPILER_ERROR;
+        return QString::fromUtf8(ex.toString().c_str());
     }
 
     return "";
@@ -626,10 +806,10 @@ MapOstringStream::~MapOstringStream()
 
 void MapOstringStream::clear()
 {
-    std::map<std::string, std::ostringstream*>::iterator it;
+    std::map<QString, std::ostringstream*>::iterator it;
     for (it=begin(); it!=end(); ++it)
         delete it->second;
-    std::map<std::string, std::ostringstream*>::clear();
+    std::map<QString, std::ostringstream*>::clear();
 }
 
 MapTableFactory::~MapTableFactory()
@@ -639,9 +819,9 @@ MapTableFactory::~MapTableFactory()
 
 void MapTableFactory::clear()
 {
-    std::map<std::string, fwcompiler::TableFactory*>::iterator it;
+    std::map<QString, fwcompiler::TableFactory*>::iterator it;
     for (it=begin(); it!=end(); ++it)
         delete it->second;
-    std::map<std::string, fwcompiler::TableFactory*>::clear();
+    std::map<QString, fwcompiler::TableFactory*>::clear();
 }
 
