@@ -29,21 +29,22 @@
 
 #include "fwcompiler/OSConfigurator.h"
 
-#include "fwbuilder/RuleElement.h"
-#include "fwbuilder/NAT.h"
 #include "fwbuilder/AddressRange.h"
-#include "fwbuilder/IPService.h"
+#include "fwbuilder/AddressTable.h"
+#include "fwbuilder/Cluster.h"
+#include "fwbuilder/FailoverClusterGroup.h"
+#include "fwbuilder/Firewall.h"
+#include "fwbuilder/Host.h"
 #include "fwbuilder/ICMPService.h"
+#include "fwbuilder/IPService.h"
+#include "fwbuilder/IPv4.h"
+#include "fwbuilder/Interface.h"
+#include "fwbuilder/Library.h"
+#include "fwbuilder/NAT.h"
+#include "fwbuilder/Network.h"
+#include "fwbuilder/RuleElement.h"
 #include "fwbuilder/TCPService.h"
 #include "fwbuilder/UDPService.h"
-#include "fwbuilder/Host.h"
-#include "fwbuilder/Network.h"
-#include "fwbuilder/Interface.h"
-#include "fwbuilder/IPv4.h"
-#include "fwbuilder/Firewall.h"
-#include "fwbuilder/AddressTable.h"
-#include "fwbuilder/FailoverClusterGroup.h"
-#include "fwbuilder/Cluster.h"
 
 #include <iostream>
 #include <iomanip>
@@ -94,7 +95,7 @@ int NATCompiler_pf::prolog()
 
     IPv4::cast(loopback_address)->setAddress(InetAddr::getLoopbackAddr());
 
-    dbcopy->add(loopback_address,false);
+    persistent_objects->add(loopback_address,false);
 
     if (tables)
     {
@@ -108,12 +109,14 @@ int NATCompiler_pf::prolog()
 
 string NATCompiler_pf::debugPrintRule(libfwbuilder::Rule *r)
 {
-    NATRule   *rule=NATRule::cast(r);
-    FWObject *rule_iface = dbcopy->findInIndex(rule->getInterfaceId());
+    NATRule *rule = NATRule::cast(r);
+    RuleElementItfOutb *itf_re = rule->getItfOutb();
+    FWObject *rule_iface = FWObjectReference::getObject(itf_re->front());
+    // FWObject *rule_iface = dbcopy->findInIndex(rule->getInterfaceId());
 
-    return NATCompiler::debugPrintRule(rule)+
-        " "+string( (rule_iface!=NULL)?rule_iface->getName():"") +
-        " (type="+rule->getRuleTypeAsString()+")";
+    return NATCompiler::debugPrintRule(rule) +
+        " " + string( (rule_iface!=NULL)?rule_iface->getName():"") +
+        " (type=" + rule->getRuleTypeAsString() + ")";
 }
 
 bool NATCompiler_pf::NATRuleType::processNext()
@@ -277,7 +280,10 @@ bool NATCompiler_pf::splitSDNATRule::processNext()
         odst=r->getODst();
         odst->clearChildren();
         for (FWObject::iterator i=rule->getTDst()->begin(); i!=rule->getTDst()->end(); i++)
-            odst->add( *i );
+        {
+            FWObject *o = FWReference::getObject(*i);
+            odst->addRef(o);
+        }
 
         if ( ! rule->getTSrv()->isAny())
         {
@@ -315,7 +321,7 @@ bool NATCompiler_pf::splitSDNATRule::processNext()
                     match_service = TCPUDPService::cast(
                         compiler->dbcopy->create(tsrv->getTypeName()));
                     match_service->setName(tsrv->getName() + "_dport");
-                    compiler->dbcopy->add(match_service);
+                    compiler->persistent_objects->add(match_service);
                     match_service->setDstRangeStart(tu_tsrv->getDstRangeStart());
                     match_service->setDstRangeEnd(tu_tsrv->getDstRangeEnd());
                 }
@@ -373,15 +379,6 @@ bool NATCompiler_pf::VerifyRules::processNext()
         return true;
     }
 
-
-    if (rule->getRuleType()==NATRule::DNAT && odst->size()!=1)
-    {
-	compiler->abort(
-            rule, 
-            "There should be no more than one object in original destination");
-        return true;
-    }
-
     if (osrv->getNeg())
     {
         compiler->abort(
@@ -688,8 +685,11 @@ bool NATCompiler_pf::splitForTSrc::processNext()
 }
 
 
-bool NATCompiler_pf::assignInterfaceToNATRule(Rule *rule, Address *addr)
+bool NATCompiler_pf::assignInterfaceToNATRule(NATRule *rule, Address *addr)
 {
+    RuleElementItfOutb *itf_re = rule->getItfOutb();
+    assert(itf_re!=NULL);
+
     if (Interface::isA(addr) || IPv4::isA(addr))
     {
         FWObject *p = addr;
@@ -706,8 +706,7 @@ bool NATCompiler_pf::assignInterfaceToNATRule(Rule *rule, Address *addr)
 
         if (intf && intf->isChildOf(fw))
         {
-            rule->setInterfaceId(intf->getId());
-            rule->setInterfaceStr(intf->getName());
+            if ( ! itf_re->hasRef(intf)) itf_re->addRef(intf);
             return true;
         }
     }
@@ -716,10 +715,19 @@ bool NATCompiler_pf::assignInterfaceToNATRule(Rule *rule, Address *addr)
 
 bool NATCompiler_pf::AssignInterface::processNext()
 {
-    NATCompiler_pf *pf_comp=dynamic_cast<NATCompiler_pf*>(compiler);
-    NATRule *rule=getNext(); if (rule==NULL) return false;
+    NATCompiler_pf *pf_comp = dynamic_cast<NATCompiler_pf*>(compiler);
+    NATRule *rule = getNext(); if (rule==NULL) return false;
 
-    if (rule->getInterfaceStr() != "")
+    if (rule->getStr(".iface") == "nil")
+    {
+        tmp_queue.push_back(rule);
+        return true;
+    }
+
+    RuleElementItfOutb *itf_re = rule->getItfOutb();
+    assert(itf_re!=NULL);
+
+    if ( ! itf_re->isAny())
     {
         tmp_queue.push_back(rule);
         return true;
@@ -729,7 +737,15 @@ bool NATCompiler_pf::AssignInterface::processNext()
     {
     case NATRule::SNAT:
     {
-        if (pf_comp->assignInterfaceToNATRule(rule, compiler->getFirstTSrc(rule)))
+        RuleElementTSrc *tsrc_re = rule->getTSrc();
+        bool have_interface = false;
+        for (FWObject::iterator i1=tsrc_re->begin(); i1!=tsrc_re->end(); ++i1)
+        {
+            Address *addr = Address::cast(FWObjectReference::getObject(*i1));
+            have_interface |= pf_comp->assignInterfaceToNATRule(rule, addr);
+        }
+
+        if (have_interface)
         {
             tmp_queue.push_back(rule);
             return true;
@@ -738,13 +754,23 @@ bool NATCompiler_pf::AssignInterface::processNext()
 /* if we appear here, then TSrc is not an interface or address of
  * an interface. Generate NAT rule without "on iface" clause
  */
-        rule->setInterfaceStr("");
+//        rule->setInterfaceStr("");
+        itf_re->clearChildren();
+        itf_re->setAnyElement();
     }
     break;
 
     case NATRule::DNAT:
     {
-        if (pf_comp->assignInterfaceToNATRule(rule, compiler->getFirstODst(rule)))
+        RuleElementODst *odst_re = rule->getODst();
+        bool have_interface = false;
+        for (FWObject::iterator i1=odst_re->begin(); i1!=odst_re->end(); ++i1)
+        {
+            Address *addr = Address::cast(FWObjectReference::getObject(*i1));
+            have_interface |= pf_comp->assignInterfaceToNATRule(rule, addr);
+        }
+
+        if (have_interface)
         {
             tmp_queue.push_back(rule);
             return true;
@@ -754,7 +780,8 @@ bool NATCompiler_pf::AssignInterface::processNext()
  * interface. If this is so, just do not specify interface for rdr
  * rule.
  */
-        rule->setInterfaceStr("");
+        itf_re->clearChildren();
+        itf_re->setAnyElement();
     }
     break;
 
@@ -784,9 +811,10 @@ bool NATCompiler_pf::ReplaceFirewallObjectsODst::processNext()
  
     if (obj->getId()==compiler->getFwId() )
     {
-	list<FWObject*> l2=compiler->fw->getByType(Interface::TYPENAME);
-	for (list<FWObject*>::iterator i=l2.begin(); i!=l2.end(); ++i) {
-	    Interface *interface_=Interface::cast(*i);
+	list<FWObject*> l2 = compiler->fw->getByTypeDeep(Interface::TYPENAME);
+	for (list<FWObject*>::iterator i=l2.begin(); i!=l2.end(); ++i)
+        {
+	    Interface *interface_ = Interface::cast(*i);
 /*
  * update 03/20/03:
  *
@@ -807,10 +835,11 @@ bool NATCompiler_pf::ReplaceFirewallObjectsODst::processNext()
 	    }
 	}
 /*
- * update for ticket 1397
- * If firewall object is in ODst, do not assign the rule to any interface
+ * update for ticket 1397 If firewall object is in ODst, do not assign
+ * the rule to any interface.  I use attribute ".iface" to signal
+ * AssignInterface that it should not do anything.
  */
-        rule->setInterfaceStr("nil");
+        rule->setStr(".iface", "nil");
     }
 
     return true;
@@ -864,11 +893,12 @@ bool NATCompiler_pf::ReplaceFirewallObjectsTSrc::processNext()
  * happened if all external interfaces are unnumbered */
                 if (rel->size()==0)
                 {
-                    char errmsg[1024];
-                    sprintf(errmsg,
-"Could not find suitable interface for the NAT rule %s. Perhaps all interfaces are unnumbered?", 
-                            rule->getLabel().c_str() );
-                    compiler->abort(rule, errmsg);
+                    QString err(
+                        "Could not find suitable interface for the NAT rule %1. "
+                        "Perhaps all interfaces are unnumbered?");
+                    compiler->abort(
+                        rule,
+                        err.arg(rule->getLabel().c_str()).toStdString());
                 }
             }
 	}
@@ -915,10 +945,9 @@ bool NATCompiler_pf::ReplaceObjectsTDst::processNext()
 
             if (loopback_address==NULL)
             {
-                char errstr[1024];
-                sprintf(errstr, "Can not configure redirection NAT rule %s because loopback interface is missing." ,
-                        rule->getLabel().c_str() );
-                compiler->abort(rule, errstr);
+                compiler->abort(rule, 
+                                "Can not configure redirection for the NAT rule "
+                                "because loopback interface is missing.");
             }
 
             rel->clearChildren();
@@ -989,7 +1018,7 @@ bool NATCompiler_pf::swapAddressTableObjectsInRE::processNext()
 
                 mart->setId( mart_id );
                 compiler->dbcopy->addToIndex(mart);
-                compiler->dbcopy->add(mart);
+                compiler->persistent_objects->add(mart);
 
 // register this object as a table
                 string tblname = atbl->getName();
@@ -1117,13 +1146,13 @@ void NATCompiler_pf::checkForDynamicInterfacesOfOtherObjects::findDynamicInterfa
 
         if (ifs && ifs->isDyn() && ! ifs->isChildOf(compiler->fw))        
         {
-            char errstr[2048];
-            sprintf(errstr,
-                    "Can not build rule using dynamic interface '%s' of the object '%s' because its address is unknown.",
-                    ifs->getName().c_str(), 
-                    ifs->getParent()->getName().c_str());
-
-            compiler->abort(rule, errstr);
+            QString err(
+                "Can not build rule using dynamic interface '%1' "
+                "of the object '%2' because its address is unknown.");
+            compiler->abort(
+                rule, err
+                .arg(ifs->getName().c_str())
+                .arg(ifs->getParent()->getName().c_str()).toStdString());
         }
     }
 }
@@ -1175,116 +1204,120 @@ void NATCompiler_pf::compile()
     if (ipv6) banner += ", IPv6";
     info(banner);
 
-    try {
+    Compiler::compile();
 
-	Compiler::compile();
+    add( new Begin());
+    add( new printTotalNumberOfRules() );
 
-        add( new Begin());
-        add( new printTotalNumberOfRules() );
-
-        add( new singleRuleFilter());
+    add( new singleRuleFilter());
     
-        add( new recursiveGroupsInOSrc("check for recursive groups in OSRC") );
-        add( new recursiveGroupsInODst("check for recursive groups in ODST") );
-        add( new recursiveGroupsInOSrv("check for recursive groups in OSRV") );
+    add(new expandGroupsInItfOutb("expand groups in Interface"));
+    add(new replaceClusterInterfaceInItfOutb(
+            "replace cluster interfaces with member interfaces in "
+            "the Interface rule element"));
+    add(new ItfOutbNegation("process negation in Itf"));
 
-        add( new recursiveGroupsInTSrc("check for recursive groups in TSRC") );
-        add( new recursiveGroupsInTDst("check for recursive groups in TDST") );
-        add( new recursiveGroupsInTSrv("check for recursive groups in TSRV") );
+    add( new recursiveGroupsInOSrc("check for recursive groups in OSRC") );
+    add( new recursiveGroupsInODst("check for recursive groups in ODST") );
+    add( new recursiveGroupsInOSrv("check for recursive groups in OSRV") );
 
-        add( new emptyGroupsInOSrc(    "check for empty groups in OSRC"    ) );
-        add( new emptyGroupsInODst(    "check for empty groups in ODST"    ) );
-        add( new emptyGroupsInOSrv(    "check for empty groups in OSRV"    ) );
+    add( new recursiveGroupsInTSrc("check for recursive groups in TSRC") );
+    add( new recursiveGroupsInTDst("check for recursive groups in TDST") );
+    add( new recursiveGroupsInTSrv("check for recursive groups in TSRV") );
 
-        add( new emptyGroupsInTSrc(    "check for empty groups in TSRC"    ) );
-        add( new emptyGroupsInTDst(    "check for empty groups in TDST"    ) );
-        add( new emptyGroupsInTSrv(    "check for empty groups in TSRV"    ) );
+    add( new emptyGroupsInOSrc(    "check for empty groups in OSRC"    ) );
+    add( new emptyGroupsInODst(    "check for empty groups in ODST"    ) );
+    add( new emptyGroupsInOSrv(    "check for empty groups in OSRV"    ) );
 
-        add( new ExpandGroups( "expand groups" ) );
-        add( new eliminateDuplicatesInOSRC(  "eliminate duplicates in OSRC") );
-        add( new eliminateDuplicatesInODST(  "eliminate duplicates in ODST") );
-        add( new eliminateDuplicatesInOSRV(  "eliminate duplicates in OSRV") );
+    add( new emptyGroupsInTSrc(    "check for empty groups in TSRC"    ) );
+    add( new emptyGroupsInTDst(    "check for empty groups in TDST"    ) );
+    add( new emptyGroupsInTSrv(    "check for empty groups in TSRV"    ) );
 
-        add( new swapMultiAddressObjectsInOSrc(
-                 " swap MultiAddress -> MultiAddressRunTime in OSrc") );
-        add( new swapMultiAddressObjectsInODst(
-                 " swap MultiAddress -> MultiAddressRunTime in ODst") );
+    add( new ExpandGroups( "expand groups" ) );
+    add( new eliminateDuplicatesInOSRC(  "eliminate duplicates in OSRC") );
+    add( new eliminateDuplicatesInODST(  "eliminate duplicates in ODST") );
+    add( new eliminateDuplicatesInOSRV(  "eliminate duplicates in OSRV") );
 
-        add( new swapAddressTableObjectsInOSrc(
-                 "AddressTable -> MultiAddressRunTime in OSrc") );
-        add( new swapAddressTableObjectsInODst(
-                 "AddressTable -> MultiAddressRunTime in ODst") );
-        add( new swapAddressTableObjectsInTDst(
-                 "AddressTable -> MultiAddressRunTime in TDst") );
+    add( new swapMultiAddressObjectsInOSrc(
+             " swap MultiAddress -> MultiAddressRunTime in OSrc") );
+    add( new swapMultiAddressObjectsInODst(
+             " swap MultiAddress -> MultiAddressRunTime in ODst") );
 
-        add( new processMultiAddressObjectsInOSrc(
-                 "process MultiAddress objects in OSrc") );
-        add( new processMultiAddressObjectsInODst(
-                 "process MultiAddress objects in ODst") );
-        add( new processMultiAddressObjectsInTDst(
-                 "process MultiAddress objects in TDst") );
+    add( new swapAddressTableObjectsInOSrc(
+             "AddressTable -> MultiAddressRunTime in OSrc") );
+    add( new swapAddressTableObjectsInODst(
+             "AddressTable -> MultiAddressRunTime in ODst") );
+    add( new swapAddressTableObjectsInTDst(
+             "AddressTable -> MultiAddressRunTime in TDst") );
 
-        add( new splitOnOSrv( "split rule on original service" ) );
-        add( new fillTranslatedSrv( "fill translated service" ) );
+    add( new processMultiAddressObjectsInOSrc(
+             "process MultiAddress objects in OSrc") );
+    add( new processMultiAddressObjectsInODst(
+             "process MultiAddress objects in ODst") );
+    add( new processMultiAddressObjectsInTDst(
+             "process MultiAddress objects in TDst") );
 
-        //add( new doOSrcNegation( "process negation in OSrc" ) );
-        //add( new doODstNegation( "process negation in ODst" ) );
-        //add( new doOSrvNegation( "process negation in OSrv" ) );
+    add( new splitOnOSrv( "split rule on original service" ) );
+    add( new fillTranslatedSrv( "fill translated service" ) );
 
-        add( new NATRuleType( "determine NAT rule types" ) );
-        add( new splitSDNATRule("split SDNAT rules"      ) );
-        add( new NATRuleType( "determine NAT rule types" ) );
-        add( new VerifyRules( "verify NAT rules" ) );
-        //add( new splitODstForSNAT(
-        //       "split rule if objects in ODst belong to different subnets"));
-        add( new ReplaceFirewallObjectsODst(
-                 "replace references to the firewall in ODst" ) );
-        add( new ReplaceFirewallObjectsTSrc(
-                 "replace references to the firewall in TSrc" ) );
-        add( new ReplaceObjectsTDst( "replace objects in TDst" ) );
+    //add( new doOSrcNegation( "process negation in OSrc" ) );
+    //add( new doODstNegation( "process negation in ODst" ) );
+    //add( new doOSrvNegation( "process negation in OSrv" ) );
 
-        add( new ExpandMultipleAddresses( "expand multiple addresses" ) );
+    add( new NATRuleType( "determine NAT rule types" ) );
+    add( new splitSDNATRule("split SDNAT rules"      ) );
+    add( new NATRuleType( "determine NAT rule types" ) );
+    add( new VerifyRules( "verify NAT rules" ) );
 
-        // we might get empty RE after expanding multiple addresses,
-        // for example when unnumbered interface is used in TSRC. Note
-        // that VerifyRules should not allow this, but we may still
-        // get here in the test mode. Calling dropRuleWithEmptyRE works
-        // as a fail-safe and prevents crash.
-        add( new dropRuleWithEmptyRE("drop rules with empty rule elements"));
+    add( new ReplaceFirewallObjectsODst(
+             "replace references to the firewall in ODst" ) );
+    add( new ReplaceFirewallObjectsTSrc(
+             "replace references to the firewall in TSrc" ) );
 
-	if ( manage_virtual_addr )
-            add( new addVirtualAddress("add virtual addresses for NAT rules"));
+    add( new ReplaceObjectsTDst( "replace objects in TDst" ) );
 
-        add( new checkForUnnumbered("check for unnumbered interfaces" ) );
-        add( new checkForDynamicInterfacesOfOtherObjects(
-                 "check for dynamic interfaces of other hosts and firewalls"));
-        add( new ExpandAddressRanges( "expand address range objects" ) );
-        //add( new ConvertToAtomicForTSrc( "convert to atomic rules" ) );
-        add( new splitForTSrc(
-                 "split if addresses in TSrc belong to different networks" ));
-        add( new AssignInterface( "assign rules to interfaces" ) );
-        add( new convertInterfaceIdToStr("prepare interface assignments") );
+    add( new ExpandMultipleAddresses( "expand multiple addresses" ) );
 
-        add( new checkForObjectsWithErrors(
-                 "check if we have objects with errors in rule elements"));
+    // we might get empty RE after expanding multiple addresses,
+    // for example when unnumbered interface is used in TSRC. Note
+    // that VerifyRules should not allow this, but we may still
+    // get here in the test mode. Calling dropRuleWithEmptyRE works
+    // as a fail-safe and prevents crash.
+    add( new dropRuleWithEmptyRE("drop rules with empty rule elements"));
 
-        add( new createTables("create tables"));
+    if ( manage_virtual_addr )
+        add( new addVirtualAddress("add virtual addresses for NAT rules"));
+
+    add( new checkForUnnumbered("check for unnumbered interfaces" ) );
+    add( new checkForDynamicInterfacesOfOtherObjects(
+             "check for dynamic interfaces of other hosts and firewalls"));
+    add( new ExpandAddressRanges( "expand address range objects" ) );
+
+    add( new splitForTSrc(
+             "split if addresses in TSrc belong to different networks" ));
+
+    add( new AssignInterface( "assign rules to interfaces" ) );
+
+    add( new checkForObjectsWithErrors(
+             "check if we have objects with errors in rule elements"));
+
+    add( new createTables("create tables"));
 //        add( new PrintTables(       "print tables"     ) );
 
-        add( new PrintRule("generate pf code") );
-        add( new simplePrintProgress() );
+    add( new PrintRule("generate pf code") );
+    add( new simplePrintProgress() );
 
-        runRuleProcessors();
+    runRuleProcessors();
 
-
-    } catch (FWException &ex)
-    {
-	error(ex.toString());
-	exit(1);
-    }
 }
 
 
 void NATCompiler_pf::epilog()
 {
 }
+
+NATCompiler_pf::~NATCompiler_pf()
+{
+    //if (tables) tables->detach();
+}
+
