@@ -33,8 +33,13 @@
 
 
 #include <map>
+#include <QMap>
+#include <QSet>
+#include <QString>
 
-namespace libfwbuilder {
+
+namespace libfwbuilder
+{
     class Host;
     class IPv4;
     class IPService;
@@ -49,24 +54,29 @@ namespace libfwbuilder {
     class RuleElementTSrv;
 };
 
-namespace fwcompiler {
-
-
-    class NATCompiler_pf : public NATCompiler {
+namespace fwcompiler
+{
 
+    class NATCompiler_pf : public NATCompiler
+    {
         public:
 
-
-        struct redirectRuleInfo {
-            std::string            natrule_label;
-            libfwbuilder::FWObject *old_tdst;
-            libfwbuilder::FWObject *new_tdst;
-            libfwbuilder::Service *tsrv;
+        struct redirectRuleInfo
+        {
+            std::string natrule_label;
+            int old_tdst;
+            int new_tdst;
+            int tsrv;
             redirectRuleInfo(const std::string &rl,
                              libfwbuilder::FWObject *oa,
                              libfwbuilder::FWObject *na,
                              libfwbuilder::Service *s) 
-            {  natrule_label=rl; old_tdst=oa; new_tdst=na; tsrv=s; }
+            {
+                natrule_label = rl;
+                old_tdst = oa->getId();
+                new_tdst = na->getId();
+                tsrv = s->getId();
+            }
         };
 
 
@@ -75,7 +85,6 @@ namespace fwcompiler {
 	libfwbuilder::FWObject *loopback_address;
         TableFactory   *tables;
 
-
         virtual std::string debugPrintRule(libfwbuilder::Rule *rule);
 
         /**
@@ -86,7 +95,7 @@ namespace fwcompiler {
          * function checks if @obj is cluster interface and then
          * uses corresponding real interface instead of it.
          */
-        bool assignInterfaceToNATRule(libfwbuilder::Rule *rule,
+        bool assignInterfaceToNATRule(libfwbuilder::NATRule *rule,
                                       libfwbuilder::Address *obj);
         
 	/**
@@ -349,7 +358,6 @@ namespace fwcompiler {
         {
             void _printAnchorRule(const std::string &anchor_command,
                                   const std::string &ruleset_name,
-                                  const std::string &interface_name,
                                   libfwbuilder::NATRule *rule);
                 
     protected:
@@ -364,6 +372,7 @@ namespace fwcompiler {
             virtual void _printREAddr(libfwbuilder::RuleElement *o);
             virtual void _printAddr(libfwbuilder::FWObject *o);
             virtual void _printNATRuleOptions(libfwbuilder::Rule *rule);
+            virtual void _printInterface(libfwbuilder::NATRule *r);
 
             virtual void _printNegation(libfwbuilder::RuleElement *o);
 
@@ -383,12 +392,13 @@ namespace fwcompiler {
 		       libfwbuilder::Firewall *fw,
                        bool ipv6_policy,
 		       fwcompiler::OSConfigurator *_oscnf,
-                       TableFactory *tbf = NULL) :
-        NATCompiler(_db, fw, ipv6_policy, _oscnf)
+                       TableFactory *tbf = NULL
+        ) : NATCompiler(_db, fw, ipv6_policy, _oscnf)
         {
             tables = tbf;
         }
 
+        virtual ~NATCompiler_pf();
 
 	virtual int  prolog();
 	virtual void compile();
