@@ -41,24 +41,9 @@
 #include "fwbuilder/Firewall.h"
 #include "fwbuilder/DNSName.h"
 
-
-#include <iostream>
-#if __GNUC__ > 3 || \
-    (__GNUC__ == 3 && (__GNUC_MINOR__ > 2 || (__GNUC_MINOR__ == 2 ) ) ) || \
-    _MSC_VER
-#  include <streambuf>
-#else
-#  include <streambuf.h>
-#endif
-#include <iomanip>
-#include <fstream>
-#include <sstream>
-
 #include <assert.h>
-
 #include <QStringList>
 
-
 using namespace libfwbuilder;
 using namespace fwcompiler;
 using namespace std;
@@ -78,6 +63,30 @@ NATCompiler_pf::PrintRule::PrintRule(const std::string &name) : NATRuleProcessor
     init=true; 
 }
 
+void NATCompiler_pf::PrintRule::_printInterface(NATRule *rule)
+{
+    RuleElementItf *intf_re = rule->getItfOutb();
+    QStringList rule_interfaces;
+
+    if (!intf_re->isAny())
+    {
+        for (FWObject::iterator it=intf_re->begin(); it!=intf_re->end(); ++it)
+        {
+            FWObject *o = FWObjectReference::getObject(*it);
+            rule_interfaces << o->getName().c_str();
+        }
+        if (rule_interfaces.size() > 1)
+        {
+            rule_interfaces.push_front("{");
+            rule_interfaces.push_back("}");
+        }
+        compiler->output << "on "
+                         << rule_interfaces.join(" ").toStdString()
+                         << " ";
+    }
+}
+
+
 bool NATCompiler_pf::PrintRule::processNext()
 {
     NATRule *rule=getNext(); if (rule==NULL) return false;
@@ -88,9 +97,6 @@ bool NATCompiler_pf::PrintRule::processNext()
 
     compiler->output << compiler->printComment(rule, current_rule_label, "#");
 
-    // string err = rule->getStr(".error_msg");
-    // if (!err.empty()) compiler->output << "# " << err << endl;
-
     RuleElementOSrc *osrcrel = rule->getOSrc();
     RuleElementODst *odstrel = rule->getODst();
     RuleElementTSrc *tsrcrel = rule->getTSrc();
@@ -114,34 +120,21 @@ bool NATCompiler_pf::PrintRule::processNext()
     Address  *tdst = compiler->getFirstTDst(rule);  //assert(tdst);
     Service  *tsrv = compiler->getFirstTSrv(rule);  //assert(tsrv);
 
-    char errstr[1024];
-
     if (osrc==NULL || odst==NULL || osrv==NULL ||
         tsrc==NULL || tdst==NULL || tsrv==NULL) 
     {
-        if (osrc==NULL)
-            sprintf(errstr,"NAT rule %s: osrc==NULL", rule->getLabel().c_str());
-        if (odst==NULL)
-            sprintf(errstr,"NAT rule %s: odst==NULL", rule->getLabel().c_str());
-        if (osrv==NULL)
-            sprintf(errstr,"NAT rule %s: osrv==NULL", rule->getLabel().c_str());
-
-        if (tsrc==NULL)
-            sprintf(errstr,"NAT rule %s: tsrc==NULL", rule->getLabel().c_str());
-        if (tdst==NULL)
-            sprintf(errstr,"NAT rule %s: tdst==NULL", rule->getLabel().c_str());
-        if (tsrv==NULL)
-            sprintf(errstr,"NAT rule %s: tsrv==NULL", rule->getLabel().c_str());
-
-        compiler->abort(rule, errstr);
+        QString err;
+        if (osrc==NULL) err = QString("NAT rule %1: osrc==NULL");
+        if (odst==NULL) err = QString("NAT rule %1: odst==NULL");
+        if (osrv==NULL) err = QString("NAT rule %1: osrv==NULL");
+        if (tsrc==NULL) err = QString("NAT rule %1: tsrc==NULL");
+        if (tdst==NULL) err = QString("NAT rule %1: tdst==NULL");
+        if (tsrv==NULL) err = QString("NAT rule %1: tsrv==NULL");
+        compiler->abort(rule, err.arg(rule->getLabel().c_str()).toStdString());
     }
 
-    string  iface_name = rule->getInterfaceStr();
-//    Interface *iface = compiler->getCachedFwInterface(iface_id);
-//    string iface_name= (iface!=NULL) ? iface->getName() : "";
-    if (iface_name=="nil") iface_name="";
-
-    switch ( rule->getRuleType() ) {
+    switch ( rule->getRuleType() )
+    {
     case NATRule::Continue:
     case NATRule::NONAT:
     {
@@ -154,7 +147,9 @@ bool NATCompiler_pf::PrintRule::processNext()
              * show in examples and there is no "no" keyword anymore.
              */
             compiler->output  << "pass in quick ";
-            if (iface_name!="") compiler->output << "on " << iface_name << " ";
+
+            _printInterface(rule);
+
             _printProtocol(osrv);
             compiler->output  << "from ";
             _printREAddr(osrcrel);
@@ -166,7 +161,9 @@ bool NATCompiler_pf::PrintRule::processNext()
         } else
         {
             compiler->output  << "no nat ";
-            if (iface_name!="") compiler->output << "on " << iface_name << " ";
+
+            _printInterface(rule);
+
             _printProtocol(osrv);
             compiler->output  << "from ";
             _printREAddr(osrcrel);
@@ -175,7 +172,9 @@ bool NATCompiler_pf::PrintRule::processNext()
             compiler->output  << endl;
 
             compiler->output  << "no rdr ";
-            if (iface_name!="") compiler->output << "on " << iface_name << " ";
+
+            _printInterface(rule);
+
             _printProtocol(osrv);
             compiler->output  << "from ";
             _printREAddr( osrcrel );
@@ -190,7 +189,9 @@ bool NATCompiler_pf::PrintRule::processNext()
         if (XMLTools::version_compare(version, "4.7")>=0)
         {
             compiler->output  << "match out ";
-            if (iface_name!="") compiler->output << "on " << iface_name << " ";
+
+            _printInterface(rule);
+
             _printProtocol(osrv);
             compiler->output  << "from ";
             _printREAddr( osrcrel );
@@ -209,7 +210,9 @@ bool NATCompiler_pf::PrintRule::processNext()
         } else
         {
             compiler->output  << "nat ";
-            if (iface_name!="") compiler->output << "on " << iface_name << " ";
+
+            _printInterface(rule);
+
             _printProtocol(osrv);
             compiler->output  << "from ";
             _printREAddr( osrcrel );
@@ -234,7 +237,9 @@ bool NATCompiler_pf::PrintRule::processNext()
         if (XMLTools::version_compare(version, "4.7")>=0)
         {
             compiler->output  << "match in ";
-            if (iface_name!="") compiler->output << "on " << iface_name << " ";
+
+            _printInterface(rule);
+
             _printProtocol(osrv);
             compiler->output  << "from ";
             _printREAddr( osrcrel );
@@ -250,7 +255,9 @@ bool NATCompiler_pf::PrintRule::processNext()
         } else
         {
             compiler->output  << "rdr ";
-            if (iface_name!="") compiler->output << "on " << iface_name << " ";
+
+            _printInterface(rule);
+
             _printProtocol(osrv);
             compiler->output  << "from ";
             _printREAddr( osrcrel );
@@ -272,7 +279,9 @@ bool NATCompiler_pf::PrintRule::processNext()
         if (XMLTools::version_compare(version, "4.7")>=0)
         {
             compiler->output  << "match in ";
-            if (iface_name!="") compiler->output << "on " << iface_name << " ";
+
+            _printInterface(rule);
+
             _printProtocol(osrv);
             compiler->output  << "from ";
             _printREAddr( osrcrel );
@@ -287,7 +296,9 @@ bool NATCompiler_pf::PrintRule::processNext()
         } else
         {
             compiler->output  << "rdr ";
-            if (iface_name!="") compiler->output << "on " << iface_name << " ";
+
+            _printInterface(rule);
+
             _printProtocol(osrv);
             compiler->output  << "from ";
             _printREAddr( osrcrel );
@@ -321,11 +332,11 @@ bool NATCompiler_pf::PrintRule::processNext()
 
         if (XMLTools::version_compare(version, "4.6")>=0)
         {
-            _printAnchorRule("anchor", ruleset_name, iface_name, rule);
+            _printAnchorRule("anchor", ruleset_name, rule);
         } else
         {
-            _printAnchorRule("nat-anchor", ruleset_name, iface_name, rule);
-            _printAnchorRule("rdr-anchor", ruleset_name, iface_name, rule);
+            _printAnchorRule("nat-anchor", ruleset_name, rule);
+            _printAnchorRule("rdr-anchor", ruleset_name, rule);
         }
 
     }
@@ -339,7 +350,6 @@ bool NATCompiler_pf::PrintRule::processNext()
 
 void NATCompiler_pf::PrintRule::_printAnchorRule(const string &anchor_command,
                                                  const std::string &ruleset_name,
-                                                 const std::string &interface_name,
                                                  NATRule *rule)
 {
     RuleElementOSrc *osrcrel = rule->getOSrc();
@@ -348,7 +358,9 @@ void NATCompiler_pf::PrintRule::_printAnchorRule(const string &anchor_command,
     Service *osrv = compiler->getFirstOSrv(rule);
 
     compiler->output << anchor_command << " \"" << ruleset_name << "\" ";
-    if (interface_name!="") compiler->output << "on " << interface_name << " ";
+
+    _printInterface(rule);
+
     if (!osrvrel->isAny() || !osrcrel->isAny() || !odstrel->isAny())
     {
         _printProtocol(osrv);
