@@ -44,32 +44,14 @@
 #include "fwbuilder/DNSName.h"
 #include "fwbuilder/FWObjectDatabase.h"
 
-#include <iostream>
-
-#if __GNUC__ > 3 || \
-    (__GNUC__ == 3 && (__GNUC_MINOR__ > 2 || (__GNUC_MINOR__ == 2 ) ) ) || \
-    _MSC_VER
-#  include <streambuf>
-#else
-#  include <streambuf.h>
-#endif
-#include <iomanip>
-#include <fstream>
-#include <sstream>
-
 #include <assert.h>
-
 #include <QStringList>
 
-
 using namespace libfwbuilder;
 using namespace fwcompiler;
 using namespace std;
 
 
-
-
-
 /**
  *-----------------------------------------------------------------------
  *                    Methods for printing
@@ -356,21 +338,6 @@ void PolicyCompiler_ipfw::PrintRule::_printInterface(PolicyRule *r)
         }
 	compiler->output << intf->getName() << " ";
     }
-
-    // int iface_id = r->getInterfaceId();
-    // if (iface_id > -1) 
-    // {
-    //     switch (r->getDirection())
-    //     {
-    //     case PolicyRule::Outbound: compiler->output << "xmit "; break;
-    //     case PolicyRule::Inbound:  compiler->output << "recv "; break;
-    //     case PolicyRule::Both:     compiler->output << "via  "; break;
-    //     default: break;
-    //     }
-
-    //     FWObject *rule_iface = compiler->dbcopy->findInIndex( iface_id );
-    //     compiler->output << rule_iface->getName() << " ";
-    // }
 }
 
 void PolicyCompiler_ipfw::PrintRule::_printSrcService(RuleElement  *rel)
@@ -541,9 +508,6 @@ bool PolicyCompiler_ipfw::PrintRule::processNext()
 
     compiler->output << compiler->printComment(rule, current_rule_label, "#");
 
-    // string err = rule->getStr(".error_msg");
-    // if (!err.empty()) compiler->output << "# " << err << endl;
-
     RuleElementSrc *srcrel=rule->getSrc();
     Address        *src   =compiler->getFirstSrc(rule);  assert(src);
     RuleElementDst *dstrel=rule->getDst();
