@@ -28,20 +28,21 @@
 #include "PolicyCompiler_pf.h"
 #include "NATCompiler_pf.h"
 
-#include "fwbuilder/FWObjectDatabase.h"
-#include "fwbuilder/RuleElement.h"
-#include "fwbuilder/IPService.h"
-#include "fwbuilder/ICMPService.h"
-#include "fwbuilder/TCPService.h"
-#include "fwbuilder/UDPService.h"
-#include "fwbuilder/TagService.h"
-#include "fwbuilder/Policy.h"
-#include "fwbuilder/Interface.h"
-#include "fwbuilder/Firewall.h"
-#include "fwbuilder/Network.h"
 #include "fwbuilder/AddressTable.h"
+#include "fwbuilder/FWObjectDatabase.h"
 #include "fwbuilder/FailoverClusterGroup.h"
+#include "fwbuilder/Firewall.h"
+#include "fwbuilder/ICMPService.h"
+#include "fwbuilder/IPService.h"
+#include "fwbuilder/Interface.h"
+#include "fwbuilder/Library.h"
+#include "fwbuilder/Network.h"
+#include "fwbuilder/Policy.h"
+#include "fwbuilder/RuleElement.h"
 #include "fwbuilder/StateSyncClusterGroup.h"
+#include "fwbuilder/TCPService.h"
+#include "fwbuilder/TagService.h"
+#include "fwbuilder/UDPService.h"
 
 #include <algorithm>
 #include <functional>
@@ -49,6 +50,8 @@
 #include <iomanip>
 
 #include <assert.h>
+#include <QtDebug>
+
 
 using namespace libfwbuilder;
 using namespace fwcompiler;
@@ -72,11 +75,11 @@ int PolicyCompiler_pf::prolog()
             list<FWObject*> l3=iface->getByType(IPv4::TYPENAME);
             if (l3.size()>0)
             {
-                char errstr[256];
-                sprintf(errstr,
-                        "Dynamic interface %s should not have an IP address object attached to it. This IP address object will be ignored.",
-                        iface->getName().c_str() );
-                warning(errstr );
+                QString err(
+                    "Dynamic interface %1 should not have an IP "
+                    "address object attached to it. This IP address "
+                    "object will be ignored.");
+                warning(err.arg(iface->getName().c_str()).toStdString());
                 for (list<FWObject*>::iterator j=l3.begin(); j!=l3.end(); ++j) 
                     iface->remove(*j);
             }
@@ -154,7 +157,7 @@ bool PolicyCompiler_pf::swapAddressTableObjectsInRE::processNext()
 
                 mart->setId( mart_id );
                 compiler->dbcopy->addToIndex(mart);
-                compiler->dbcopy->add(mart);
+                compiler->persistent_objects->add(mart);
 
 // register this object as a table
                 string tblname = atbl->getName();
@@ -249,41 +252,41 @@ bool PolicyCompiler_pf::splitIfFirewallInSrc::processNext()
 {
     PolicyRule *rule=getNext(); if (rule==NULL) return false;
 
-    PolicyRule     *r;
-    RuleElementSrc *src=rule->getSrc();    assert(src);
+    PolicyRule *r;
+    RuleElementSrc *src = rule->getSrc();
+    assert(src);
 
     if (src->size()==1 || src->getNeg())
     {
 	tmp_queue.push_back(rule);
 	return true;
     }
-    FWObject       *fw_in_src=NULL;
-    vector<FWObject*> cl;
-    for (FWObject::iterator i1=src->begin(); i1!=src->end(); ++i1) {
 
-	FWObject *o   = *i1;
-	FWObject *obj = NULL;
-//	if (FWReference::cast(o)!=NULL) obj=FWReference::cast(o)->getPointer();
-	if (FWReference::cast(o)!=NULL) obj=FWReference::cast(o)->getPointer();
+    FWObject *fw_in_src = NULL;
+    vector<FWObject*> cl;
+    for (FWObject::iterator i1=src->begin(); i1!=src->end(); ++i1)
+    {
+	FWObject *obj = FWReference::getObject(*i1);
 	if (obj==NULL)
             compiler->abort(rule, "Broken Src object");
 
-	if (obj->getId()==compiler->getFwId()) {
-	    fw_in_src=o;   // can not remove right now because remove invalidates iterator
+	if (obj->getId()==compiler->getFwId())
+        {
+	    fw_in_src = obj;
 
 	    RuleElementSrc *nsrc;
 
-	    r= compiler->dbcopy->createPolicyRule();
+	    r = compiler->dbcopy->createPolicyRule();
 	    compiler->temp_ruleset->add(r);
 	    r->duplicate(rule);
-	    nsrc=r->getSrc();
+	    nsrc = r->getSrc();
 	    nsrc->clearChildren();
 	    nsrc->setAnyElement();
 	    nsrc->addRef( compiler->fw );
 	    tmp_queue.push_back(r);
 	}
     }
-    if (fw_in_src!=NULL) src->remove( fw_in_src );
+    if (fw_in_src!=NULL) src->removeRef( fw_in_src );
 
     tmp_queue.push_back(rule);
     return true;
@@ -292,10 +295,10 @@ bool PolicyCompiler_pf::splitIfFirewallInSrc::processNext()
 
 bool PolicyCompiler_pf::splitIfFirewallInDst::processNext()
 {
-    PolicyRule *rule=getNext(); if (rule==NULL) return false;
+    PolicyRule *rule = getNext(); if (rule==NULL) return false;
 
-    PolicyRule     *r;
-    RuleElementDst *dst=rule->getDst();    assert(dst);
+    PolicyRule *r;
+    RuleElementDst *dst = rule->getDst();    assert(dst);
 
     if (dst->size()==1 || dst->getNeg())
     {
@@ -303,33 +306,31 @@ bool PolicyCompiler_pf::splitIfFirewallInDst::processNext()
 	return true;
     }
 
-    FWObject       *fw_in_dst=NULL;
+    FWObject *fw_in_dst = NULL;
     vector<FWObject*> cl;
     for (FWObject::iterator i1=dst->begin(); i1!=dst->end(); ++i1)
     {
-	FWObject *o   = *i1;
-	FWObject *obj = NULL;
-//	if (FWReference::cast(o)!=NULL) obj=FWReference::cast(o)->getPointer();
-	if (FWReference::cast(o)!=NULL) obj=FWReference::cast(o)->getPointer();
+	FWObject *obj = FWReference::getObject(*i1);
 	if (obj==NULL)
             compiler->abort(rule, "Broken Dst");
 
-	if (obj->getId()==compiler->getFwId()) {
-	    fw_in_dst=o;   // can not remove right now because remove invalidates iterator
+	if (obj->getId()==compiler->getFwId())
+        {
+	    fw_in_dst = obj;
 
 	    RuleElementDst *ndst;
 
-	    r= compiler->dbcopy->createPolicyRule();
+	    r = compiler->dbcopy->createPolicyRule();
 	    compiler->temp_ruleset->add(r);
 	    r->duplicate(rule);
-	    ndst=r->getDst();
+	    ndst = r->getDst();
 	    ndst->clearChildren();
 	    ndst->setAnyElement();
 	    ndst->addRef( compiler->fw );
 	    tmp_queue.push_back(r);
 	}
     }
-    if (fw_in_dst!=NULL) dst->remove( fw_in_dst );
+    if (fw_in_dst!=NULL) dst->removeRef( fw_in_dst );
 
     tmp_queue.push_back(rule);
     return true;
@@ -387,124 +388,18 @@ bool PolicyCompiler_pf::fillDirection::processNext()
         {
             rule->setDirection( PolicyRule::Outbound );
             compiler->warning(
-                                  rule, "Changing rule direction due to self reference");
+                rule, "Changing rule direction due to self reference");
         }
         if (!dst->isAny() && compiler->complexMatch(compiler->fw, dst))
         {
             rule->setDirection( PolicyRule::Inbound );
             compiler->warning(
-                                  rule, "Changing rule direction due to self reference");
+                rule, "Changing rule direction due to self reference");
         }
     }
     return true;
 }
 
-/*
- * add default and auto-generated rules, except when we process rules
- * in a branch (i.e. anchor, since this is PF)
- */
-void PolicyCompiler_pf::addDefaultPolicyRule()
-{
-    if (getSourceRuleSet()->isTop() && !inSingleRuleCompileMode())
-    {
-        if (getCachedFwOpt()->getBool("mgmt_ssh") &&
-             !getCachedFwOpt()->getStr("mgmt_addr").empty())
-        {
-            PolicyRule *r;
-            TCPService *ssh = dbcopy->createTCPService();
-            ssh->setDstRangeStart(22);
-            ssh->setDstRangeEnd(22);
-
-            ssh->setName("mgmt_ssh");
-            dbcopy->add(ssh,false);
-
-            string mgmt_addr = getCachedFwOpt()->getStr("mgmt_addr");
-            InetAddr  addr;
-            InetAddr netmask(InetAddr::getAllOnes());
-            try
-            {
-                addr = InetAddr(mgmt_addr);
-                string::size_type sep = mgmt_addr.find("/");
-                if (sep != string::npos)
-                {
-                    addr = InetAddr(mgmt_addr.substr(0,sep));
-                    string nm = mgmt_addr.substr(sep+1);
-                    int o1,o2,o3,o4;
-                    if (sscanf(nm.c_str(),
-                               "%3u.%3u.%3u.%3u", &o1, &o2, &o3, &o4)==4)
-                    {
-                        netmask = InetAddr(nm);
-                    } else
-                    {
-                        sscanf(nm.c_str(),"%u",&o1);
-                        netmask = InetAddr(o1);
-                    }
-                }
-            } catch(FWException &ex)
-            {
-                char errstr[256];
-                sprintf(errstr,
-                        "Invalid address for the backup ssh access: '%s'",
-                        mgmt_addr.c_str());
-                abort(errstr);
-            }
-
-            Network *mgmt_workstation = dbcopy->createNetwork();
-            mgmt_workstation->setName("mgmt_addr");
-            mgmt_workstation->setAddress(addr);
-            mgmt_workstation->setNetmask(netmask);
-//        IPv4 *mgmt_workstation = IPv4::cast(dbcopy->create(IPv4::TYPENAME));
-//        mgmt_workstation->setAddress(getCachedFwOpt()->getStr("mgmt_addr"));
-            dbcopy->add(mgmt_workstation,false);
-
-            r = dbcopy->createPolicyRule();
-            temp_ruleset->add(r);
-            r->setAction(PolicyRule::Accept);
-            r->setLogging(false);
-            r->setDirection(PolicyRule::Inbound);
-            r->setPosition(9998);
-            r->setComment("   backup ssh access rule ");
-            r->setHidden(true);
-            r->setFallback(false);
-            r->setLabel("backup ssh access rule");
-            r->setBool("needs_established",true);  // supported in ipfw
-
-            RuleElement *src = r->getSrc();
-            assert(src!=NULL);
-            src->addRef(mgmt_workstation);
-
-            RuleElement *dst = r->getDst();
-            assert(dst!=NULL);
-            dst->addRef(fw);
-
-            RuleElement *srv = r->getSrv();
-            assert(srv!=NULL);
-            srv->addRef(ssh);
-
-            combined_ruleset->push_front(r);
-
-        }
-
-        insertCarpRule();
-        insertPfsyncRule();
-
-        PolicyRule *r = dbcopy->createPolicyRule();
-        FWOptions *ruleopt;
-        temp_ruleset->add(r);
-        r->setAction(PolicyRule::Deny);
-        r->setLogging(getCachedFwOpt()->getBool("fallback_log"));
-        r->setDirection(PolicyRule::Both);
-        r->setPosition(10000);
-        r->setComment("   fallback rule ");
-        r->setHidden(true);
-        r->setFallback(true);
-        r->setLabel("fallback rule");
-        ruleopt = r->getOptionsObject();
-        ruleopt->setBool("stateless", true);
-        combined_ruleset->push_back(r);
-    }
-}
-
 bool PolicyCompiler_pf::SpecialServices::processNext()
 {
     PolicyRule *rule=getNext(); if (rule==NULL) return false;
@@ -612,11 +507,43 @@ bool PolicyCompiler_pf::setQuickFlag::processNext()
     PolicyRule *rule=getNext(); if (rule==NULL) return false;
     tmp_queue.push_back(rule);
 
-    if ( rule->getAction()!=PolicyRule::Scrub &&
-         rule->getAction()!=PolicyRule::Accounting &&
-         rule->getAction()!=PolicyRule::Tag &&
-         rule->getAction()!=PolicyRule::Branch
-    ) rule->setBool("quick",true);
+    FWOptions *ropt = rule->getOptionsObject();
+
+    // as of 4.2.0 build 3477 we provide checkboxes to make Tag and
+    // Classify actions (PF) terminating or non-terminating on
+    // per-rule basis. Old behavior: Tag was non-terminating and
+    // Classify was terminating. Set options accordingly if they are
+    // not set.
+
+    switch (rule->getAction())
+    {
+    case PolicyRule::Scrub:
+    case PolicyRule::Accounting:
+    case PolicyRule::Branch:
+        break;
+
+    case PolicyRule::Tag:
+    {
+        string pf_tag_terminating = ropt->getStr("pf_tag_terminating");
+        if (pf_tag_terminating.empty())
+            ropt->setBool("pf_tag_terminating", false);
+        if (ropt->getBool("pf_tag_terminating")) rule->setBool("quick", true);
+        break;
+    }
+
+    case PolicyRule::Classify:
+    {
+        string pf_classify_terminating = ropt->getStr("pf_classify_terminating");
+        if (pf_classify_terminating.empty())
+            ropt->setBool("pf_classify_terminating", true);
+        if (ropt->getBool("pf_classify_terminating")) rule->setBool("quick", true);
+        break;
+    }
+
+    default:
+        rule->setBool("quick", true);
+        break;
+    }
 
     return true;
 }
@@ -718,12 +645,11 @@ bool PolicyCompiler_pf::doSrvNegation::processNext()
 
 bool PolicyCompiler_pf::addLoopbackForRedirect::processNext()
 {
-    PolicyRule *rule=getNext(); if (rule==NULL) return false;
-    PolicyCompiler_pf *pf_comp=dynamic_cast<PolicyCompiler_pf*>(compiler);
+    PolicyRule *rule = getNext(); if (rule==NULL) return false;
+    PolicyCompiler_pf *pf_comp = dynamic_cast<PolicyCompiler_pf*>(compiler);
 
-//    RuleElementSrc *src=rule->getSrc();
-    RuleElementDst *dst=rule->getDst();
-    RuleElementSrv *srv=rule->getSrv();
+    RuleElementDst *dst = rule->getDst();
+    RuleElementSrv *srv = rule->getSrv();
 
     if (pf_comp->redirect_rules_info==NULL)
         compiler->abort(
@@ -733,41 +659,34 @@ bool PolicyCompiler_pf::addLoopbackForRedirect::processNext()
 
     tmp_queue.push_back(rule);
 
-    //const list<NATCompiler_pf::redirectRuleInfo> lst = 
-    //  pf_comp->natcmp->getRedirRulesInfo();
-
     if (pf_comp->redirect_rules_info->empty()) return true;
 
-/*
- *  struct redirectRuleInfo {
- *    string   natrule_label;
- *    Address *tdst;
- *    Service *tsrv;
- *  };
- */
-
     for (FWObject::iterator i=srv->begin(); i!=srv->end(); i++) 
     {
-	FWObject *o1= *i;
-	if (FWReference::cast(o1)!=NULL) o1=FWReference::cast(o1)->getPointer();
-	Service *s=Service::cast( o1 );
+	FWObject *o1 = FWReference::getObject(*i);
+	Service *s = Service::cast( o1 );
 	assert(s);
 
         for (FWObject::iterator j=dst->begin(); j!=dst->end(); j++) 
         {
-            FWObject *o2= *j;
-            if (FWReference::cast(o2)!=NULL) o2=FWReference::cast(o2)->getPointer();
-            Address *a=Address::cast( o2 );
+            FWObject *o2 = FWReference::getObject(*j);
+            Address *a = Address::cast( o2 );
             assert(a);
 
             list<NATCompiler_pf::redirectRuleInfo>::const_iterator k;
             for (k=pf_comp->redirect_rules_info->begin();
                  k!=pf_comp->redirect_rules_info->end(); ++k)
             {
-                if ( *a == *(k->old_tdst) &&  *s == *(k->tsrv) )
+                Address *old_tdst_obj = Address::cast(
+                    compiler->dbcopy->findInIndex(k->old_tdst));
+                Service *tsrv_obj = Service::cast(
+                    compiler->dbcopy->findInIndex(k->tsrv));
+
+                if ( *a == *(old_tdst_obj) &&  *s == *(tsrv_obj) )
                 {
 // insert address used for redirection in the NAT rule.
-                    dst->addRef( k->new_tdst );
+                    FWObject *new_tdst_obj = compiler->dbcopy->findInIndex(k->new_tdst);
+                    dst->addRef(new_tdst_obj);
                     return true;
                 }
             }
@@ -795,13 +714,14 @@ void PolicyCompiler_pf::checkForDynamicInterfacesOfOtherObjects::findDynamicInte
             ifs->getParent()->getId()!=compiler->fw->getId() &&
             ! ifs->getParent()->getBool("pf_table") )
         {
-            char errstr[2048];
-            sprintf(errstr,"Can not build rule using dynamic interface '%s' of the object '%s' because its address in unknown. Rule %s",
-                    ifs->getName().c_str(), 
-                    ifs->getParent()->getName().c_str(),
-                    rule->getLabel().c_str() );
-
-            compiler->abort(rule, errstr);
+            QString err(
+                "Can not build rule using dynamic interface '%1' "
+                "of the object '%2' because its address in unknown.");
+            compiler->abort(
+                rule, 
+                err
+                .arg(ifs->getName().c_str())
+                .arg(ifs->getParent()->getName().c_str()).toStdString());
         }
     }
 }
@@ -910,78 +830,6 @@ bool PolicyCompiler_pf::printScrubRule::processNext()
 
 
 
-
-PolicyCompiler_pf::separateServiceObject::separateServiceObject(
-    const string &name) : PolicyRuleProcessor(name)
-{
-}
-
-bool PolicyCompiler_pf::separateServiceObject::processNext()
-{
-    PolicyRule *rule=getNext(); if (rule==NULL) return false;
-
-    RuleElementSrv *rel= rule->getSrv();
-
-    if (rel->size()==1) {
-	tmp_queue.push_back(rule);
-	return true;
-    }
-
-    list<Service*> services;
-    for (FWObject::iterator i=rel->begin(); i!=rel->end(); i++)
-    {
-	FWObject *o= *i;
-	if (FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
-	Service *s=Service::cast(o);
-	assert(s!=NULL);
-
-	if (condition(s))
-        {
-            PolicyRule *r = compiler->dbcopy->createPolicyRule();
-            compiler->temp_ruleset->add(r);
-            r->duplicate(rule);
-            RuleElementSrv *nsrv=r->getSrv();
-            nsrv->clearChildren();
-            nsrv->addRef( s );
-            tmp_queue.push_back(r);
-            services.push_back(s);
-        }
-    }
-    for (list<Service*>::iterator i=services.begin(); i!=services.end(); i++) 
-	rel->removeRef( (*i) );
-
-    if (!rel->isAny())
-	tmp_queue.push_back(rule);
-
-    return true;
-}
-
-
-bool PolicyCompiler_pf::separateSrcPort::condition(const Service *srv)
-{
-    if ( TCPService::isA(srv) || UDPService::isA(srv))
-    {
-        int srs = TCPUDPService::constcast(srv)->getSrcRangeStart();
-        int sre = TCPUDPService::constcast(srv)->getSrcRangeEnd();
-
-        compiler->normalizePortRange(srs,sre);
-
-        return (srs!=0 || sre!=0);
-    }
-    return false;
-}
-
-bool PolicyCompiler_pf::separateTagged::condition(const Service *srv)
-{
-    return ( TagService::isA(srv));
-}
-
-bool PolicyCompiler_pf::separateTOS::condition(const Service *srv)
-{
-    const IPService *ip = IPService::constcast(srv);
-    return (ip && !ip->getTOSCode().empty());
-}
-
 void PolicyCompiler_pf::compile()
 {
     string banner = " Compiling " + fw->getName();
@@ -989,173 +837,167 @@ void PolicyCompiler_pf::compile()
     if (ipv6) banner += ", IPv6";
     info(banner);
 
-    try 
+    Compiler::compile();
+
+    bool check_for_recursive_groups=true;
+
+    if (fw->getOptionsObject()->getBool("check_shading") &&
+        ! inSingleRuleCompileMode())
     {
-	Compiler::compile();
-
-	addDefaultPolicyRule();
-        bool check_for_recursive_groups=true;
-
-        if (fw->getOptionsObject()->getBool("check_shading") &&
-            ! inSingleRuleCompileMode())
-        {
-            add(new Begin("Detecting rule shadowing"));
-            add(new printTotalNumberOfRules());
-
-            add(new ItfNegation("process negation in Itf"));
-            add(new InterfacePolicyRules(
-                     "process interface policy rules and store interface ids"));
-
-            add(new recursiveGroupsInSrc("check for recursive groups in SRC"));
-            add(new recursiveGroupsInDst("check for recursive groups in DST"));
-            add(new recursiveGroupsInSrv("check for recursive groups in SRV"));
-            check_for_recursive_groups=false;
-
-            add(new ExpandGroups("expand groups"));
-            add(new dropRuleWithEmptyRE("drop rules with empty rule elements"));
-            add(new eliminateDuplicatesInSRC("eliminate duplicates in SRC"));
-            add(new eliminateDuplicatesInDST("eliminate duplicates in DST"));
-            add(new eliminateDuplicatesInSRV("eliminate duplicates in SRV"));
-
-            add(new swapAddressTableObjectsInSrc(
-                     "AddressTable -> MultiAddressRunTime in Src"));
-            add(new swapAddressTableObjectsInDst(
-                     "AddressTable -> MultiAddressRunTime in Dst"));
-
-            add(new swapMultiAddressObjectsInSrc(
-                     "MultiAddress -> MultiAddressRunTime in Src"));
-            add(new swapMultiAddressObjectsInDst(
-                     "MultiAddress -> MultiAddressRunTime in Dst"));
-
-            add(new ExpandMultipleAddressesInSrc(
-                     "expand objects with multiple addresses in SRC"));
-            add(new ExpandMultipleAddressesInDst(
-                     "expand objects with multiple addresses in DST"));
-            add(new dropRuleWithEmptyRE(
-                     "drop rules with empty rule elements"));
-            add(new ConvertToAtomic("convert to atomic rules"));
-
-            add( new checkForObjectsWithErrors(
-                     "check if we have objects with errors in rule elements"));
-
-            add(new DetectShadowing("Detect shadowing"));
-            add(new simplePrintProgress());
-
-            runRuleProcessors();
-            deleteRuleProcessors();
-        }
-
-        add(new Begin());
+        add(new Begin("Detecting rule shadowing"));
         add(new printTotalNumberOfRules());
 
-        add( new singleRuleFilter());
+        add(new ItfNegation("process negation in Itf"));
+        add(new InterfacePolicyRules(
+                "process interface policy rules and store interface ids"));
 
-//        add(new printScrubRule(" Defragmentation"));
-        if (check_for_recursive_groups)
-        {
-            add(new recursiveGroupsInSrc("check for recursive groups in SRC"));
-            add(new recursiveGroupsInDst("check for recursive groups in DST"));
-            add(new recursiveGroupsInSrv("check for recursive groups in SRV"));
-        }
+        add(new recursiveGroupsInSrc("check for recursive groups in SRC"));
+        add(new recursiveGroupsInDst("check for recursive groups in DST"));
+        add(new recursiveGroupsInSrv("check for recursive groups in SRV"));
+        check_for_recursive_groups=false;
 
-        add(new emptyGroupsInSrc("check for empty groups in SRC"));
-        add(new emptyGroupsInDst("check for empty groups in DST"));
-        add(new emptyGroupsInSrv("check for empty groups in SRV"));
-
-//        add(new doSrcNegation("process negation in Src"));
-//        add(new doDstNegation("process negation in Dst"));
-	add(new doSrvNegation("process negation in Srv"));
-
-// ExpandGroups opens groups, as well as groups in groups etc.
         add(new ExpandGroups("expand groups"));
         add(new dropRuleWithEmptyRE("drop rules with empty rule elements"));
-
-        add(new CheckForTCPEstablished(
-                 "check for TCPService objects with flag \"established\""));
-            
         add(new eliminateDuplicatesInSRC("eliminate duplicates in SRC"));
         add(new eliminateDuplicatesInDST("eliminate duplicates in DST"));
         add(new eliminateDuplicatesInSRV("eliminate duplicates in SRV"));
 
         add(new swapAddressTableObjectsInSrc(
-                 "AddressTable -> MultiAddressRunTime in Src"));
+                "AddressTable -> MultiAddressRunTime in Src"));
         add(new swapAddressTableObjectsInDst(
-                 "AddressTable -> MultiAddressRunTime in Dst"));
+                "AddressTable -> MultiAddressRunTime in Dst"));
 
         add(new swapMultiAddressObjectsInSrc(
-                 "MultiAddress -> MultiAddressRunTime in Src"));
+                "MultiAddress -> MultiAddressRunTime in Src"));
         add(new swapMultiAddressObjectsInDst(
-                 "MultiAddress -> MultiAddressRunTime in Dst"));
+                "MultiAddress -> MultiAddressRunTime in Dst"));
 
-        add(new processMultiAddressObjectsInSrc(
-                 "process MultiAddress objects in Src"));
-        add(new processMultiAddressObjectsInDst(
-                 "process MultiAddress objects in Dst"));
-
-        add(new replaceFailoverInterfaceInItf("replace carp interfaces"));
-
-
-        add(new expandGroupsInItf("expand groups in Interface"));
-        add(new replaceClusterInterfaceInItf(
-                "replace cluster interfaces with member interfaces in the Interface rule element"));
-        add(new ItfNegation("process negation in Itf"));
-
-        //add(new InterfacePolicyRules(
-        //    "process interface policy rules and store interface ids"));
-
-	add(new splitIfFirewallInSrc("split rule if firewall is in Src"));
-	add(new splitIfFirewallInDst("split rule if firewall is in Dst"));
-	add(new fillDirection("determine directions"));
-
-// commented out for bug #2828602
-// ... and put back per #2844561
-// both bug reports/patches are by Tom Judge (tomjudge on sourceforge)
-	add( new SplitDirection("split rules with direction 'both'"  ));
-
-        add(new addLoopbackForRedirect(
-                 "add loopback to rules that permit redirected services"));
-	add(new ExpandMultipleAddresses(
-                 "expand objects with multiple addresses"));
-        add(new dropRuleWithEmptyRE("drop rules with empty rule elements"));
-        add(new checkForDynamicInterfacesOfOtherObjects(
-                 "check for dynamic interfaces of other hosts and firewalls"));
-        add(new MACFiltering("verify for MAC address filtering"));
-        add(new checkForUnnumbered("check for unnumbered interfaces"));
-	add(new addressRanges("expand address range objects"));
-	add(new splitServices("split rules with different protocols"));
-	add(new separateTCPWithFlags("separate TCP services with flags"));
-        add(new separateSrcPort("split on TCP and UDP with source ports"));
-        add(new separateTagged("split on TagService"));
-        add(new separateTOS("split on IPService with TOS"));
-
-        if (ipv6)
-            add( new DropIPv4Rules("drop ipv4 rules"));
-        else
-            add( new DropIPv6Rules("drop ipv6 rules"));
-
-	add(new verifyCustomServices("verify custom services for this platform"));
-//	add(new ProcessScrubOption("process 'scrub' option"));
-	add(new SpecialServices("check for special services"));
-	add(new setQuickFlag("set 'quick' flag"));
-        add(new checkForZeroAddr("check for zero addresses"));
-        add(new convertInterfaceIdToStr("prepare interface assignments"));
+        add(new ExpandMultipleAddressesInSrc(
+                "expand objects with multiple addresses in SRC"));
+        add(new ExpandMultipleAddressesInDst(
+                "expand objects with multiple addresses in DST"));
+        add(new dropRuleWithEmptyRE(
+                "drop rules with empty rule elements"));
+        add(new ConvertToAtomic("convert to atomic rules"));
 
         add( new checkForObjectsWithErrors(
                  "check if we have objects with errors in rule elements"));
 
-        add(new createTables("create tables"));
-//        add(new PrintTables("print tables"));
+        add(new setQuickFlag("set 'quick' flag"));
 
-        add(new PrintRule("generate pf code"));
+        add(new DetectShadowing("Detect shadowing"));
         add(new simplePrintProgress());
 
         runRuleProcessors();
-
-    } catch (FWException &ex)
-    {
-	error(ex.toString());
-	exit(1);
+        deleteRuleProcessors();
     }
+
+    add(new Begin());
+    add(new printTotalNumberOfRules());
+
+    add( new singleRuleFilter());
+
+//        add(new printScrubRule(" Defragmentation"));
+    if (check_for_recursive_groups)
+    {
+        add(new recursiveGroupsInSrc("check for recursive groups in SRC"));
+        add(new recursiveGroupsInDst("check for recursive groups in DST"));
+        add(new recursiveGroupsInSrv("check for recursive groups in SRV"));
+    }
+
+    add(new emptyGroupsInSrc("check for empty groups in SRC"));
+    add(new emptyGroupsInDst("check for empty groups in DST"));
+    add(new emptyGroupsInSrv("check for empty groups in SRV"));
+
+//        add(new doSrcNegation("process negation in Src"));
+//        add(new doDstNegation("process negation in Dst"));
+    add(new doSrvNegation("process negation in Srv"));
+
+// ExpandGroups opens groups, as well as groups in groups etc.
+    add(new ExpandGroups("expand groups"));
+    add(new dropRuleWithEmptyRE("drop rules with empty rule elements"));
+
+    add(new CheckForTCPEstablished(
+            "check for TCPService objects with flag \"established\""));
+            
+    add(new eliminateDuplicatesInSRC("eliminate duplicates in SRC"));
+    add(new eliminateDuplicatesInDST("eliminate duplicates in DST"));
+    add(new eliminateDuplicatesInSRV("eliminate duplicates in SRV"));
+
+    add(new swapAddressTableObjectsInSrc(
+            "AddressTable -> MultiAddressRunTime in Src"));
+    add(new swapAddressTableObjectsInDst(
+            "AddressTable -> MultiAddressRunTime in Dst"));
+
+    add(new swapMultiAddressObjectsInSrc(
+            "MultiAddress -> MultiAddressRunTime in Src"));
+    add(new swapMultiAddressObjectsInDst(
+            "MultiAddress -> MultiAddressRunTime in Dst"));
+
+    add(new processMultiAddressObjectsInSrc(
+            "process MultiAddress objects in Src"));
+    add(new processMultiAddressObjectsInDst(
+            "process MultiAddress objects in Dst"));
+
+    add(new replaceFailoverInterfaceInItf("replace carp interfaces"));
+
+
+    add(new expandGroupsInItf("expand groups in Interface"));
+    add(new replaceClusterInterfaceInItf(
+            "replace cluster interfaces with member interfaces in "
+            "the Interface rule element"));
+    add(new ItfNegation("process negation in Itf"));
+
+    //add(new InterfacePolicyRules(
+    //    "process interface policy rules and store interface ids"));
+
+    add(new splitIfFirewallInSrc("split rule if firewall is in Src"));
+    add(new splitIfFirewallInDst("split rule if firewall is in Dst"));
+    add(new fillDirection("determine directions"));
+
+// commented out for bug #2828602
+// ... and put back per #2844561
+// both bug reports/patches are by Tom Judge (tomjudge on sourceforge)
+    add( new SplitDirection("split rules with direction 'both'"  ));
+
+    add(new addLoopbackForRedirect(
+            "add loopback to rules that permit redirected services"));
+    add(new ExpandMultipleAddresses(
+            "expand objects with multiple addresses"));
+    add(new dropRuleWithEmptyRE("drop rules with empty rule elements"));
+    add(new checkForDynamicInterfacesOfOtherObjects(
+            "check for dynamic interfaces of other hosts and firewalls"));
+    add(new MACFiltering("verify for MAC address filtering"));
+    add(new checkForUnnumbered("check for unnumbered interfaces"));
+    add(new addressRanges("expand address range objects"));
+    add(new groupServicesByProtocol("split rules with different protocols"));
+    add(new separateTCPWithFlags("separate TCP services with flags"));
+    add(new separateSrcPort("split on TCP and UDP with source ports"));
+    add(new separateTagged("split on TagService"));
+    add(new separateTOS("split on IPService with TOS"));
+
+    if (ipv6)
+        add( new DropIPv4Rules("drop ipv4 rules"));
+    else
+        add( new DropIPv6Rules("drop ipv6 rules"));
+
+    add(new verifyCustomServices("verify custom services for this platform"));
+//	add(new ProcessScrubOption("process 'scrub' option"));
+    add(new SpecialServices("check for special services"));
+    add(new setQuickFlag("set 'quick' flag"));
+    add(new checkForZeroAddr("check for zero addresses"));
+
+    add( new checkForObjectsWithErrors(
+             "check if we have objects with errors in rule elements"));
+
+    add(new createTables("create tables"));
+//        add(new PrintTables("print tables"));
+
+    add(new PrintRule("generate pf code"));
+    add(new simplePrintProgress());
+
+    runRuleProcessors();
+
 }
 
 
@@ -1163,80 +1005,23 @@ void PolicyCompiler_pf::epilog()
 {
 }
 
-void PolicyCompiler_pf::insertCarpRule()
+/**
+ * virtual method to let policy compiler check rules using
+ * options specific for the given fw platform. Base class
+ * PolicyCompiler has no visibility into platform-specific
+ * options and can not do this.
+ */
+bool PolicyCompiler_pf::checkForShadowingPlatformSpecific(PolicyRule *,
+                                                          PolicyRule *r2)
 {
-    /* Add CARP-Service to database */
-    IPService* carp_service = IPService::cast(dbcopy->create(IPService::TYPENAME));
-    carp_service->setComment("CARP service");
-    carp_service->setProtocolNumber(112);
-    dbcopy->add(carp_service);
+    bool quick = r2->getBool("quick");
+    // if quick == false, the rule is non-terminating
+    if (!quick) return false;
 
-    FWObjectTypedChildIterator interfaces = fw->findByType(Interface::TYPENAME);
-    for (; interfaces != interfaces.end(); ++interfaces) 
-    {
-        Interface *iface = Interface::cast(*interfaces);
-
-        if (iface->isFailoverInterface())
-        {
-            FWObject *failover_group =
-                iface->getFirstByType(FailoverClusterGroup::TYPENAME);
-            if (failover_group->getStr("type") == "carp")
-            {
-                /* Add automatic rules for CARP 
-                 * Rule should be associated with physical interface
-                 */
-                
-                string phys_iface_name = iface->getOptionsObject()->getStr("base_device");
-                Interface *phys_iface = Interface::cast(
-                    fw->findObjectByName(Interface::TYPENAME, phys_iface_name));
-                if (phys_iface)
-                {
-                    PolicyRule *rule = 
-                        addMgmtRule(NULL, NULL, carp_service, phys_iface,
-                                    PolicyRule::Both, PolicyRule::Accept, "CARP");
-                    FWOptions *ruleopt = rule->getOptionsObject();
-                    assert(ruleopt!=NULL);
-                    ruleopt->setBool("firewall_is_part_of_any_and_networks", false);
-                } else
-                {
-                    warning(
-                        "Can not find interface " + phys_iface_name + 
-                        " for the CARP interface " + iface->getName() +
-                        " of the cluster");
-                }
-            }
-        }
-    }
+    return true;
 }
 
-void PolicyCompiler_pf::insertPfsyncRule()
+PolicyCompiler_pf::~PolicyCompiler_pf()
 {
-    /* Add pfsync service to database */
-    IPService* pfsync_service = IPService::cast(dbcopy->create(IPService::TYPENAME));
-    pfsync_service->setComment("pfsync service");
-    pfsync_service->setProtocolNumber(240);
-    dbcopy->add(pfsync_service);
-
-    FWObjectTypedChildIterator interfaces = fw->findByType(Interface::TYPENAME);
-    for (; interfaces != interfaces.end(); ++interfaces) 
-    {
-        Interface *iface = Interface::cast(*interfaces);
-        if (iface->getOptionsObject()->getBool("state_sync_group_member"))
-        {
-            FWObject *state_sync_group = dbcopy->findInIndex(
-                dbcopy->getIntId(
-                    iface->getOptionsObject()->getStr("state_sync_group_id")));
-            assert(state_sync_group!=NULL);
-            if (state_sync_group && state_sync_group->getStr("type") == "pfsync")
-            {
-                PolicyRule *rule = addMgmtRule(NULL, NULL, pfsync_service, iface,
-                                               PolicyRule::Both,
-                                               PolicyRule::Accept, "pfsync");
-                FWOptions *ruleopt = rule->getOptionsObject();
-                assert(ruleopt!=NULL);
-                ruleopt->setBool("firewall_is_part_of_any_and_networks", false);
-            }
-        }
-    }
+    // if (tables) tables->detach();
 }
-
