@@ -70,8 +70,8 @@ using namespace std;
  */
 void PolicyCompiler_pf::PrintRule::_printAction(PolicyRule *rule)
 {
-    FWOptions *ruleopt =rule->getOptionsObject();
-    Service *srv=compiler->getFirstSrv(rule);    assert(srv);
+    FWOptions *ruleopt = rule->getOptionsObject();
+    Service *srv = compiler->getFirstSrv(rule);    assert(srv);
     string version = compiler->fw->getStr("version");
 
     switch (rule->getAction())
@@ -152,8 +152,7 @@ void PolicyCompiler_pf::PrintRule::_printAction(PolicyRule *rule)
             compiler->output << "anchor \"UNDEFINED\" ";
         }else
         {
-            string ruleset_name = ruleset->getName();
-            compiler->output << "anchor \"" << ruleset_name << "\" ";
+            compiler->output << "anchor \"" << ruleset->getName() << "\" ";
         }
         break;
     }
@@ -338,7 +337,7 @@ void PolicyCompiler_pf::PrintRule::_printQueue(PolicyRule *rule)
     FWOptions *ruleopt =rule->getOptionsObject();
 
     if (rule->getAction() == PolicyRule::Classify)
-        compiler->output << "queue " << ruleopt->getStr("classify_str") << " ";
+        compiler->output << "queue " << ruleopt->getStr("pf_classify_str") << " ";
 }
 
 void PolicyCompiler_pf::PrintRule::_printUser(PolicyRule *rule)
@@ -447,8 +446,7 @@ string PolicyCompiler_pf::PrintRule::_printLogPrefix(PolicyRule *rule,
         {
             for (FWObject::iterator it=intf_re->begin(); it!=intf_re->end(); ++it)
             {
-                FWObject *o   = *it;
-                if (FWReference::cast(o)!=NULL) o = FWReference::cast(o)->getPointer();
+                FWObject *o = FWObjectReference::getObject(*it);
                 rule_interfaces += " " + o->getName();
             }
         }
@@ -456,14 +454,6 @@ string PolicyCompiler_pf::PrintRule::_printLogPrefix(PolicyRule *rule,
             s.replace(n, 2, rule_interfaces);
         else
             s.replace(n, 2, "global");
-
-        // string rule_iface =  rule->getInterfaceStr();
-        // if (rule_iface!="") 
-        // {
-        //     s1 << rule_iface;
-        //     s.replace(n,2,s1.str());
-        // } else
-        //     s.replace(n,2,"global");
     }
     if (rule && (n=s.find("%C"))!=string::npos )
     {
@@ -476,28 +466,24 @@ string PolicyCompiler_pf::PrintRule::_printLogPrefix(PolicyRule *rule,
 void PolicyCompiler_pf::PrintRule::_printInterface(PolicyRule *rule)
 {
     RuleElementItf *intf_re = rule->getItf();
-    string rule_interfaces;
-    int intf_count = 0;
+    QStringList rule_interfaces;
 
     if (!intf_re->isAny())
     {
         for (FWObject::iterator it=intf_re->begin(); it!=intf_re->end(); ++it)
         {
-            FWObject *o   = *it;
-            if (FWReference::cast(o)!=NULL) o = FWReference::cast(o)->getPointer();
-            rule_interfaces += " " + o->getName();
-            intf_count++;
+            FWObject *o = FWObjectReference::getObject(*it);
+            rule_interfaces << o->getName().c_str();
         }
-        compiler->output << "on";
-        if (intf_count > 1) compiler->output << " {";
-        compiler->output << rule_interfaces;
-        if (intf_count > 1) compiler->output << " }";
-        compiler->output << " ";
+        if (rule_interfaces.size() > 1)
+        {
+            rule_interfaces.push_front("{");
+            rule_interfaces.push_back("}");
+        }
+        compiler->output << "on "
+                         << rule_interfaces.join(" ").toStdString()
+                         << " ";
     }
-
-    // string iface_name = rule->getInterfaceStr();
-    // if (iface_name!="") 
-    //     compiler->output << "on " << iface_name << " ";
 }
 
 // print address family
@@ -931,17 +917,14 @@ PolicyCompiler_pf::PrintRule::PrintRule(const std::string &name) : PolicyRulePro
 
 bool PolicyCompiler_pf::PrintRule::processNext()
 {
-    PolicyRule *rule=getNext(); if (rule==NULL) return false;
-    FWOptions  *ruleopt =rule->getOptionsObject();
-    string version=compiler->fw->getStr("version");
+    PolicyRule *rule = getNext(); if (rule==NULL) return false;
+    FWOptions  *ruleopt = rule->getOptionsObject();
+    string version = compiler->fw->getStr("version");
 
     tmp_queue.push_back(rule);
 
     compiler->output << compiler->printComment(rule, current_rule_label, "#");
 
-    // string err = rule->getStr(".error_msg");
-    // if (!err.empty()) compiler->output << "# " << err << endl;
-
     RuleElementSrc *srcrel=rule->getSrc();
 //    Address        *src   =compiler->getFirstSrc(rule);  assert(src);
     RuleElementDst *dstrel=rule->getDst();
@@ -980,9 +963,10 @@ bool PolicyCompiler_pf::PrintRule::processNext()
  * Dealing with "keep state" and "modulate state" flags
  *
  * 1. both flags do not apply to deny/reject rules.
- * 2. modulate state applies only to TCP services. Since we use splitServices,
- *    all services in a rule are of the same protocol, therefore we can simply
- *    check type of srv
+ *
+ * 2. modulate state applies only to TCP services. Since we use
+ *    groupServicesByProtocol, all services in a rule are of the same
+ *    protocol, therefore we can simply check type of srv
  */
     if ( ! ruleopt->getBool("stateless") )
     {
