@@ -27,6 +27,7 @@
 
 #include "fwbuilder/Firewall.h"
 #include "fwbuilder/Interface.h"
+#include "fwbuilder/Constants.h"
 
 #include <QStringList>
 #include <QDebug>
@@ -37,8 +38,6 @@ using namespace libfwbuilder;
 using namespace CppUnit;
 
 
-extern string librespath;
-
 class UpgradePredicate: public XMLTools::UpgradePredicate
 {
     public:
@@ -369,7 +368,6 @@ void interfacePropertiesTest::isEligibleForCluster()
     Interface *parent1 = Interface::cast(db->create(Interface::TYPENAME));
     Interface *iface1 = Interface::cast(db->create(Interface::TYPENAME));
     db->add(parent1);
-    db->add(iface1);
     parent1->add(iface1);
 
     iface1->getOptionsObject()->setStr("type", "ethernet");
@@ -403,7 +401,6 @@ void interfacePropertiesTest::isEligibleForCluster()
     Interface *subface = Interface::cast(db->create(Interface::TYPENAME));
     subface->setName("iface");
     fw->add(iface);
-    fw->add(subface);
     iface->getOptionsObject()->setStr("type", "bonding");
     iface->add(subface);
     CPPUNIT_ASSERT ( interfaceProperties().isEligibleForCluster(subface) == false );
@@ -586,7 +583,7 @@ void interfacePropertiesTest::testManageIpAddresses()
 
     string file_name = "test.fwb";
     db->setReadOnly( false );
-    db->load(file_name, &upgrade_predicate, librespath);
+    db->load(file_name, &upgrade_predicate, Constants::getDTDDirectory());
     db->setFileName(file_name);
     db->reIndex();
 
@@ -656,7 +653,7 @@ void interfacePropertiesTest::testManageIpAddressesCluster()
 
     string file_name = "test.fwb";
     db->setReadOnly( false );
-    db->load(file_name, &upgrade_predicate, librespath);
+    db->load(file_name, &upgrade_predicate, Constants::getDTDDirectory());
     db->setFileName(file_name);
     db->reIndex();
 
